# -*- coding: utf-8 -*-

#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import os
import sys
import shutil

from emesenecommon import *

DEFAULT_GLOBAL_CONFIG = {
    'rememberMe': '1',
    'rememberMyPassword': '0',
    'debug': '0',
    'nodns': '0',
    'useProxy': '0',
    'proxyHost': 'http://127.0.0.1',
    'proxyPort': '80',
    'mainWindowWidth': '260',
    'mainWindowHeight': '440',
    'mainWindowX': '-1',
    'mainWindowY': '-1',
    'autoLogin': '0',
    'lastLoggedAccount': '',
}

DEFAULT_USER_CONFIG = {
    'theme': 'default',
    'smilieTheme': 'default',
    'conversationLayout' : 'default',
    'smallIcons': '0',
    'showByNick': '1',
    'showOffline': '0',
    'showEmptyGroups': '0',
    'showByStatus': '0',
    'orderByStatus': '0',
    'autoReply': '0',
    'autoReplyMessage': 'Im not here (this is an autoreply)',
    'rememberMyPassword': '0',
    'fontFace': 'Sans',
    'fontColor': '#000000',
    'fontSize': '10',
    'fontUnderline' : '0',
    'fontBold' : '0',
    'fontItalic' : '0',
    'fontStrike' : '0',
    'showTimestamp': '0',
    'convWindowWidth': '450',
    'convWindowHeight': '480',
    'saveLogsAutomatically' : '0',
    'logPath' : '',
    'logFormats': '1',  # binary: 1 & text, 2 & html
    'activePlugins': 'PersonalMessage', # CSV of plugin names
    'avatarDir' : '',
    'avatarPath' : '',
    'parsePlusMarkup' : '1',
    'parseSmilies' : '1',
    'typingColor': '#339999',
    'messageWaitingColor': '#ef2929',
    'personalMessageColor': '#AAAAAA',
    'nickColor': '#000000',
    'windows' : '0',
    'avatarDual'  : '1',
    
    'showUserPanel' : '1',
    'showSearchEntry' : '1',
    'showStatusCombo' : '1',
    'showHeader' : '1',
    'showToolbar' : '1',
    'showMyAvatar' : '1',
    
    'avatarsInUserList' : '1',
    'mediaEnabled' : '0',
}

class Config:
    '''This class contains all the stuff relative to the configuration of emesene.
    ou can get or set configuration values and will be saved to a file for persistence'''

    # -- GLOBAL CONFIG -- #
    def __init__(self):
        '''
        Constructor : this will create needed files for global configuration
        '''
        
        self.currentUser = ''
        self.globalConfigDict = {}
        self.userConfigDict = {}
        
        # This dict is a bit different from the two others
        # It's not a key : value dict but a plugin_name : [ key : value ] dict
        self.pluginsConfigDict = {}
        
        if os.name == "posix":
            if not os.path.isdir(HOME_DIR + "/.config"):
                os.mkdir(HOME_DIR + "/.config")
            
            if not os.path.isdir(CONFIG_DIR):
                os.mkdir(CONFIG_DIR)
                debugmsg('Config dir created: ' + CONFIG_DIR)
                                
            # Create files
            if not os.path.isdir(THEME_HOME_PATH):
                os.mkdir(THEME_HOME_PATH) 
                debugmsg('Theme dir in config created: ' + THEME_HOME_PATH)
            
            if not os.path.isdir(SMILIES_HOME_PATH):
                os.mkdir(SMILIES_HOME_PATH)
                debugmsg('Smilies dir in config created: ' + SMILIES_HOME_PATH)
            
            if not os.path.isdir(CONVTHEMES_HOME_PATH):
                os.mkdir(CONVTHEMES_HOME_PATH)
                debugmsg('Conversation themes dir in config created: ' + CONVTHEMES_HOME_PATH)
            
            if not os.path.isdir(THEME_HOME_PATH):
                os.mkdir(THEME_HOME_PATH) 
                debugmsg('Theme dir in config created: ' + THEME_HOME_PATH)
            
            if not os.path.isdir(PLUGIN_HOME_PATH):
                os.mkdir(PLUGIN_HOME_PATH)
                shutil.copyfile(PLUGIN_SYSTEM_WIDE_PATH + '/__init__.py', \
                                 PLUGIN_HOME_PATH + '/__init__.py')
                debugmsg('Plugins dir in config created: ' + PLUGIN_HOME_PATH)
            
        if not os.path.isfile(CONFIG_DIR + os.sep + "config"):
            self.globalConfigDict = DEFAULT_GLOBAL_CONFIG.copy()
            self.writeGlobalConfig()
            debugmsg('Config file created: ' + CONFIG_DIR + os.sep + "config")
            
        if not os.path.isdir(CONFIG_DIR + os.sep + "conversation_themes"):
            os.mkdir(CONFIG_DIR + os.sep + 'conversation_themes')
        
        self.readGlobalConfig()
        
    def readGlobalConfig(self):
        '''
        read the config file and create a dictionarie with key and value
        of all the key=value\n in the config file
        '''
        
        self.globalConfigDict = DEFAULT_GLOBAL_CONFIG.copy()
        conf = None
        try:
            conf = open(CONFIG_DIR + "/config", "r")
            string = conf.read()
            
            for i in string.splitlines():
                if i != "":
                    try:
                        delim = i.find("=")
                        key = i[:delim]
                        value = i[delim+1:]
                        if key in DEFAULT_GLOBAL_CONFIG:
                            self.globalConfigDict[ key ] = value
                        else:
                            debugmsg(key + ' is not a valid config key, ignored')
                    except Exception, e:
                        debugmsg(key + ' config value is incorrect')
                                    
            conf.close()
        except:
            if conf:
                conf.close()
            
    def writeGlobalConfig(self):
        '''
        write the config to the file, overwrite current config file
        '''
        
        try:
            conf = open(CONFIG_DIR + "/config", "w")
        
            for k, v in self.globalConfigDict.iteritems():
                conf.write(k + "=" + v + "\n")

            conf.close()
        except Exception, e:
            errormsg('exception writing config:\n')
            errormsg(e)
                
    def getGlobalDefaultValue(self, key):
        '''
        get the value from key in DEFAULT_GLOBAL_CONFIG
        '''
        
        if key in DEFAULT_GLOBAL_CONFIG:
            return DEFAULT_GLOBAL_CONFIG[ key ]
        else:            
            debugmsg('Default config value was not found: ' + key)
            return ''
            
    def getGlobalValue(self, key, default = ''):
        '''
        get the value from key in globalConfigDict
        '''
        
        if key in self.globalConfigDict:
            return self.globalConfigDict[ key ]
        else:            
            debugmsg('Config value was not found: ' + key)
            return default
    
    def setGlobalValue(self, key, value):
        '''
        set a value in globalConfigDict, to make it last to the next session
        you have to call writeGlobalConfig before you close
        '''
        if key in DEFAULT_GLOBAL_CONFIG:
            self.globalConfigDict[ key ] = value
            self.writeGlobalConfig()
        else:
            debugmsg(key + ' is not a valid config key, ignored')       
        
    # -- USER CONFIG -- #
    def setCurrentUser(self, email):
        ''' 
        Create and/or read needed file for user config
        /!\ This function MUST be called before any set/getUserConfig
        '''
        self.currentUser = email
        self.setGlobalValue("lastLoggedAccount", email);
        
        if email != '':
            self.currentUser = self.currentUser.replace('@', '_')
            self.currentUser = self.currentUser.replace('.', '_')
           
            if not os.path.isdir(CONFIG_DIR + os.sep + self.currentUser):
                os.mkdir(CONFIG_DIR + os.sep + self.currentUser)
                debugmsg('User config dir created: ' + CONFIG_DIR + os.sep + self.currentUser)

            if not os.path.isfile(CONFIG_DIR + os.sep + self.currentUser + os.sep + "config"):
                    self.userConfigDict = DEFAULT_USER_CONFIG.copy()
                    self.writeUserConfig()
                    debugmsg('Config file created: ' + CONFIG_DIR + os.sep + self.currentUser + os.sep + "config")

            if not os.path.isdir(CONFIG_DIR + os.sep + self.currentUser + os.sep + "logs"):
                os.mkdir(CONFIG_DIR + os.sep + self.currentUser + os.sep + "logs")
            
            if not os.path.isdir(CONFIG_DIR + os.sep + self.currentUser + os.sep + "cache"):
                    os.mkdir(CONFIG_DIR + os.sep + self.currentUser + os.sep + 'cache')
                    
            if not os.path.isdir(CONFIG_DIR + os.sep + self.currentUser + os.sep + "avatars"):
                    os.mkdir(CONFIG_DIR + os.sep + self.currentUser + os.sep + 'avatars')
                    
            if not os.path.isdir(CONFIG_DIR + os.sep + self.currentUser + os.sep + "custom_emoticons"):
                    os.mkdir(CONFIG_DIR + os.sep + self.currentUser + os.sep + 'custom_emoticons')
                                
            self.readUserConfig()

        else:
            self.userConfigDict = {}
      
    def getCurrentUser(self):
        return self.currentUser
        
    def readUserConfig(self):
        '''
        read the config file and create a dictionarie with key and value
        of all the key=value\n in the config file
        '''
        
        self.userConfigDict = DEFAULT_USER_CONFIG.copy()
        conf = None
        try:
            conf = open(CONFIG_DIR + "/" + self.currentUser + "/config", "r")
            string = conf.read()
                        
            for i in string.splitlines():
                if i != "":
                    try:
                        delim = i.find("=")
                        key = i[:delim]
                        value = i[delim+1:]
                        if key in DEFAULT_USER_CONFIG:
                            self.userConfigDict[ key ] = value
                        else:
                            debugmsg(key + ' is not a valid config key, ignored')
                    except Exception, e:
                        debugmsg(key + ' config value is incorrect')
                    
            conf.close()
        except:
            if conf:
                conf.close()
            
    def writeUserConfig(self):
        '''
        write the config to the file, overwrite current config file
        '''
        
        try:
            conf = open(CONFIG_DIR + "/" + self.currentUser + "/config", "w")
        
            for k, v in self.userConfigDict.iteritems():
                conf.write(k + "=" + v + "\n")

            conf.close()
        except Exception, e:
            errormsg('exception writing config:\n')
            errormsg(e)
        
    def getUserValue(self, key, default = ''):
        '''
        get the value from key in userConfigDict
        '''
        
        if key in self.userConfigDict:
            return self.userConfigDict[ key ]
        else:
            debugmsg('Config value was not found: ' + key)
            return default
            
    def getUserDefaultValue(self, key):
        '''
        get the value from key in DEFAULT_USER_CONFIG
        '''
        
        if key in DEFAULT_USER_CONFIG:
            return DEFAULT_USER_CONFIG[ key ]
        else:
            debugmsg('Default config value was not found: ' + key)
            return ''
               
    def setUserValue(self, key, value):
        '''
        set a value in userConfigDict, to make it last to the next session
        you have to call writeConfig before you close
        '''
        if key in DEFAULT_USER_CONFIG:
            self.userConfigDict[ key ] = value
            self.writeUserConfig()
        else:
            debugmsg(key + ' is not a valid config key, ignored')
            
    # --- PLUGINS CONFIG --- work in progress - #
    def readPluginConfig(self, pluginName):
        '''
        read the plugin config file and create a dictionarie with key and value
        of all the key=value\n in the config file
        '''
        
        self.pluginsConfigDict[pluginName] = {}
        conf = None
        try:
            conf = open(self.getUserConfigPath() + os.sep + pluginName + '.conf', "r")
            string = conf.read()
                        
            for i in string.splitlines():
                if i != "":
                    try:
                        delim = i.find("=")
                        key = i[:delim]
                        value = i[delim+1:]
                        self.pluginsConfigDict[pluginName][key] = value
                    except Exception, e:
                        debugmsg(key + ' config value is incorrect')
                    
            conf.close()
        except:
            if conf:
                conf.close()
                
    def writePluginConfig(self, pluginName):
        '''
        write the config to the file, overwrite current plugin config file
        '''

        try:
            conf = open(self.getUserConfigPath() + os.sep + pluginName + '.conf', "w")
        
            for k, v in self.pluginsConfigDict[pluginName].iteritems():
                conf.write(k + "=" + v + "\n")

            conf.close()
        except Exception, e:
            errormsg('exception writing config:\n')
            errormsg(e)
    
    def setPluginValue(self, pluginName, key, value):
        self.pluginsConfigDict[pluginName][key] = value
        self.writePluginConfig(pluginName)
        
    def getPluginValue(self, pluginName, key, default):
        if pluginName in self.pluginsConfigDict:
            if key in self.pluginsConfigDict[pluginName]:
                return self.pluginsConfigDict[pluginName][key]
       
        return default
       
                    
    # --- OTHER STUFF --- #
    def _explorePath( self, path ):
        result = []
        for root, dirs, files in os.walk(path, True):
            if root != path: continue
            for dir in dirs:
                if not dir.startswith('.') and dir not in result:
                    result.append(dir)
            # stops walking the tree
            dirs = []
        return result
        
    def getThemes(self):
        '''
        this method is here because i dont know where to put it,
        it return a list of dirs in the theme directory
        '''
        
        return self._explorePath(THEME_HOME_PATH) + \
               self._explorePath(THEME_SYSTEM_WIDE_PATH)
        
    def getSmilieThemes(self):
        '''
        the same with smilie themes
        '''
        
        return self._explorePath(SMILIES_HOME_PATH) + \
               self._explorePath(SMILIES_SYSTEM_WIDE_PATH)
        
    def getLogPath(self):
        logPath = self.getUserValue('logPath')
        if logPath != '':
            return logPath
        else:
            return CONFIG_DIR + os.sep + self.currentUser + os.sep + "logs"
        
    def getCachePath(self):
        return CONFIG_DIR + os.sep + self.currentUser + os.sep + "cache"
        
    def getUserConfigPath(self):
        return CONFIG_DIR + os.sep + self.currentUser
    
    def getAvatarsCachePath(self):
        return CONFIG_DIR + os.sep + self.currentUser + os.sep + 'avatars'
        
    def getCustomEmoticonsPath(self):
        return CONFIG_DIR + os.sep + self.currentUser + os.sep + 'custom_emoticons'
        
    def getConversationThemesPath(self):
        return CONFIG_DIR + os.sep + 'conversation_themes'
