# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import os
import gtk
import sys
import time
import gobject
import gettext
import thread
import traceback
import getopt

import Hotmail
import Theme
import ConversationLayoutManager
import desktop
from emesenelib import core
from emesenelib import Socket
import Avatar
import Config
import Dialogs
import MainWindow
import PluginManager
import PreferenceWindow
import PluginManagerDialog
import ConversationManager
import StatusMenu
from AvatarChooser import AvatarChooser

from Parser import UnifiedParser

from emesenecommon import *

import TrayIcon
import SlashCommands

from CustomEmoticons import CustomEmoticons

gettext.install('emesene')
gettext.install('emesene', LANG_PATH, unicode=False)
lang = {}

SERVER_HOST = "messenger.hotmail.com"
SERVER_IP = "65.54.239.80"

class Controller( gobject.GObject ):
    '''The Controller class concentrate all the logic of the program
    leaving the other classes to only implement the GUI.
    All the classes with GUI should receive a Controller instance to
    be able to do things.'''

    __gsignals__ = {
        #'font-changed', font, bold, italic, pangoDesc.get_size() / pango.SCALE
        'font-changed' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, 
        (gobject.TYPE_STRING,gobject.TYPE_BOOLEAN,gobject.TYPE_BOOLEAN,gobject.TYPE_INT)),
        #'color-changed', '#%02X%02X%02X' % (red, green, blue)
        'color-changed' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, 
        (gobject.TYPE_STRING,)),
        'input-format-changed' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, 
        (gobject.TYPE_PYOBJECT,)),
        'avatar-changed' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, 
        ()),
        'preferences-changed' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, 
        ()),

    }

    def __init__( self, username, minimized ):
        '''Contructor'''

        gobject.GObject.__init__(self)

        self.status_ordered = [
            [ 'NLN', 'AWY', 'BSY', 'BRB', 'PHN',
             'LUN', 'HDN', 'IDL', 'FLN' ],

            [ 'online', 'away', 'busy', 'brb', 'phone',
             'lunch', 'invisible', 'idle', 'offline' ],

            [ _( 'Online' ), _( 'Away' ), _( 'Busy' ), _( 'Be right back' ),
             _( 'On the phone' ), _( 'Gone to lunch' ), _( 'Invisible' ),
             _( 'Idle' ), _( 'Offline' ) ],
            
            [ _( '_Online' ), _( '_Away' ), _( '_Busy' ), _( 'Be _right back' ),
             _( 'On the _phone' ), _( 'Gone to _lunch' ), _( '_Invisible' ),
             _( 'I_dle' ), _( 'O_ffline' ) ]
        ]

        self.connected = False
        self.userEmail = None
        self.userStatus = None
        self.config = Config.Config()
        self.theme = Theme.Theme( self.config, 'default', 'default' )
        self.mainWindow = MainWindow.MainWindow( self )
        
        
        theme = self.theme
        
        self.unifiedParser = UnifiedParser( theme )
        
        gtk.window_set_default_icon_list( theme.getImage( 'icon16' ),
                                          theme.getImage( 'icon32' ),
                                          theme.getImage( 'icon48' ),
                                          theme.getImage( 'icon96' ) )

        self.conversationLayoutManager = ConversationLayoutManager.ConversationLayoutManager(self)
                
        if not TrayIcon.disabled:
            self.trayIcon = TrayIcon.TrayIcon( self )

        self.autoReplyMessage = ''
        self.conversationManager = None
        self.msn = None
        self.lastILN = 0
        
        self.pluginManager = None
        
        self.trayDisconnect = None
        self.traySeparator = None
        self.trayStatusMenu = None
        
        self.avatar = None
        self.cancel = False
        self.mediaEnabled = bool( int( self.config.getUserValue( 'mediaEnabled', '0' ) ) )

        if TrayIcon.disabled or not minimized:
            self.mainWindow.show()

        if username in self.mainWindow.login.users.getUsers():
            user = username
            self.mainWindow.login.getPreferences(user)
            pwd = self.mainWindow.login.getPass()
            status = self.mainWindow.login.getStatus()
        elif self.config.getGlobalValue( "autoLogin" ) == '1':
            user = self.mainWindow.login.getUser()
            pwd = self.mainWindow.login.getPass()
            status = self.mainWindow.login.getStatus()
        else:
            user = ''
            pwd = ''

        if user != '' and pwd != '':
            self.login( user , pwd , status )
         
    def getContactManager(self):
        return (None, self.msn.contactManager)[self.msn != None]

    def login( self, user, password, status ):
        '''do the login'''
        global loginStatus
        self.cancel = False
        self.userEmail = user
        self.userStatus = status
        self.mainWindow.buildInterface( 'loading' )
        self.config.setCurrentUser(user)
        
        debug = self.config.getGlobalValue( 'debug' ) == '1'
        if self.config.getGlobalValue( 'nodns' ) == '1':
            server = SERVER_IP
        else:
            server = SERVER_HOST

        proxy = None

        if self.config.getGlobalValue( 'useProxy' ) == '1':
            proxy = Socket.Proxy( self.config.getGlobalValue( 'proxyHost' ), \
                int( self.config.getGlobalValue( 'proxyPort' ) ) )

        # messenger.hotmail.com
        self.msn = core.Msnp( server, 1863, user, password,\
             self.config.getUserConfigPath(), debug, proxy )
        
        if self.cancel == True:
            self.config.setCurrentUser('')
            del self.msn
            return
        
        thread.start_new_thread( login , ( self.msn , ) )

        while loginStatus == 0:
            time.sleep( 0.01 )
            if self.cancel == True:
                self.config.setCurrentUser('')
                return
            gtk.main_iteration( True )

        if loginStatus == -1:
            text = _( "Error during login, please retry" )
            if loginError:
                text = text + "\n(" + str(loginError) + ")"
            Dialogs.Error( self.mainWindow, text ).run()
            
            self.mainWindow.buildInterface( 'login' )
            loginStatus = 0
            self.config.setCurrentUser('')
            del self.msn
            return

        if self.cancel == True:
            self.config.setCurrentUser('')
            del self.msn
            return

        self.connected = True
        self.autoReplyMessage = self.config.getUserValue("autoReplyMessage")
        self.themeChanged()
        self.msn.changeStatus( status )
        self.msn.connect( 'disconnected', self.disconnected )
        self.msn.connect( 'connection-closed', self.connectionClosed )
        gobject.timeout_add( 200 , self.process )
        self.mainWindow.buildInterface( 'userlist' )
        self.mainWindow.refreshUserList()
        self.changeAvatar( self.config.getUserValue('avatarPath') )
        
        if not self.conversationLayoutManager.load(self.config.getUserValue('conversationLayout', 'default')):
            self.conversationLayoutManager.setDefault()
        
        # menu items only available when connected
        if not self.trayStatusMenu and not TrayIcon.disabled:
            statusMenuItem = gtk.MenuItem( _( "_Status" ) )
            statusMenu = StatusMenu.StatusMenu( self )
            statusMenuItem.set_submenu( statusMenu )
            separator = gtk.SeparatorMenuItem()
            disconnect = gtk.ImageMenuItem( gtk.STOCK_DISCONNECT )
            disconnect.connect( 'activate', self.on_tray_disconnect )
            
            self.trayDisconnect = disconnect
            self.traySeparator = separator
            self.trayStatusMenu = statusMenuItem
        
        self.conversationManager = ConversationManager.ConversationManager( self )
        
        #Slash Command Handler
        self.Slash = SlashCommands.SlashCommands( self )
        
        # add here
        
        if not TrayIcon.disabled:
            # REMEMBER to remove them when disconnect    
            self.trayIcon.menu.prepend( self.trayDisconnect )
            self.trayIcon.menu.prepend( self.traySeparator )
            self.trayIcon.menu.prepend( self.trayStatusMenu )
            self.trayIcon.menu.show_all()
            self.trayIcon.update( self.msn.status )
        
        self.pluginManager = PluginManager.PluginManager( self )

        plugins = self.config.getUserValue( 'activePlugins' ).split( ',' )

        self.startPlugins( plugins )

        self.msn.connect( 'status-change', self.onStatusChange )
        self.msn.connect( 'initial-status-change', self.onInitialStatusChange )
        self.msn.connect( 'status-online', self.onStatusOnline )
        self.msn.connect( 'status-offline', self.onStatusOffline )
        self.msn.connect( 'new-conversation', self.newConversation )
        self.msn.connect( 'add-notification', self.addNotification )
        #self.msn.connect( 'remove-notification', self.removeNotification )
        self.msn.connect( 'user-list-change', self.refreshUserList )
        self.msn.connect( 'user-disconnected', self.userDisconnected )
        self.msn.connect( 'connection-problem', self.connectionProblem )
        
        self.msn.connect( 'offline-message-waiting', self.offlineMessageWaiting )
        self.msn.connect( 'offline-message-received', self.offlineMessageReceived )
        self.msn.connect( 'send-message-error', self.messageError )
        self.msn.connect( 'msnobj-changed', self.msnobjChanged )
        
        self.hotmail = Hotmail.Hotmail( self.msn, self.config )
        
        self.customEmoticons =  CustomEmoticons( self.msn, self.config )

        self.checkPending()
        self.cancel = False

    def disconnected( self, msnp ):
        '''called when a error occurs and we cant keep connected.'''
        Dialogs.Error( self.mainWindow , _( "Connection error" ) ).run()
        self.logout()
        
    def connectionClosed( self, msnp ):
        '''called whrn a connection error happen'''
        Dialogs.Error( self.mainWindow , _( "Connection error" ) ).run()
        self.logout()
        
    def logout( self ):
        '''do the logout'''
        global loginStatus
        try:
            self.conversationManager.closeAll()
            self.pluginManager.stopAll()
        except:
            pass
        self.mainWindow.buildInterface( 'login' )
        
        self.connected = False
        
        if not TrayIcon.disabled:
            try:
                self.trayIcon.menu.remove( self.trayDisconnect )
                self.trayIcon.menu.remove( self.traySeparator )
                self.trayIcon.menu.remove( self.trayStatusMenu )
                self.trayIcon.update( self.msn.status )
            except:
                pass
        
        try:    
            self.msn.logout()
        except:
            pass
        self.msn = None
        self.conversationManager = None
        self.pluginManager = None
        loginStatus = 0
        
    def cancelLogin(self):
        self.cancel = True
        self.goToLoginWindow()
        
    def goToLoginWindow( self ):
        global loginStatus
        self.mainWindow.buildInterface( 'login' )
        self.msn = None
        self.conversationManager = None
        self.pluginManager = None
        loginStatus = 0
        
    def quit( self, status ):
        '''close the window, and do all the things needed...'''
        
        # makes it more responsive to the user by effectively hiding the
        # the window before disconnecting and quitting
        #self.mainWindow.connect( 'unmap-event', self.on_main_hidden, status )
        self.mainWindow.hide()
        if not TrayIcon.disabled:
            self.trayIcon.tray.hide()
            
        if self.connected:
            self.msn.logout()
            self.config.writeUserConfig()

        self.mainWindow.saveToQuit()
        
        self.config.writeGlobalConfig()
        sys.exit( status )

    def getMediaEnabled( self ):
        '''Get if can set media'''
        return self.mediaEnabled
        
    def setMediaEnabled( self, enabled=True):
        '''Set if can set media'''
        self.mediaEnabled = enabled
        
        if enabled:
            self.config.setUserValue('mediaEnabled', '1' )  
        else:
            self.config.setUserValue('mediaEnabled', '0' )  
            
        if not enabled:
            self.msn.changeCurrentMedia('')

    def process( self ):
        '''call all the process methods to process
        the incoming data.'''

        if self.connected:
            self.msn.process()
            # refresh user list when login is complete
            if self.lastILN != 0 and time.time() - self.lastILN > 1:
                self.refreshUserList()
                self.lastILN = 0

        else:
            return False

        return True

    def checkPending( self ):
        '''Check for users pending to be added'''
        users = self.msn.checkPending()
        if len( users ) > 0:
            for i in users:
                self.userAddedYouDialog(i, i)

    def startPlugins( self, plugins ):
        '''Start the plugins in pluginList'''

        if plugins[ 0 ] != '':
            for i in plugins:
                try:
                    (success, message) = self.pluginManager.checkPlugin( i )
                except:
                    sucess = False
                    message = _( 'invalid check() return value' )
                if not success:
                    debugmsg( _( 'plugin %s could not be initialized, reason:' )%i, 3 )
                    debugmsg( message, 3 )
                else:
                    self.pluginManager.startPlugin( i )
                    debugmsg( _( 'plugin %s started successfully' )%i, 3 )

    def changeAvatarDialog( self ):
        ''' Shows a dialog to change personal avatar '''
        response, filename = AvatarChooser(parent = self.mainWindow, config = self.config).run()
        if response == gtk.RESPONSE_OK:
            self.changeAvatar(filename)
        elif response == RESPONSE_NO_AVATAR:
            self.changeAvatar(False)

    def changeAvatar( self, filename ):
        if not filename or not os.path.exists( filename ):
            filename = ''

        self.avatar = Avatar.Avatar( filename, self.config.getAvatarsCachePath() )
        if filename == '':
            self.msn.setDisplayPicture( '' )
            self.mainWindow.setAvatar( gtk.gdk.pixbuf_new_from_file( DEFAULT_THEME_PATH + 'userPanel.png' ) )
            self.config.setUserValue('avatarPath', '' )          
        else:
            self.msn.setDisplayPicture( self.avatar.getImagePath() )
            self.mainWindow.setAvatar( self.avatar.getThumb() )
            self.config.setUserValue('avatarPath', self.avatar.getImagePath() )
        
        self.emit('avatar-changed')

    def changeNickDialog( self, user = 'Me', errorOnEmpty=True ):
        '''show a dialog requesting the nick to be changed
        and change it.'''

        if not self.connected:
            return

        if user == 'Me':
            response = Dialogs.ChangeNick( self.mainWindow , self.msn.nick ).run()
        else:
            alias = self.msn.contactManager.getContactAlias( user )
            deleteBut = True
            if not alias:
                alias = ''
                deleteBut = False
            response = Dialogs.ChangeNick(
                                  self.mainWindow , alias, deleteBut ).run()

        if response == None:
            return
        elif errorOnEmpty and response == "":
            Dialogs.Error( self.mainWindow , _( "Empty Nickname" ) ).run()
            return

        self.changeNick( response, user )

    def changeNick( self , nick, user = 'Me' ):
        '''chang the nick in the session'''
        self.msn.changeNick( nick, user )

    def changePersonalMessage( self, pm ):
        '''change your personal message'''
        self.msn.changePersonalMessage( pm )

    def setStatus( self , status, source='' ):
        '''set the status'''

        if ( self.mainWindow.currentInterface == 'userlist' ):
            self.msn.changeStatus( status )
            if not TrayIcon.disabled:
                self.trayIcon.update(self.msn.status)

    def addUserDialog( self, toGroup=None ):
        '''show a dialog requesting the email of the ser to be added'''
        if not self.connected:
            return

        response = Dialogs.AddFriend( self.mainWindow, self.msn.getGroupNames(), toGroup ).run()

        if response == None:
            return
        elif response[ 0 ] == "":
            Dialogs.Error( self.mainWindow , _( "Empty Field" ) ).run()
            return

        self.addUser( response[ 0 ], response[ 1 ] )
        self.refreshUserList()

    def addUser( self, email, group = '' ):
        '''add an user'''
        self.msn.addUser( email, group )

    def removeUser( self, email = None ):
        '''remove an user, if email is None the email will be the one selected
        on the user list'''

        if email == None:
            email = self.mainWindow.userList.getUserSelected()

        if email == "":
            Dialogs.Error( self.mainWindow , _( "Choose an user to remove" ) ).run()
            return

        if Dialogs.Confirm( self.mainWindow , _( "Are you sure you want to remove %s from your contact list?" )%email, \
                        gtk.STOCK_DELETE ).run() == gtk.RESPONSE_ACCEPT:
            self.msn.removeUser( email )

        self.refreshUserList()

    def blockUser( self, email = None ):
        '''block an user, if email is None the email will be the one selected
        on the user list'''

        if email == None:
            user = self.mainWindow.userList.getUserSelected()

            if user == "":
                Dialogs.Error( self.mainWindow , _( "Choose an user" ) ).run()
                return

            self.msn.blockUser( user )

        else:
            self.msn.blockUser( email )
        
        self.refreshUserList()

    def unblockUser( self, email = None ):
        '''unblock an user, if email is None the email will be the one selected
        on the user list'''

        if email == None:
            user = self.mainWindow.userList.getUserSelected()

            if user == "":
                Dialogs.Error( self.mainWindow , _( "Choose an user" ) ).run()
                return

            self.msn.unblockUser( user )

        else:
            self.msn.unblockUser( email )

        self.refreshUserList()

    def addGroupDialog( self ):
        '''show a dialog requesting the name of the group to be added'''

        if not self.connected:
            return

        response = Dialogs.AddGroup( self.mainWindow ).run()

        if response == None:
            return
        elif response == "":
            Dialogs.Error( self.mainWindow , _( "Empty Field" ) ).run()
            return

        self.addGroup( response )
        self.refreshUserList()

    def addGroup( self, group ):
        '''add a group'''

        self.msn.addGroup( group )

    def addNotification( self, msnp, command, tid, params, payload ):
        '''this method is called when a user adds you'''

        # this looks like when someone adds you
        # <ml><d n="gmail.com"><c n="luismarianoguerra" t="1" l="8" f="[Mariano%20Guerra]%20(IRC%20is%20just%20multiplayer%20notepad.)" /></d></ml>

        email = payload.split( '<c n="' )[ 1 ].split( '"' )[ 0 ] + '@' + payload.split( '<ml><d n="' )[ 1 ].split( '"' )[ 0 ]
        nick = ''

        try:
            nick = payload.split( ' f="' )[ 1 ].split( '"' )[ 0 ].replace( '%20', ' ' )
        except:
            pass

        self.userAddedYouDialog(nick, email)

    def userAddedYouDialog( self, nick, email ):
        d = Dialogs.AddBuddy( self.mainWindow , nick + " (" + email + ")"  )
        response = d.run()

        if response == gtk.RESPONSE_ACCEPT:
            self.addUser(email)
        else:
            pass # TODO make the user that adds you not able to see you!!

    def removeNotification( self, msn, command, tid, params, payload ):
        '''this method is called when a user removes you'''

        # this looks like when someone removes you:
        # <ml><d n="gmail.com"><c n="luismarianoguerra" t="1" l="8" /></d></ml>

        #mail = payload.split( '<c n="' )[ 1 ].split( '"' )[ 0 ] + '@' + payload.split( '<ml><d n="' )[ 1 ].split( '"' )[ 0 ]
        #Dialogs.Information( self.mainWindow , _( 'The user ' ) + mail + _( ' has deleted you' ) ).run()
        pass

    def removeGroup( self, group = None ):
        '''remove a group, if group is None the group will be the selected on the userlist'''

        if group == None:
            group = self.mainWindow.userList.getGroupSelected()

        if group == "":
            Dialogs.Error( self.mainWindow , _( "Choose a group to delete" ) ).run()
            return

        if Dialogs.Confirm( self.mainWindow , _( "Are you sure you want to delete %s?" )%group, \
            gtk.STOCK_DELETE ).run() == gtk.RESPONSE_ACCEPT:
            self.msn.removeGroup( group )

        self.refreshUserList()

    def renameGroupDialog( self, group = None ):
        '''show a dialog requesting the new name of the group, if group is None
        use the selected group in the userlist'''

        if group == None:
            group = self.mainWindow.userList.getGroupSelected()

            if group == "":
                Dialogs.Error( self.mainWindow , _( "Choose a group" ) ).run()
                return

        response = Dialogs.RenameGroup( self.mainWindow , group ).run()

        if response == None:
            return
        elif response == "":
            Dialogs.Error( self.mainWindow , _( "Empty group name" ) ).run()
            return

        self.renameGroup( group, response )
        self.refreshUserList()

    def renameGroup( self, oldName, newName ):
        '''rename a group from oldName to newName'''

        self.msn.renameGroup(oldName, newName)

    def preferencesDialog( self ):
        '''show a preference dialog'''
        p = PreferenceWindow.PreferenceWindow( self, self.config, self.mainWindow )
        p.show()

    def themeChanged( self ):
        '''called when the theme changes'''

        theme = self.config.getUserValue( "theme" )
        isSmall = bool( int( self.config.getUserValue( "smallIcons" ) ) )
        smilieTheme = self.config.getUserValue( "smilieTheme" )
        
        error = False
        if not self.theme.setTheme( theme, isSmall ): error = True
        if not self.theme.smilies.setTheme( smilieTheme ): error = True
        
        if error:
            Dialogs.Error( self.mainWindow, \
                       _( "Error loading theme, falling back to default" ) \
                       ).run()
            self.config.setUserValue( 'theme', 'default' )
            self.config.setUserValue( 'smilieTheme', 'default' )
            theme = 'default'
            stheme = 'default'

        self.mainWindow.rebuild()
        if self.trayStatusMenu:
            self.trayStatusMenu.set_submenu( StatusMenu.StatusMenu( self ) )
            self.trayIcon.menu.show_all()
            self.trayIcon.update( self.msn.status )
            
        return theme

    def setAutoReplyDialog( self ):
        '''show a dialog requesting the autoreply message'''

        if not self.connected:
            return

        response = Dialogs.SetAutoReply( self.mainWindow , self.autoReplyMessage ).run()

        if response == None:
            return
        elif response == "":
            Dialogs.Error( self , _( "Empty AutoReply" ) ).run()
        else:
            self.autoReplyMessage = response
            self.config.setUserValue( "autoReplyMessage" , response )

    def showUserData( self, data ):
        '''show the user nick or mail in the status bar'''
        self.mainWindow.showUserData( data )

    def refreshUserList( self, msnp = None ):
        '''call to refreshUserList on MainWindow that calls refresh on userList
        userList get the values of showOffline etc from config, so if you want
        to change something, change it on config.
        if it's loading contacts (ILN), don't do anything'''
        
        if self.lastILN == 0 or time.time() - self.lastILN > 1:
            self.mainWindow.refreshUserList()
            self.lastILN = 0

    # signal handlers
    def connectionProblem( self, msnp ):
        '''called when a PNG is not answered'''
        
        Dialogs.Error( self.mainWindow, \
                       _( "Connection  error" ) \
                       ).run()
                       
        self.logout()
    
    def onInitialStatusChange( self, msnp, command, tid, params):
        '''callback for ILN messages. saves the timestamp to speed up login'''
        self.lastILN = time.time()
        
    def onStatusChange( self, msnp, command, tid, params ):
        '''callback for the status change event'''
        self.refreshUserList()

    def onStatusOnline( self, msnp, command, tid, params ):
        '''callback for the status online event'''
        self.refreshUserList()

    def onStatusOffline( self, msnp, command, tid, params ):
        '''callback for the status offline event'''
        self.refreshUserList()
        
    def on_tray_disconnect( self, *args ):
        self.logout()

    def msnobjChanged( self, msnp, msnobj, wasOffline ):
        '''callback called when the user change his msnobj'''
        # disabled.. this breaks everything
        return
        
        if msnobj != None and not wasOffline:
            print "asking " + repr( msnobj )
            if msnobj.getCreator() == '': return
            sb = msnp.getSwitchboard( msnobj.getCreator() )
            
            if sb == None:
                sb = msnp.newSwitchboard()
                sb.invite( msnobj.getCreator() )
                
            sb.getDisplayPicture( msnobj.getCreator() )

    def newConversation( self, msnp, mail, switchboard = None, weStarted = False ):
        '''This method is called when the user want to initiate a new conversation with someone,
        or when a switchboard has been created and we want to give a window to it.
        (callback for 'new conversation' signal emmited by msnp)
        weStarted is a boolean that indicate if we started the conversation or 
        a friend'''

        self.conversationManager.newConversation( self.msn, mail, switchboard, weStarted )

    def pluginDialog( self ):
        '''Show the pluginManagerDialog'''

        p = PluginManagerDialog.PluginManagerDialog( self.pluginManager, self.mainWindow, self.config )
        p.show()

    def seeSpace( self, email ):

        #self.msn.getSpaceData( email )
        #desktop.open( 'http://spaces.live.com/' + user )
        self.msn.getUserProfile( email )

    def offlineMessageWaiting( self, msnp, msnOIM ):
        '''process the OIM messages'''
        for i in msnOIM.getMessages():
            msnOIM.retrieve( i[ 'I' ] )

    def offlineMessageReceived( self, msnp, oim ):
        '''process the OIM message'''
        user, date, message = oim
        self.newConversation( msnp, user['addr'], None, False )
        conv = self.conversationManager.conversations[user['addr']][1]
        conv.receiveOIM( user['name'], message, date )
       
    def messageError( self, msnp, to, message,error ):
        self.newConversation( self.msn, to, None, True )
        conv = self.conversationManager.conversations[to][1]
        conv.receiveError( msnp, to, message, error )

    def userDisconnected( self, msnp, tid, params ):
        '''method called when the server disconnect us'''

        Dialogs.Error( self.mainWindow , _( 'The server has disconnected you.' ) ).run()
        self.logout()

    def getGroupNames( self, ordered=True):
        '''Returns a list with the group names in the contact list'''
        
        groups = self.msn.getGroupNames()
        groups.sort( key= lambda x: x.lower() )
        
        return groups

    def moveUserToGroup( self, user, oldGroup, group ):
        '''move a user to a group'''

        self.msn.moveUserToGroup( user, oldGroup, group )

    def addUserToGroup( self, user, group ):
        '''add a user to a group'''

        if group == 'No group':
            return

        self.msn.addUserToGroup( user, group )

    def removeUserFromGroup( self, user, group ):
        '''remove a user from a group'''
        self.msn.removeUserFromGroup( user, group )
        
    def getMenuData( self ):
        return self.mainWindow.userList.getMenuData()
        
    def getAvatar(self):
        return self.avatar
    
    def getConfig(self):
        return self.config
        
    def getTheme(self):
        return self.theme
        
    def getUnifiedParser( self ):
        return self.unifiedParser
        
    def getConversationLayoutManager(self):
        return self.conversationLayoutManager
        
    def getMSN(self):
        return self.msn
        
    def getNick( self ):
        if self.msn:
            return self.msn.nick
        else:
            return ''
        
    def getPersonalMessage( self ):
        if self.msn:
            return self.msn.personalMessage
        else:
            return ''
        
    def getUnreadMails( self ):
        if self.msn:
            return self.msn.inboxUnreadMessages
        else:
            return 0
            
    def getContactStatus( self, contact = None ):
        '''return the contact status, if the contact is None return
        the user status'''

        if not self.msn:
            return 'FLN'
        
        if contact:
            return self.msn.contactManager.getContactStatus(contact)
        else:
            return self.msn.status
    
    def getContactAlias( self, contact = None ):
        '''return the contact alias'''

        if not self.msn:
            return ''

        return self.msn.contactManager.getContactAlias( contact )
        
    def getContactIsBlocked( self, contact = None ):
        if not self.msn:
            return False

        return self.msn.contactManager.getContactIsBlocked( contact )
        
loginStatus = 0
loginError = ''

def login( msn ):
    global loginStatus
    global loginError
    try:
        msn.login()
    except Exception, e:
        traceback.print_exc()
        loginStatus = -1
        print e
        loginError = e
        return False
    except:
        loginStatus = -1
        return False

    loginStatus = 1
    return False

if __name__ == '__main__':
    
    path = sys.path[0]
    
    originalPath = os.getcwd()
    os.chdir( path )
    sys.path.append(path)

    # parse arguments
    try:
        args = getopt.getopt(sys.argv[1:], "m", ["minimized","user="])
    except getopt.GetoptError, e:
        # wrong input, display usage
        print e
        print "Usage: " + sys.argv[0] + " [-m|--minimized] [--user=mail@address]"
        sys.exit(1)

    # parse getopt output
    username = ''
    minimized = False
    for key, value in args[0]:
        if key=='--user':
            username = value
        if key=='-m' or key=='--minimized':
            minimized = True
    # start controller
    controller = Controller(username, minimized)
    try:
        gtk.main()
    except KeyboardInterrupt:
        print "Interrupt"
        os.chdir( originalPath )
        exit()
    os.chdir( originalPath )
