# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


import gtk
import pango
import gobject

import Conversation
import ConversationWindow

class ConversationManager( gobject.GObject ):
    '''This class handle a collection of conversations'''

    __gsignals__ = {
        # window, conversation
        'new-conversation-ui' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, 
        (gobject.TYPE_PYOBJECT,gobject.TYPE_PYOBJECT)),
        # window, conversation
        'close-conversation-ui' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, 
        (gobject.TYPE_PYOBJECT,gobject.TYPE_PYOBJECT)),
        # send conversation, message
        'send-message' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, 
        (gobject.TYPE_PYOBJECT,gobject.TYPE_STRING)),
        # receive conversation, message
        'receive-message' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, 
        (gobject.TYPE_PYOBJECT,gobject.TYPE_STRING,gobject.TYPE_STRING,gobject.TYPE_STRING,gobject.TYPE_STRING,gobject.TYPE_STRING)),
    }

    def __init__( self, controller ):
        '''Contructor'''

        gobject.GObject.__init__(self)

        self.controller = controller
        self.theme = controller.theme
        self.config = controller.config

        #a dict of tuples assigned by email key
        #eg.: "jhon@ta.com" : (ConversationWindow, Conversation)
        self.conversations = {}
        
        self.controller.connect( 'preferences-changed', self.guiPreferencesChanged )
        
    def guiPreferencesChanged( self, controller ):
        for ( window, conversation ) in self.conversations.values():
            conversation.getUI().update()

    def newConversation( self,msnp, mail, switchboard, weStarted ):
        '''Create a new conversation, add it to the dict with the mail and return it
        if switchboard is None we create the switchboard'''
        #TODO: refactor me! (but dont introduce new bugs :D)
        
        contactStatus = self.controller.getContactStatus(mail)
        windowIcon = self.theme.statusToPixbuf(contactStatus)
        
        # remove the closed windows
        for i in self.conversations.keys():
            if self.conversations[ i ][ 0 ].isClosed() or \
                ( self.conversations[ i ][ 0 ].notOpen and \
                    self.conversations[ i ][ 1 ].getStatus() == 'closed' ):
                self.emit('close-conversation-ui', self.conversations[ i ][ 1 ], self.conversations[ i ][ 0 ])
                del self.conversations[ i ]
            elif self.conversations[ i ][ 1 ].isClosed():
                self.emit('close-conversation-ui', self.conversations[ i ][ 1 ], self.conversations[ i ][ 0 ])
                del self.conversations[ i ]

        # we see if there is a switchboard open with this user and not
        # associated witha any conversation
        # this can happen if we are requesting his DP and you try to open
        # a conversation
        # this call can return None if there is no switchboard opened with this
        # user
        # switchboard = msnp.getSwitchboard( mail )
                
        if not self.conversations.has_key( mail ):
            wasNone = (switchboard == None)
            
            # if we havent a previously conversation and the parameter of
            # switchboard is None we create it
            if switchboard == None:
                switchboard = msnp.newSwitchboard()
                switchboard.invite( mail )
                
            conversation = Conversation.Conversation( self.controller, switchboard )
            
            # we see if we have tabbed chat or single windows
            # if we have tabbed chat we try to get the first window and add it
            # a new tab if not, just create a new window
            if self.config.getUserValue( 'windows' ) == '0' and len( self.conversations ) > 0:
                window = self.conversations.itervalues().next()[ 0 ]
                window.openTab( conversation )
            else:
                window = ConversationWindow.ConversationWindow(self.controller, conversation)
                
            window.set_icon(windowIcon)
            self.conversations[ mail ] = (window, conversation)
            conversation.setWindow( window )

            if wasNone:
                if not weStarted:
                    window.iconify() 
                    
                window.show()
                
            self.emit( 'new-conversation-ui',conversation,window )
        else:
            if switchboard != None:
                self.conversations[ mail ][ 1 ].setSwitchboard( switchboard )
            elif weStarted:
                window = self.conversations[ mail ][ 0 ]
                conversation = self.conversations[ mail ][ 1 ]
                
                # if tabs
                if self.config.getUserValue( 'windows' ) == '0':
                    window.showTab( conversation.getTabNum() )

                window.present()
                
        return self.conversations[ mail ][ 0 ]
      
    def do_send_message( self, conversation, message ):
        '''Send message to conversation'''
        conversation.do_send_message( message )
        
    def do_receive_message( self, conversation, mail, nick, message, format, charset ):
        '''Receive a message from'''
        conversation.do_receive_message( mail, nick, message, format, charset  )

    def closeAll( self ):
        '''close all the conversations'''

        for (key,value) in self.conversations.iteritems():
            value[ 0 ].hide()

        self.conversations = {}
