# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import gtk
import gobject

import ListOfUsers
import SmilieWindow
from emesenecommon import escape
from emesenecommon import debugmsg
from msnPlusMarkup import msnPlusMarkup
import pango
import time

from htmltextview import HtmlTextView

class ConversationUI( gtk.VBox ):
    '''this class represent all the widgets that are inside a tab
    also old the tab widget because there is no better place than
    this to hold it...'''
    
    def __init__( self, controller, parentConversation ):
        gtk.VBox.__init__( self )
        
        self.parentConversation = parentConversation
        self.controller = controller
        self.config = self.controller.config
        
        # the header that contain the nick, the PM, status etc
        self.header = Header( self, controller )
        # the widget that contain the tabs and the conversations
        self.tabWidget = TabWidget( self, controller )
        # the widget where you can put the text
        self.input = InputWidget( self, controller )
        # the status bar
        self.status = gtk.Statusbar()
        # the toolbar
        self.toolbar = ToolbarWidget( self, controller )
        self.toolbar.set_style(gtk.TOOLBAR_ICONS)
        self.toolbarinput = gtk.VBox()
        # the list of users when we are on a group chat
        self.listOfUsers = ListOfUsers.ListOfUsers( self.parentConversation.getMembersDict(), self.controller.getTheme(), self.controller )
        
        # the scroll for the list of users
        self.scrollList = gtk.ScrolledWindow()
        self.scrollList.set_shadow_type(gtk.SHADOW_IN)
        self.scrollList.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        self.scrollList.set_shadow_type( gtk.SHADOW_IN )
        self.scrollList.set_size_request( 180, -1 )
        self.scrollList.add( self.listOfUsers )
        
        # the widget where the conversation is shown
        self.textview = HtmlTextView( self.parentConversation.getTextBuffer() )
        self.textview.set_wrap_mode( gtk.WRAP_WORD_CHAR )
        self.textview.set_left_margin( 6 )
        self.textview.set_right_margin( 6 )
        self.textview.set_editable( False )
        self.textview.set_cursor_visible( False )

        # the scroll for the textview above
        self.scroll = gtk.ScrolledWindow()
        self.scroll.set_shadow_type(gtk.SHADOW_IN)
        self.scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_ALWAYS)
        self.scroll.add( self.textview )

        # Friends avatar
        
        self.imageFrame = gtk.AspectFrame()
        self.image = gtk.Image()
        self.image.set_from_pixbuf(self.controller.getTheme().getUserDisplayPicture( self.getParentConversation().switchboard.firstUser ) )   
        self.imageFrame.add( self.image )
        self.controller.getMSN().connect( 'display-picture-changed', self.displayPictureChanged )
        
        # Vbox to pack both the userlist and friend display image
        self.vbox = gtk.VBox()
        self.vbox.pack_start( self.imageFrame, False,False )
        self.vbox.pack_start( self.scrollList, True, True)
        self.vbox.set_border_width( 2 )


        # the horizontal panel to show or hide the userlist
        self.hpaned = gtk.HPaned()
        self.hpaned.pack1( self.scroll, True, False )
        self.hpaned.pack2( self.vbox, False, True )
        self.hpaned.compute_position(400, 300,100)
        
        self.hpaned.connect( 'size-allocate', self.setUserListSize )

        # the hbox that pack the textview, the images of the other users and the userlist
        self.hbox = gtk.HBox()
        self.hbox.set_border_width( 2 )

        self.hbox.pack_start(self.hpaned, True, True)

        self.toolbarinput.pack_start( self.toolbar, False, False )
        self.toolbarinput.pack_start( self.input, True, True )

        # the vertical panel that divide the textview and the input ( toolbar and input )
        self.vpaned = gtk.VPaned()
        self.vpaned.pack1( self.hbox, True, True )
        self.vpaned.pack2( self.toolbarinput, False, True )

        self.pack_start( self.header, False, False )
        self.pack_start( self.vpaned, True, True )
        self.pack_start( self.status, False, False )
        
        self.messageWaiting = False
        self.contactTyping = False
        self.typingTimeoutID = 0
        self.typingStatusbarMessages = 0

        #self.input.grabFocus()
        self.show_all()
        
        self.update()
         
    def getInput(self):
        return self.input
        
    def getParentConversation(self):
        return self.parentConversation
        
    def update( self ):
        self.header.update()
        self.tabWidget.update()
        self.input.update()
        
        if self.contactTyping:
            self.setTyping()
        else:
            self.setDefault()        
        
        members = self.parentConversation.getMembersDict()
        self.listOfUsers.populate( members )

        # if you want to show the avatar and hide the grouplist or
        # viceversa first do the hide and then the show, because this methods
        # also hide the container vbox to not let an empty widget there

        if bool(int(self.config.getUserValue( 'avatarDual' ))):
            if len( self.parentConversation.getMembers() ) > 1:
                self.hideAvatar()
                self.showUserList()
            else:
                self.hideUserList()
                self.showAvatar()
        else:
            self.hideUserList()
            self.hideAvatar()
            
        if self.config.getUserValue( 'showHeader', '1' ) == '0':
            self.header.hide()
        else:
            self.header.show()

        if self.config.getUserValue( 'showToolbar', '1' ) == '0':
            self.toolbar.hide()
        else:
            self.toolbar.show()
        
    def setUserListSize( self, *args ):
        rect = self.hpaned.get_allocation()
        listWidth = int( rect.width * 1 / 4 ) 
        self.scrollList.set_size_request( listWidth, -1 )
        
    def showUserList( self ):
        self.vbox.show()
        self.scrollList.show_all()
        self.listOfUsers.show()
        
    def hideUserList( self ):
        self.vbox.hide()
        self.scrollList.hide()
        self.listOfUsers.hide()

    def showAvatar(self):
        self.vbox.show()
        self.imageFrame.show_all()

    def hideAvatar(self):
        self.vbox.hide()
        self.imageFrame.hide()
        
    def getTabWidget( self ):
        return self.tabWidget
    
    def getTextView( self ):
        return self.textview
    
    def setCloseTabCallback( self, callback ):
        self.tabWidget.setCloseTabCallback( callback )
        
    def setMessageWaiting( self ):
        if self.parentConversation.getIsCurrent(): return self.setDefault()
        
        context = self.status.get_context_id('typing')
        while self.typingStatusbarMessages > 0:
            self.status.pop(context)
            self.typingStatusbarMessages -= 1
        
        self.tabWidget.setMessageWaiting()
        self.header.setDefault()
        self.contactTyping = False
        self.messageWaiting = True
        
    def setTyping( self ):
        if self.messageWaiting: return
        self.header.setTyping()
        self.tabWidget.setTyping()
        
        # TODO: add multiple user support
        string =  _( 'The user' ) + ' ' + _( 'is typing a message...' )
        contextid = self.status.get_context_id('typing')
        self.status.push(contextid, string)
        self.typingStatusbarMessages += 1
        
        self.contactTyping = True
        if self.typingTimeoutID > 0:
            gobject.source_remove(self.typingTimeoutID)
        self.typingTimeoutID = gobject.timeout_add( 8000 , self.clearTyping )

    def clearTyping ( self ):
        if self.messageWaiting:
            self.setMessageWaiting()
        else:
            self.setDefault()
        self.contactTyping = False
        return False

    def setDefault( self ):
        context = self.status.get_context_id('typing')
        while self.typingStatusbarMessages > 0:
            self.status.pop(context)
            self.typingStatusbarMessages -= 1
        
        self.tabWidget.setDefault()
        self.header.setDefault()
        self.contactTyping = False
        self.messageWaiting = False
        
    def getInput(self):
        return self.input
 
    def displayPictureChanged( self, msnp, email):
        if self.getParentConversation().getSwitchboard().firstUser == email:
            self.image.set_from_pixbuf(self.controller.getTheme().getUserDisplayPicture( email ) )  
                
class Header( gtk.HBox ):
    '''the header of the conversation'''
    
    def __init__( self, parentUI, controller ):
        '''Contructor'''
        gtk.HBox.__init__( self )
        self.parentUI = parentUI
        self.controller = controller
        self.plusMarkup = msnPlusMarkup()
        self.set_border_width( 6 )
        self.set_spacing( 6 )
        
        self.statusImage = gtk.Image()
        self.statusImage.set_from_pixbuf( self.controller.getTheme().getImage( 'groupChat' ) )
        
        self.label = gtk.Label()
        self.label.set_alignment( 0.0, 0.5 )
        self.label.set_line_wrap( True )
        self.label.set_ellipsize( pango.ELLIPSIZE_END )
        # when someone fin how to make that this label is selected all the time,
        # discoment the next line
        #self.label.set_selectable( True )
                
        self.label.set_markup( '<b><i>' + _( 'Connecting...' ) + '</i></b>' )
        
        self.reconnect = gtk.Button( _( 'Reconnect' ) )
        self.reconnect.connect('clicked', self.reconnectClicked)
        self.pack_start( self.statusImage, False, False )
        self.pack_start( self.label )
        self.pack_start( self.reconnect, False, False)
        self.statusImage.show()
        self.label.show()
        self.show()
        
    def reconnectClicked(self, button):
        self.parentUI.getParentConversation().reconnect()
        self.reconnect.hide()
        
    def update( self ):
        conversation = self.parentUI.getParentConversation()
        members = conversation.getMembersDict()
        if conversation.getSwitchboard().getStatus() == 'error':
            self.setLabel()
        elif len( members ) == 1:
            contact = members.values()[ 0 ]
            self.statusImage.set_from_pixbuf(self.controller.getTheme().statusToPixbuf( contact.status ) )
        else:
            self.statusImage.set_from_pixbuf(self.controller.getTheme().getImage( 'userPanel' ) )

    def setTyping( self ):
        typingColor = self.controller.getConfig().getUserValue('typingColor','#339999')
        styleOpen = "<b><span foreground='" + typingColor + "'>"
        styleClose = "</span></b>"
        self.setLabel( styleOpen, styleClose )
    
    def setLabel( self, styleOpen='<b>', styleClose='</b>' ):
        conversation = self.parentUI.getParentConversation()
        members = conversation.getMembersDict()
        personalMessageColor = self.controller.getConfig().getUserValue('personalMessageColor','#AAAAAA')
        SB = conversation.getSwitchboard()
        if SB.getStatus() == 'error':
            self.text = '%s<i> %s (%d) </i>%s' % \
                (styleOpen, _( "Can't connect" ), SB.error, styleClose)
            self.label.set_markup( self.text )
            self.reconnect.show()
            return
        self.reconnect.hide()

        if len( members ) == 1:
            contact = members.values()[ 0 ]
            self.text = styleOpen + self.plusMarkup.removeMarkup(escape(contact.nick)) + styleClose
            
            if len( contact.personalMessage ) > 0:
                self.text += "\n<span foreground='" + personalMessageColor + "'>"
                self.text += escape( contact.personalMessage ) + "</span>"
        
            self.text += "\n<span foreground='" + personalMessageColor + "'>"
            self.text += contact.email + "</span>"
        elif len( members ) > 1:
            self.text = styleOpen + _( 'Group chat' ) + styleClose
            self.text += "\n<span foreground='" + personalMessageColor + "'>"
            self.text += str( len( members ) ) + ' ' + _( 'Members' ) + "</span>"

            self.statusImage.set_from_pixbuf(self.controller.getTheme().getImage( 'groupChat' ) )
        else:
            self.text = styleOpen + '<i>' + _( 'Connecting...' ) + '</i>' + styleClose
                
        self.label.set_markup( self.text )
          
    def setDefault( self ):
        self.setLabel()
        
class TabWidget( gtk.HBox ):
    '''this class represent the widget that is inside the tab
    it contains a label and a close button'''
    
    def __init__( self, parentUI, controller ):
        '''Contructor'''
        gtk.HBox.__init__( self )
        self.parentUI = parentUI
        self.controller = controller
        self.text = escape(self.parentUI.getParentConversation().getTitle())

        self.label = gtk.Label()
        self.label.set_use_markup( True )
        self.label.set_markup( self.text )
        
        img = gtk.Image()
        img.set_from_stock( gtk.STOCK_CLOSE, gtk.ICON_SIZE_MENU )
        
        self.closeButton = gtk.Button()
        self.closeButton.set_image( img )
        self.closeButton.set_relief( gtk.RELIEF_NONE )
        
        self.pack_start( self.label, True, True )
        self.pack_start( self.closeButton, False,False )
        self.show_all()
        
        self.closeCallback = None
        self.closeButton.connect( 'clicked', self.onCloseClicked )
        
    def onCloseClicked( self, *args ):    
        if self.closeCallback != None:
            self.closeCallback( self.parentUI.getParentConversation(), self.parentUI.getParentConversation().tabNum )
            
    def setCloseCallback( self, callback ):
        self.closeCallback = callback
        
    def update( self ):
        title = self.parentUI.getParentConversation().getTitle()
        if not title == '':
            self.text = escape(title)
    
    def setDefault( self ):
        self.label.set_markup( self.text )
        
    def setTyping( self ):
        typingColor = self.controller.getConfig().getUserValue('typingColor', '#339999')
        self.label.set_markup( '<span foreground="%s">%s</span>' % (typingColor, self.text) )
    
    def setMessageWaiting( self ):
        messageWaitingColor = self.controller.getConfig().getUserValue( 'messageWaitingColor', '#E53B1A' )
        self.label.set_markup( '<span foreground="%s">%s</span>' % (messageWaitingColor, self.text) )
    
    def setCloseTabCallback( self, callback ):
        self.closeCallback = callback
    
class InputWidget(gtk.HBox):
    '''This class represents the input widgets (text entry) and avatar (if enabled)'''
    def __init__(self, parentUI, controller):
        gtk.HBox.__init__(self)
        self.parentUI = parentUI
        self.controller = controller
        self.lastKeyPressed = 0
        self.history = []
        self.historyIndex = -1
        self.config = controller.config
        self.set_border_width( 2 )

        self.avatarFrame = gtk.AspectFrame()
        
        self.localimage = gtk.Image()
        self.imageEventBox = gtk.EventBox()
        self.imageEventBox.set_events(gtk.gdk.BUTTON_PRESS_MASK)
        self.imageEventBox.connect( 'button-press-event', self.changeAvatar )
        self.imageEventBox.add( self.localimage )
        
        if self.controller.avatar != None:
            self.localimage.set_from_pixbuf(self.controller.avatar.getImage())
        else:
            self.pixbuf = self.controller.theme.getImage( 'icon96' )
            self.localimage.set_from_pixbuf( self.pixbuf )
        
        self.avatarFrame.add( self.imageEventBox )
        self.controller.connect('avatar-changed',self.localDisplayPictureChanged)
        
        self.pack_end(self.avatarFrame, False, False, 0)
            
        self.input = gtk.TextView()
        self.input.set_left_margin( 6 )
        self.input.set_right_margin( 6 )
        self.input.set_wrap_mode(gtk.WRAP_WORD)
        
        self.inputBuffer = self.input.get_buffer()
        
        self.scrollInput = gtk.ScrolledWindow()
        self.scrollInput.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.scrollInput.add( self.input )
        
        frameInput = gtk.Frame()
        frameInput.add(self.scrollInput)
        
        self.pack_start(frameInput, True, True, 0)
        
        self.inputBuffer.connect( 'changed', self.on_changed_event )
        self.input.connect( "key-press-event" , self.on_key_press_event )
        
        self.input.connect_after( 'map-event', self.on_input_map_event )
        self.input.grab_focus()
        self.tag = None
        self.applyAttrsToInput()
        self.show_all()
        self.update()
        
    def update( self ):
        if self.config.getUserValue( 'showMyAvatar', '1' ) == '0':
            self.avatarFrame.hide()
        else:
            self.avatarFrame.show()
    
    def grabFocus(self):
        self.input.grab_focus() 
        
    def getInput(self):
        return self.input
    
    def getBuffer(self):
        return self.inputBuffer
    
    def on_input_map_event( self, widget, event ):
        self.input.grab_focus()
    
    def clearMessageInput(self):
        self.setInputText('')
        return True
    
    def sendMessage(self):
        if self.getInputText() != '':
            # Message size is limited so we must truncate it to send it in several parts
            message = self.getInputText()
            
            self.parentUI.getParentConversation().sendMessage( message )
            self.setInputText( '' )
            self.history.append( message )
            self.historyIndex = -1
        return True
    
    def setInputText( self , string , tag = None ):
        if tag == None:
            self.inputBuffer.set_text( string )
        else:
            pass
        
    def getInputText( self ):
        '''return the text in the input textview'''
        return self.inputBuffer.get_text( self.inputBuffer.get_start_iter(),\
                self.inputBuffer.get_end_iter() , True )
        
    def appendInputText( self, text, tag = None ):
        '''append the given text to the inputBuffer, if tag != None inset with the given tag'''
        self.inputBuffer.insert_at_cursor( text )
        
    def on_changed_event( self, *args ):
        '''Method called when text is inserted in the textbuffer (typing)
        First, it checks if the message length is >= 5 or if
        the message is a slash command ("^\/[^\/]" is confusing)
        Then, checks the last key pressed time, and if it's more
        than 5 seconds, send a typing message'''
        
        text = self.getInputText()
        if len(text) < 5 or (text[0] == '/' and text[1] != '/'):
            return self.updateInputFormat()
        
        # sents user typing notification
        actualTime = int( time.time() )
        
        if ( actualTime - self.lastKeyPressed ) > 5:
            self.lastKeyPressed = actualTime

            try:
                self.parentUI.getParentConversation().sendIsTyping()
            except Exception , e:
                print str( e )
                
        self.updateInputFormat()
                
    def applyAttrsToInput( self ):
        '''apply the current attributes to the text in input'''

        self.tag = self.parentUI.getParentConversation().getTextTag()
        self.inputBuffer.get_tag_table().add( self.tag )
        self.tag.set_property( 'left-margin', 6 )
        self.tag.set_priority( self.inputBuffer.get_tag_table().get_size() - 1 )
        self.inputBuffer.apply_tag( self.tag,
                                            self.inputBuffer.get_start_iter(),
                                            self.inputBuffer.get_end_iter() )
        
        self.controller.emit( 'input-format-changed', self.input )
        
    def updateInputFormat(self):
        if self.tag != None:
            self.inputBuffer.apply_tag( self.tag,
                                        self.inputBuffer.get_start_iter(),
                                        self.inputBuffer.get_end_iter() )
        # i don't see why
        #self.controller.emit( 'input-format-changed', self.input )
        
    def on_key_press_event( self , widget, event ):
        if not( event.state & gtk.gdk.SHIFT_MASK ) and \
            not( event.state & gtk.gdk.CONTROL_MASK ) and \
            not( event.state & gtk.gdk.MOD1_MASK ) and \
            ( event.keyval == gtk.keysyms.Return or
                event.keyval == gtk.keysyms.KP_Enter ):

            return self.sendMessage()
        
        elif event.keyval == gtk.keysyms.Escape:
            self.parentUI.getParentConversation().close()

        elif event.keyval in (gtk.keysyms.Up, gtk.keysyms.Down) and \
            (event.state & gtk.gdk.CONTROL_MASK):
            up = event.keyval == gtk.keysyms.Up
            
            if up and self.historyIndex == -1 and len(self.history) > 0:
                self.historyIndex = len(self.history) - 1
                if self.getInputText() != '':
                    self.history.append(self.getInputText())
            
            elif self.historyIndex > 0 and self.historyIndex < len(self.history):
                if up: self.historyIndex -= 1
                else: self.historyIndex += 1
            
            if self.historyIndex > -1 and self.historyIndex < len(self.history):
                self.setInputText(self.history[self.historyIndex])
            elif self.historyIndex != -1:
                self.setInputText('')

    def displayPictureChanged( self, msnp, email):
        if self.parentUI.getParentConversation().getSwitchboard().firstUser == email:
            self.image.set_from_pixbuf(self.controller.getTheme().getUserDisplayPicture( email ) )  

    def localDisplayPictureChanged( self, controller):
        self.localimage.set_from_pixbuf(self.controller.avatar.getImage()) 

    def changeAvatar( self, widget, event ):
        self.controller.changeAvatarDialog()
            

            
class ToolbarWidget(gtk.Toolbar):
    '''This represents a toolbar that contains text formatation, smilies buttom, do nudge button,
    and so on...'''
    
    def __init__(self, parentUI, controller):
        gtk.Toolbar.__init__(self)
        self.parentUI = parentUI
        self.controller = controller
        self.smilieWindow = SmilieWindow.SmilieWindow( self.controller, \
                                                       self.smilieSelected, \
                                                       None )

        self.fontFace = gtk.ToolButton()
        self.fontFace.set_stock_id(gtk.STOCK_SELECT_FONT)
        self.fontFace.connect('clicked', self.on_font_face_clicked)
        
        self.fontColor = gtk.ToolButton()
        self.fontColor.set_stock_id(gtk.STOCK_SELECT_COLOR)
        self.fontColor.connect('clicked', self.on_font_color_clicked)
        
        self.boldButton = gtk.ToggleToolButton()
        self.boldButton.set_stock_id(gtk.STOCK_BOLD)
        self.boldButton.connect( 'toggled', self.toggled, 'bold' )
        self.boldButton.set_active(self.controller.getConfig().getUserValue('fontBold') == '1')
        
        self.italicButton = gtk.ToggleToolButton()
        self.italicButton.set_stock_id(gtk.STOCK_ITALIC)
        self.italicButton.connect( 'toggled', self.toggled, 'italic' )
        self.italicButton.set_active(self.controller.getConfig().getUserValue('fontItalic') == '1')

        self.underlineButton = gtk.ToggleToolButton()
        self.underlineButton.set_stock_id(gtk.STOCK_UNDERLINE)
        self.underlineButton.connect( 'toggled', self.toggled, 'underline' )
        self.underlineButton.set_active(self.controller.getConfig().getUserValue('fontUnderline') == '1')

        self.strikeButton = gtk.ToggleToolButton()
        self.strikeButton.set_stock_id(gtk.STOCK_STRIKETHROUGH)
        self.strikeButton.connect( 'toggled', self.toggled, 'strike' )
        self.strikeButton.set_active(self.controller.getConfig().getUserValue('fontStrike') == '1')
                
        smilie = gtk.ToolButton()
        smilie.set_label( _( 'Smilie' ) )
        imgSmilie = gtk.Image()
        imagenGrin =self.controller.getTheme().getSmiley( ":D" )

        if type( imagenGrin ) == gtk.gdk.PixbufAnimation:
            imgSmilie.set_from_pixbuf( imagenGrin.get_static_image() )
        elif type( imagenGrin ) == gtk.gdk.Pixbuf:
            imgSmilie.set_from_pixbuf( imagenGrin )
        else:
            imgSmilie.set_from_stock( gtk.STOCK_MISSING_IMAGE ,gtk.ICON_SIZE_SMALL_TOOLBAR )
            print ':D image is not a gtk.pixbuf or pixbufanimation this should not happen please fix me!'
            print 'the type is ' + str( type( imagenGrin ) ) 
            
        smilie.set_icon_widget( imgSmilie )

        smilie.connect( 'clicked', self.showSmilieWindow )

        nudge = gtk.ToolButton()
        nudge.set_label( _( 'Nudge' ) )
        imgNudge = gtk.Image()
        imgNudge.set_from_pixbuf(self.controller.getTheme().getImage( 'nudge' ) )
        nudge.set_icon_widget( imgNudge )

        nudge.connect( 'clicked', self.doNudge )

        invite = gtk.ToolButton()
        invite.set_label( _( 'Invite' ) )
        invite.set_stock_id( gtk.STOCK_ADD )

        invite.connect( 'clicked', self.showInviteDialog )
        
        #clear conversation toolbar button
        imgclear = gtk.Image()
        imgclear.set_from_stock(gtk.STOCK_CLEAR, gtk.ICON_SIZE_LARGE_TOOLBAR )
        clear = gtk.ToolButton( imgclear, _('Clear Conversation') )

        clear.connect( 'clicked', self.clearOutputText )

        # Add buttons to the toolbar
        self.insert( self.fontFace, -1 )
        self.insert( self.fontColor, -1 )
        self.insert( self.boldButton, -1 )
        self.insert( self.italicButton, -1 )
        self.insert( self.underlineButton, -1 )
        self.insert( self.strikeButton, -1 )
        
        self.insert( gtk.SeparatorToolItem(), -1 )
        
        self.insert( smilie, -1 )
        self.insert( nudge, -1 )

        self.insert( gtk.SeparatorToolItem(), -1 )

        self.insert( invite, -1 )
        self.insert(clear, -1)

        # Set tooltips to buttons
        self.tooltips = gtk.Tooltips()
        self.tooltips.enable()

        self.fontFace.set_tooltip( self.tooltips, _( 'Font selection' ) )
        self.fontColor.set_tooltip( self.tooltips, _( 'Font color selection' ) )
        self.boldButton.set_tooltip( self.tooltips, _( 'Bold' ) )
        self.italicButton.set_tooltip( self.tooltips, _( 'Italic' ) )
        self.underlineButton.set_tooltip( self.tooltips, _( 'Underline' ) )
        self.strikeButton.set_tooltip( self.tooltips, _( 'Strike' ) )
        smilie.set_tooltip( self.tooltips, _( 'Insert a smilie' ) )
        nudge.set_tooltip( self.tooltips, _( 'Send nudge' ) )
        invite.set_tooltip( self.tooltips, _( 'Invite a friend to the conversation' ) )
        clear.set_tooltip( self.tooltips, _( 'Clear conversation' ) )
        
        self.show_all()
        
    def on_font_face_clicked( self, *args ):
        ''' Called when user clicks on the font selection button in the toolbar'''
        fontDesc = pango.FontDescription( self.parentUI.getParentConversation().getFontFace() )
        if self.parentUI.getParentConversation().getFontItalic():
            uStyle = pango.STYLE_ITALIC
        else:
            uStyle = pango.STYLE_NORMAL
        if self.parentUI.getParentConversation().getFontBold():
            uWeight = pango.WEIGHT_BOLD
        else:
            uWeight = pango.WEIGHT_NORMAL
        fontDesc.set_style( uStyle )
        fontDesc.set_weight( uWeight )
        fontDesc.set_size( self.parentUI.getParentConversation().getFontSize() * pango.SCALE )

        fontDialog = gtk.FontSelectionDialog( _( 'Choose a font' ) )
        fontDialog.set_font_name( fontDesc.to_string() )
        response = fontDialog.run()

        if response == gtk.RESPONSE_OK:
            pangoDesc = pango.FontDescription( fontDialog.get_font_name() )
            font = pangoDesc.get_family()
            style = pangoDesc.get_style()
            weight = pangoDesc.get_weight()

            italic = bold = False

            if style == pango.STYLE_ITALIC:
                italic = True
            if weight == pango.WEIGHT_BOLD:
                bold = True

            self.controller.emit( 'font-changed', font, bold, italic, pangoDesc.get_size() / pango.SCALE )
            
        fontDialog.destroy()
    
    def on_font_color_clicked( self, *args ):
        ''' Called when user clicks on the color selection button in the toolbar'''
        colorDialog = gtk.ColorSelectionDialog( _( 'Choose a color' ) )
        colorDialog.colorsel.set_has_palette( True )
        colorDialog.colorsel.set_current_color(gtk.gdk.color_parse(self.controller.getConfig().getUserValue('fontColor')))
        response = colorDialog.run()

        if response == gtk.RESPONSE_OK:
            color = colorDialog.colorsel.get_current_color()
            red = color.red >> 8
            green = color.green >> 8
            blue = color.blue >> 8
            
            self.controller.emit( 'color-changed', '#%02X%02X%02X' % ( red , green , blue ) )
            
        colorDialog.destroy()
    
    def toggled( self, button, option ):
        '''callback for the toggled signal'''

        if option == 'bold':
            self.parentUI.getParentConversation().setFontBold( button.get_active() )
        elif option == 'italic':
            self.parentUI.getParentConversation().setFontItalic( button.get_active() )
        elif option == 'underline':
            self.parentUI.getParentConversation().setFontUnderline( button.get_active() )
        elif option == 'strike':
            self.parentUI.getParentConversation().setFontStrike( button.get_active() )

        self.parentUI.getInput().applyAttrsToInput()
        
    def setFontBold( self, value ):
        self.boldButton.set_active(value)

    def setFontUnderline( self, value ):
        self.underlineButton.set_active(value)
        
    def setFontStrike( self, value ):
        self.strikeButton.set_active(value)
        
    def setFontItalic( self, value ):
        self.italicButton.set_active(value)
                
    def smilieSelected( self, smilie=None ):
        '''this method is called when the user click a smilie in the smiliewindow'''

        if smilie:
            self.parentUI.getInput().appendInputText( smilie )
        self.parentUI.getInput().grabFocus()
        
    def showSmilieWindow( self, *args ):
        '''this method is called when the user click the smilie button'''
        self.smilieWindow.show()
        
    def showInviteDialog( self, *args ):
        '''this method is called when the user click the invite button'''
        dictionary = self.parentUI.getParentConversation().contactManager.getOnlineUsersDict()
        members = {}
        
        for i in self.parentUI.getParentConversation().getMembers() + [ self.parentUI.getParentConversation().getUser() ]:
            members[ i ] = self.parentUI.getParentConversation().contactManager.getContact( i )
            
        InviteWindow( self.controller, None, dictionary, members, self.controller.getTheme(), self.parentUI.getParentConversation().inviteUser ).run()
          
    def clearOutputText(self, *args):
        self.parentUI.getParentConversation().getTextBuffer().set_text('')
        
    def doNudge( self, *args ):
        '''this method is called when the user click the nudge button'''
        self.parentUI.getParentConversation().doNudge() 
 
    def doBlock( self, *args ):
        '''
        this method is called when the user clicks the block button
        '''
        users = self.parentUI.getParentConversation().getMembers()
        if len(users)==1:
            self.controller.blockUser(users[0])
 
    def doUnblock( self, *args ):
        '''
        this method is called when the user clicks the block button
        '''
        users = self.parentUI.getParentConversation().getMembers()
        if len(users)==1:
            self.controller.unblockUser(users[0])
   

class InviteWindow(gtk.Dialog):
    '''This class represent a list where the user can pick wich users he want to
    invite to a conversation'''

    def __init__( self, controller, father, onlineUsers, usersInConversation, theme, callback ):
        '''Constructor callback is called when a user is selected with double click'''
        gtk.Dialog.__init__( self, _( 'Invite' ), father,
                     gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                     (gtk.STOCK_ADD, gtk.RESPONSE_ACCEPT) )

        self.set_default_size( 200, 200 )
        self.callback = callback
        users = onlineUsers
        
        self.controller = controller
        
        for i in usersInConversation.keys():
            if users.has_key( i ):
                del users[ i ]

        self.userList = ListOfUsers.ListOfUsers( users, theme, self.controller, self.userListClicked )
        
        self.scroll = gtk.ScrolledWindow()
        self.scroll.set_policy( gtk.POLICY_AUTOMATIC , gtk.POLICY_AUTOMATIC )
        self.scroll.add( self.userList )
        
        self.connect( 'response', self.invite )
        self.connect( 'delete-event', self.close )

        self.vbox.pack_start( self.scroll )
        self.vbox.show_all()
        
    def invite( self, dialog, response_id ):
        '''get the user in the userlist and call the callback'''

        try:
            if response_id == gtk.RESPONSE_ACCEPT:
                self.callback( self.userList.getSelected() )
                self.close()
                return True
            else:
                return False
        except Exception, e:
            print e
            
    def userListClicked( self, email ):
        self.callback( email )
        self.close()

    def close( self, *args ):
        '''close the window'''

        self.hide()
