# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import gtk
import time
import pango
import gobject

import SmilieWindow
from emesenecommon import escape
import ListOfUsers

class ConversationWindow(gtk.Window):
    '''This class is the conversation window, is contains a textbuffer for the
    input a toolbox and a set of tabs, if there is just one conversation
    the window is like the one in earlier versions but if we add more
    conversations it behaves like a Gaim window'''

    def __init__(self, controller, conversation):
        '''
        Constructor, the conversation parameter is an instance of Conversation,
        
        '''
        
        gtk.Window.__init__(self)
        self.controller = controller
        self.closed = False
        self.notOpen = True
         
        self.width = int(controller.getConfig().getUserValue('convWindowWidth'))
        self.height = int(controller.getConfig().getUserValue('convWindowHeight'))
        self.set_default_size(self.width, self.height)
        self.connect('size-allocate', self.on_size_alloc)
        
        #self.set_icon(theme.getImage('icon'))
        
        # accelerators
        accelGroup = gtk.AccelGroup()
        self.add_accel_group(accelGroup)
        self.accelGroup = accelGroup
        accelGroup.connect_group(gtk.keysyms.Page_Down, \
                                  gtk.gdk.CONTROL_MASK, gtk.ACCEL_LOCKED, \
                                  self.on_key_change_tab)
        accelGroup.connect_group(gtk.keysyms.Page_Up, \
                                  gtk.gdk.CONTROL_MASK, gtk.ACCEL_LOCKED, \
                                  self.on_key_change_tab)
        accelGroup.connect_group(gtk.keysyms.Tab, \
                                  gtk.gdk.CONTROL_MASK, gtk.ACCEL_LOCKED, \
                                  self.on_key_change_tab)
        accelGroup.connect_group(gtk.keysyms.KP_Tab, \
                                  gtk.gdk.CONTROL_MASK, gtk.ACCEL_LOCKED, \
                                  self.on_key_change_tab)
                                  

        self.conversation = conversation
        self.conversation.setIsCurrent(True)
        self.set_title(self.conversation.getTitle(False))

        self.textTagCount = 0

        self.user = self.conversation.getUser()
        self.conversations = {}
        # holds the timestamp of the last key pressed for he is typing message
#        self.lastKeyPressed = 0

        self.menu = ConversationWindowMenu(self, controller)
        
        self.vbox = gtk.VBox()
        self.tabs = gtk.Notebook()

        self.tabs.set_show_border(False)
        self.tabs.set_property('tab-vborder', 0)

        self.tabs.set_show_tabs(False) #ok, ok, never show tabs on window init
        self.tabs.set_scrollable(True)
        self.tabs.popup_enable()
        self.openTab(conversation)

        self.vbox.pack_start(self.menu, False, False)
        self.vbox.pack_start(self.tabs, True, True)

        self.vbox.show_all()

        self.connect('delete-event', self.close)
        self.connect('focus-in-event', self.on_focus_in_event)
        self.tabs.connect('switch-page', self.changeTabs)

        self.add(self.vbox)
        
        self.conversation.getUI().hideUserList()

    def getConversation(self):
        return self.conversation
                
    def on_focus_in_event(self, widget, event):
        self.unsetUrgency()
        self.conversation.getUI().getInput().grabFocus()

    def on_size_alloc(self, widget, allocation):
        self.width = allocation.width
        self.height = allocation.height
        
    def on_key_change_tab(self, accelGroup, window, keyval, modifier):
        '''Catches events like Ctrl+AvPag and consequently changes current tab'''
        
        if not modifier == gtk.gdk.CONTROL_MASK:
            return
        
        print "jo's here"
        
        if keyval == gtk.keysyms.Page_Down:
            cycle = 1
        elif keyval == gtk.keysyms.Tab or keyval == gtk.keysyms.KP_Tab:
            print "jo's here1111"
            cycle = 1
        elif keyval == gtk.keysyms.Page_Up:
            cycle = -1
        else:
            return
        
        notebook = self.tabs
        pages = notebook.get_n_pages()
        active = notebook.get_current_page()
        self.showTab((active + cycle) % pages)

    def showTab( self, num ):
        '''put the num tab as de selected one'''
        
        # TODO: put page as second parameter, we currently dont use it
        # but if you do please put it here.
        self.changeTabs(self.tabs, None, num)
        self.tabs.set_current_page(num)

    def doIsTyping(self, mail):
        self.conversation.receiveTyping(mail)
    
    def getTabPage(self, conversation):
        '''return a tab page according to its conversation object'''
        
        tabNum = None

        for k, v in self.conversations.items():
            if v == conversation:
                tabNum = k
                break
                
        return tabNum
    
    def getOutputTextView(self):
        '''return the current textview'''
        
        return self.conversation.getUI().getTextView()
    
    def changeTabs(self, notebook, page, page_num):
        '''callback for the change-current-page signal'''
        
        self.conversation.setIsCurrent(False)
        self.conversation = self.conversations[page_num]
        self.set_icon(self.conversation.getUI().header.statusImage.get_pixbuf())
        self.output = self.conversation.getUI().getTextView()
        title = self.conversation.getTitle(False)
        if not title == '':
            self.set_title(title)
        self.conversation.setIsCurrent(True)
        self.conversation.getUI().update()
        
    def openTab(self, conversation):
        '''open a new tab, that represent a new conversation'''
        
        self.output = conversation.getUI().getTextView()
        tabNum = self.tabs.append_page(child = conversation.getUI(), tab_label = conversation.getUI().getTabWidget())
        self.conversations[ tabNum ] = conversation
        #add the new tab label to notebook menu popup
        self.addOrUpdateMenuPopupItem(tabNum, conversation)

        conversation.tabNum = tabNum
        conversation.getUI().setCloseTabCallback(self.close_tab)

        self.tabs.set_show_tabs(len(self.conversations) > 1)  #ok, ok, show tabs only if needed
        self.tabs.show_all()
        conversation.getUI().hideUserList()
        self.tabs.set_current_page(tabNum)
        self.present()
        #conversation.getUI().getInput().grabFocus()
        
    def addOrUpdateMenuPopupItem(self, tabNum, conversation):
        '''add the new tab label to notebook popup menu'''
        
        page = self.tabs.get_nth_page(tabNum)
        hbox = gtk.HBox()
        hbox.set_spacing(6)
        image = gtk.Image()
        image.set_from_pixbuf(self.controller.getTheme().getImage('online'))
        hbox.pack_start(image, False, False)
        hbox.pack_start(gtk.Label(conversation.getTitle()))
        hbox.show_all()
        self.tabs.set_menu_label(page, hbox)

    def close_tab(self, conversation, num):
        '''Close the num-th tab'''

        self.tabs.remove_page(num)

        #erase the conversation
        self.conversations[ num ].setStatus('closed')
        self.conversations[ num ].setClosed(True)
        del self.conversations[ num ]

        # compact the conversations because we get a hole now
        keyList = [ x for x in self.conversations.keys() ]
        keyList.sort()

        for i in range(0, self.tabs.get_n_pages()):

            key = keyList[ 0 ]
            if len(keyList) > 1:
                keyList = keyList[ 1: ]

            self.conversations[ i ] = self.conversations[ key ]
            self.conversations[ i ].tabNum = i
            if i != key:
                del self.conversations[ key ]

        if(len(self.conversations) == 0):
            self.destroy()
        else:
            self.conversation = self.conversations[ self.tabs.get_current_page() ]
            self.set_title(self.conversation.getTitle(False))
            self.conversation.setIsCurrent(True)
            self.output = self.conversation.getUI().getTextView()
            self.tabs.set_show_tabs(len(self.conversations) > 1)
            
    def clearOutputText(self, *args):
        self.conversation.getTextBuffer().set_text('')

    def show(self):
        gtk.Window.show(self)
        self.notOpen = False

    def close(self, *args):
        '''close the window and set the attribute self.closed to true'''

        self.hide()
        
        for i in self.conversations.keys():
            self.conversations[ i ].setClosed(True)

        self.closed = True
        
        self.controller.getConfig().setUserValue('convWindowWidth', str(self.width))
        self.controller.getConfig().setUserValue('convWindowHeight', str(self.height))

        return True

    def isClosed(self):
        '''return True if the window has been hided'''
        return self.closed

    def save(self):
        '''save the content of the conversation'''
        self.conversation.log.save()

    def scrollToBottom(self, tab_num = None):
        '''scroll to the end of the conversation the textview in tha tab_num
        if None scroll the actual'''

        if tab_num == None:
            mark = self.conversation.getTextBuffer().create_mark("end",\
                 self.conversation.getTextBuffer().get_end_iter(), False)
            self.output.scroll_to_mark(mark, 0.05, True, 0.0, 1.0)
        else:
            textview = self.conversations[ tab_num ].ui.getTextView()
            textbuffer = textview.get_buffer()
            mark = textbuffer.create_mark("end", textbuffer.get_end_iter(), False)
            textview.scroll_to_mark(mark, 0.05, True, 0.0, 1.0)

    def addChildAtAnchor(self, child, anchor, tab_num = None):
        '''add a child to an anchor in the current output if tab_num is None, otherwise add it in the tab'''

        if tab_num == None:
            self.output.add_child_at_anchor(child, anchor)
        else:
            self.conversations[ tab_num ].ui.getTextView().add_child_at_anchor(child, anchor)

    def setUrgency(self):
        '''try to set the urgency hint to the window'''

        if not self.has_toplevel_focus():
            try:
                self.set_urgency_hint(True)
            except:
                pass

    def unsetUrgency(self):
        '''try to unset the urgency hint to the window'''

        try:
            self.set_urgency_hint(False)
        except:
            pass
        
        self.conversation.getUI().update()


class ConversationWindowMenu(gtk.MenuBar):
    '''
    This class represent the menu in the conversation window, i define it here because
    no other module here cares about this one.
    '''

    def __init__(self, parentConversationWindow, controller):
        '''
        Contructor
        '''

        gtk.MenuBar.__init__(self)

        self.parentConversationWindow = parentConversationWindow
        self.controller = controller
        accelGroup = parentConversationWindow.accelGroup

        # --- File menu
        conversationMenu = gtk.Menu()
        conversationMenuItem = gtk.MenuItem(_("_File"))
        conversationMenuItem.set_submenu(conversationMenu)

        saveMenuItem = gtk.ImageMenuItem(_("_Save conversation"))
        saveMenuItem.set_image(gtk.image_new_from_stock(gtk.STOCK_SAVE, gtk.ICON_SIZE_MENU))
        saveMenuItem.add_accelerator('activate', accelGroup, ord('S'), gtk.gdk.CONTROL_MASK, gtk.ACCEL_VISIBLE)
        quitMenuItem = gtk.ImageMenuItem(gtk.STOCK_QUIT)
        quitMenuItem.add_accelerator('activate', accelGroup, ord('Q'), gtk.gdk.CONTROL_MASK, gtk.ACCEL_VISIBLE)
        
        saveMenuItem.connect('activate', self.onSaveActivate)
        quitMenuItem.connect('activate', self.onQuitActivate)
        
        conversationMenu.add(saveMenuItem)
        conversationMenu.add(gtk.SeparatorMenuItem())
        conversationMenu.add(quitMenuItem)
        self.add(conversationMenuItem)       
        
        # --- Format menu
        formatMenu = gtk.Menu()
        formatMenuItem = gtk.MenuItem(_("For_mat"))
        formatMenuItem.set_submenu(formatMenu)
        
        fontMenuItem = gtk.ImageMenuItem(gtk.STOCK_SELECT_FONT)
        colorMenuItem = gtk.ImageMenuItem(gtk.STOCK_SELECT_COLOR)

        fontMenuItem.connect('activate', self.onFontActivate)
        colorMenuItem.connect('activate', self.onColorActivate)
        
        formatMenu.add(fontMenuItem)
        formatMenu.add(colorMenuItem)
        self.add(formatMenuItem)

    def onSaveActivate(self, *args):
        '''
        This methd is called when Save is activated on the menu
        '''
        self.parentConversationWindow.save()

    def onQuitActivate(self, *args):
        '''
        This methd is called when Quit  is activated on the menu
        '''
        self.parentConversationWindow.close()

    def onLogActivate(self, check):
        '''
        This methd is called when Log is activated on the menu
        '''
        self.parentConversationWindow.getConversation().setDoLog(check.get_active())

    def onFontActivate(self, *args):
        '''
        This method is called when font is activated on the menu
        '''

        fontDesc = pango.FontDescription(self.parentConversationWindow.getConversation().getFontFace())
        if self.parentConversationWindow.getConversation().getFontItalic():
            uStyle = pango.STYLE_ITALIC
        else:
            uStyle = pango.STYLE_NORMAL
        if self.parentConversationWindow.getConversation().getFontBold():
            uWeight = pango.WEIGHT_BOLD
        else:
            uWeight = pango.WEIGHT_NORMAL
        fontDesc.set_style(uStyle)
        fontDesc.set_weight(uWeight)
        fontDesc.set_size(self.parentConversationWindow.getConversation().getFontSize() * pango.SCALE)

        fontDialog = gtk.FontSelectionDialog(_('Choose a font'))
        fontDialog.set_font_name(fontDesc.to_string())
        response = fontDialog.run()

        if response == gtk.RESPONSE_OK:
            pangoDesc = pango.FontDescription(fontDialog.get_font_name())
            font = pangoDesc.get_family()
            style = pangoDesc.get_style()
            weight = pangoDesc.get_weight()

            italic = bold = False

            if style == pango.STYLE_ITALIC:
                italic = True
            if weight == pango.WEIGHT_BOLD:
                bold = True

            self.controller.emit('font-changed', font, bold, italic, pangoDesc.get_size() / pango.SCALE)
            
        fontDialog.destroy()

    def onColorActivate(self, *args):
        '''
        This methd is called when color is activated on the menu
        '''

        colorDialog = gtk.ColorSelectionDialog(_('Choose a color'))
        colorDialog.colorsel.set_has_palette(True)
        colorDialog.colorsel.set_current_color(gtk.gdk.color_parse(self.controller.getConfig().getUserValue('fontColor')))
        response = colorDialog.run()

        if response == gtk.RESPONSE_OK:
            color = colorDialog.colorsel.get_current_color()

            red = color.red >> 8
            green = color.green >> 8
            blue = color.blue >> 8

            self.controller.emit('color-changed', '#%02X%02X%02X' % (red, green, blue))
            
        colorDialog.destroy()

