# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import gtk

from emesenecommon import escape
import TreeViewTooltips

import pango

class ListOfUsers( gtk.TreeView ):
    '''a list of users that can be used to show it on a group conversation
    or to show the invite dialog'''
    
    def __init__( self, userList, theme, controller, callback = None ):
        '''Contructor, userList is a list of Contact instances'''
        
        #icon, nick, personal message, mail
        self.model = gtk.ListStore( gtk.gdk.Pixbuf, str, str )
        gtk.TreeView.__init__( self, self.model )
        
        self.callback = callback
        self.theme = theme
        self.controller = controller
        
        self.cellRendererText = gtk.CellRendererText()
        self.cellRendererPixbuf = gtk.CellRendererPixbuf()
        
        self.cellRendererText.set_property( 'ellipsize', pango.ELLIPSIZE_END )
        
        column = gtk.TreeViewColumn()

        column.set_title('Icons & Text')
        column.pack_start(self.cellRendererPixbuf, expand=False)
        column.pack_start(self.cellRendererText, expand=True)
        column.add_attribute(self.cellRendererPixbuf, 'pixbuf', 0)
        column.add_attribute(self.cellRendererText, 'text', 1)
        column.set_attributes(self.cellRendererText, markup=1)

        self.populate( userList )
        
        self.append_column( column )
        self.set_headers_visible( False )
        
        tooltips = TreeViewTooltips.TreeViewTooltips( self, 1, 2 )
        
        self.connect( "row_activated" , self.row_activated )
        
    def row_activated( self, treeview, path, view_column ):
        '''call the callback with the user selected'''
        
        if self.callback != None:
            self.callback( self.getSelected() )
            
    def getSelected( self ):
        return self.model.get( self.get_selection().get_selected()[ 1 ], 2 )[ 0 ]
    
    def populate( self, contacts ):
        self.model.clear()
        
        for i in contacts.itervalues():
            label = escape( i.nick ) + ' - <span foreground="#AAAAAA">' + escape( i.personalMessage ) + '</span>\n'
            #label += '<span foreground="#666666">(' + i.getEmail() + ')</span>'
            
            self.model.append( [ self.theme.getUserDisplayPicture( i.email, 48, 48 ), label, i.email ] )
