# -*- coding: utf-8 -*-

#   This file is part of emesene
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import MainMenu
import UserList
import UserPanel
import FilterEntry
import Login
import Loading
import TrayIcon
import emesenecommon
from emesenecommon import debugmsg

import sys

try:
    import gtk
    import gobject
except:
    print "you need pyGTK to run emesene"
    sys.exit( -1 )

class MainWindow( gtk.Window ):
    '''
    This class represent the main window of emesene, it inherit from gtk.Window
    and basically it is a container for other classes.
    '''

    def __init__( self, controller ):
        '''
        Constructor
        '''

        gtk.Window.__init__( self )
        
        self.controller = controller
        
        # accelerators
        accelGroup = gtk.AccelGroup()
        self.add_accel_group( accelGroup )
        self.accelGroup = accelGroup

        self.set_title( "emesene" )
        #self.set_position( gtk.WIN_POS_CENTER )
        
        #self.set_gravity( gtk.gdk.GRAVITY_STATIC )
        self.x = int( self.controller.getConfig().getGlobalValue( 'mainWindowX' ) )
        self.y = int( self.controller.getConfig().getGlobalValue( 'mainWindowY' ) )
        self.connect( 'map', self.on_show )
        #self.connect( 'unmap', self.on_hide )

        self.set_geometry_hints( self, -1, -1 )
        self.width = int( controller.getConfig().getGlobalValue( 'mainWindowWidth' ) )
        self.height = int( controller.getConfig().getGlobalValue( 'mainWindowHeight' ) )
        self.set_default_size( self.width , self.height )
        self.connect( 'size-allocate', self.on_size_alloc )

        theme = controller.getTheme()
        gtk.window_set_default_icon_list( theme.getImage( 'icon16' ),
                                          theme.getImage( 'icon32' ),
                                          theme.getImage( 'icon48' ),
                                          theme.getImage( 'icon96' ) )

        self.vbox = None
        self.loading = self.login = None
        self.userList = None
        
        self.currentInterface = 'login'
        self.buildInterface( 'login' )
        
    def on_size_alloc( self, widget, allocation ):
        self.width = allocation.width
        self.height = allocation.height
    
    def on_show( self, widget ):
        if not (self.x < 0 or self.y < 0):
            self.move( self.x, self.y)
            
    def on_hide( self, widget ):
        self.x, self.y = self.get_position()
        
    def saveToQuit( self ):
        '''
        Saves configuration parametes and everything needed ir order
        to safely quit
        '''
        
        self.controller.getConfig().setGlobalValue( 'mainWindowWidth', str( self.width ) )
        self.controller.getConfig().setGlobalValue( 'mainWindowHeight', str( self.height ) )
        
        #x, y = self.get_position()
        self.controller.getConfig().setGlobalValue( 'mainWindowX', str( self.x ) )
        self.controller.getConfig().setGlobalValue( 'mainWindowY', str( self.y ) )

    def quit( self, status = 0 ):
        '''
        close the window, and do all the things needed...
        '''     
        self.controller.quit( status )
        
    def hide( self ):
        '''
        Hides itself and any other window
        '''
        # saves position
        self.on_hide( self )
        
        if self.userList:
            self.userList.tooltips.hideTooltip()
        
        # ------------ let the window hide itself the last ---------
        gtk.Window.hide( self )

    def hideOrClose( self, *args ):
        '''
        hide or close depending if we have trayicon
        '''

        if TrayIcon.disabled:
            self.quit( 0 )
        else:
            self.hide()

        return True

    def buildInterface( self , type = 'login' ):
        '''
        build the interface depending on the type parameter, by
        default build the login window.
        '''

        #self.set_icon( self.controller.theme.getImage( 'login' ) )

        if self.get_child() != None:
            self.remove( self.vbox )
            
        if self.userList:
            self.userList.tooltips.hideTooltip()

        # if i dont add this if we disconnect then the image isnt shown
        if self.currentInterface == "login" and self.login:
            try:
                self.login.remove( self.login.loginImage )
            except:
                debugmsg( "Error when removing loginImage", 1)
        elif self.currentInterface == "loading" and self.loading:
            self.loading.remove( self.loading.loading )

        self.currentInterface = type
                    
        #widgets

        self.vbox = gtk.VBox()
        #self.status = gtk.Statusbar()
        self.scroll = gtk.ScrolledWindow()
        self.scroll.set_policy( gtk.POLICY_NEVER , gtk.POLICY_AUTOMATIC )
        self.scroll.set_shadow_type( gtk.SHADOW_IN )
        self.menu = MainMenu.MainMenu( self.controller, \
                                       type, self.accelGroup )

        if type == "login":
            self.vbox.pack_start( self.menu, False, False )
            self.login = Login.Login( self.controller )
            self.vbox.pack_start( self.login )
            self.menu.show_all()
            #self.trayicon = TrayIcon.TrayIcon(self, self.theme)
        elif type == "userlist":
            self.vbox.pack_start( self.menu, False, False )
            self.userList = UserList.UserList( self.controller )
            self.userList.refresh()
            
            self.userPanel = UserPanel.UserPanel( self.controller )
            self.vbox.pack_start( self.userPanel, False, False )
            self.filterEntry = FilterEntry.FilterEntry( self.userList.setFilterText )
            self.vbox.pack_start( self.filterEntry, False, False )
            self.scroll.add( self.userList )
            vbox2 = gtk.VBox( spacing=4 )
            vbox2.pack_start( self.scroll )
            self.statusCombo = StatusCombo( self.controller )
            vbox2.pack_start( self.statusCombo, False, False )
            self.vbox.pack_start( vbox2 )
            
            self.menu.show_all()
            vbox2.show_all()
            self.controller.connect( 'preferences-changed', self.update )

        elif type == 'loading':
            self.loading = Loading.Loading( self.controller )
            self.vbox.pack_start( self.loading )

        #self.vbox.pack_start( self.status, False, False )
        self.add( self.vbox )
        self.vbox.show()
        self.update( self.controller )
        self.connect( "delete-event" , self.hideOrClose )

    def update( self, controller ):
        if not controller or self.currentInterface != "userlist":
            return
        
        if controller.getConfig().getUserValue( 'showUserPanel', '1' ) == '0':
            self.userPanel.hide()
        else:
            self.userPanel.show()
            
        if controller.getConfig().getUserValue( 'showSearchEntry', '1' ) == '0':
            self.filterEntry.hide()
        else:
            self.filterEntry.show()
                
        if controller.getConfig().getUserValue( 'showStatusCombo', '1' ) == '0':
            self.statusCombo.hide()
        else:
            self.statusCombo.show()

    def rebuild( self ):
        '''repaint the currentinteface'''

        self.buildInterface(self.currentInterface)

    def showUserData( self, data ):
        '''
        show the user data from the userlist
        '''

        pass

        #if self.status != None:
        #    self.status.pop( 1 )
        #    self.status.push( 1 , data )

    def refreshUserList( self ):
        '''
        refresh the userlist :D
        (if we are in userlist mode)
        '''

        if self.currentInterface == 'userlist':
            self.userList.refresh()

    def setAvatar( self, pixbuf ):
        if self.currentInterface == 'userlist':
            self.userPanel.setAvatar( pixbuf )

class StatusCombo( gtk.ComboBox ):
    '''this class represent the combo wwhere you set the status'''

    def __init__( self, controller ):
        '''Constructor'''

        self.statusListStore = gtk.ListStore( gtk.gdk.Pixbuf, \
                      gobject.TYPE_STRING, gobject.TYPE_STRING )

        gtk.ComboBox.__init__( self, self.statusListStore )

        self.controller = controller

        self.statusPixbufCell = gtk.CellRendererPixbuf()
        self.statusTextCell = gtk.CellRendererText()
        self.pack_start( self.statusPixbufCell, False )
        self.pack_start( self.statusTextCell, False )
        self.statusPixbufCell.set_property( 'xalign', 0.0 )
        self.statusPixbufCell.set_property( 'xpad', 5 )
        self.statusTextCell.set_property( 'xalign', 0.0 )
        self.statusTextCell.set_property( 'xpad', 5 )
        self.add_attribute(self.statusPixbufCell, 'pixbuf', 0)
        self.add_attribute(self.statusTextCell, 'text', 2)
        self.set_resize_mode(0)

        counter = 0
        flag = False
        j = 0
        for i in controller.status_ordered[ 1 ]:
            if emesenecommon.status_table[ i ] == self.controller.getContactStatus():
                flag = True

            if not flag:
                counter += 1
                
            if i != "offline":
                self.statusListStore\
                        .append([ self.controller.theme.statusToPixbuf(
                               emesenecommon.status_table[ i ] ), i,
                               self.controller.status_ordered[ 2 ][ j ] ] )
                j += 1

        self.set_active(counter)
        
        self.connect( 'changed' , self.on_status_changed )
        self.controller.msn.connect( 'self-status-changed', self.selfStatusChanged )

    def selfStatusChanged( self, msnp, status ):
        statusOrdered = self.controller.status_ordered[1]
        
        if emesenecommon.reverse_status[ status ] in statusOrdered:
            self.set_active(statusOrdered.index( emesenecommon.reverse_status[ status ] ))

    def on_status_changed( self , *args ):
        self.controller.setStatus(self.statusListStore.get(self.get_active_iter(), 1)[0], 'combo')

