# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from emesenecommon import escape, unescape
import re
import gobject

IMG_TAG_RE=re.compile('(?:<img src=\"(?P<src>\S*)\" alt=\"(?P<alt>\S*)\"/>)')
IMG_TAG_OTHER_RE=re.compile('(?:<img src=\"(?P<src>\S*)\" alt=\"(?P<alt>\S*)\"/>)|(?P<other>.)')

def extractImgAlt( img ):
    return IMG_TAG_RE.sub( lambda(data): data.groupdict()['alt'], img )


class DefaultDataType( object ):
    def __init__( self, text, parser ):
        self.parser = parser
        self.text = text
        self.theme = parser.getTheme()
        self.parser.emit( 'preparse', self )
        
    def get( self ):
        data = ''
        for item in self.parser.parse( self.text ):
            data += escape( str( item ) )
        return data
    
    def parseAll( self ):
        data = ''
        for item in self.text:
            try:
                data += item.getHtml()
            except:
                data += str( item )
        return data
        
    def getDataType( self ):
        return 'default'

class UserListDataType( DefaultDataType ): 
    def __init__( self, text, parser ):
        DefaultDataType.__init__( self, text, parser )
        self.conversation = None

    def get( self, smileys=True ):
        data = ''
        for item in self.parser.parse( self.text ):
            if type( item ) == str or type( item ) == unicode:
                data += escape( str( item ) )
            elif item.type() == 'url':
                data += str( item )
            elif item.type() == 'image':
                if smileys:
                    data += item.getHtml()
                else:
                    data += escape( str( item ) )
                    
        return data
        
    def getDataType( self ):
        return 'userlist'
        
class ConversationDataType( DefaultDataType ):
    def __init__( self, text, parser ):
        DefaultDataType.__init__( self, text, parser )
        self.conversation = None
        self.ink = None
        self.username = None

    def subOCE( self, data ):
        ce = data.groups()[0]
        msnOM = self.conversation.switchboard.getMSN().getMsnObjectsManager()
        return '<img src="file://%s" alt="%s"/>'%(msnOM.getById( unescape(ce) ).filename, ce )
            
    def subICE( self, data ):
        ce = data.groups()[0]
        customEmoticons = self.conversation.customEmoticons.get( self.username )
        return '<object type="application/x-emesene-emoticon" class="'+customEmoticons[unescape(ce)]+'" data="'+ce+'"></object>'
        
    def get( self, urls=True, smileys=True ):
        data = ''
        cePattern=''
        if self.ink:
            return '<object type="application/x-emesene-ink" class="'+self.text+'"></object>'
        if smileys == True and self.conversation != None:
            if self.username == self.conversation.switchboard.getUser():
                sub = self.subOCE
                msnOM = self.conversation.switchboard.getMSN().getMsnObjectsManager()
                for ce in msnOM.getIds():
                    cePattern += re.escape( escape( ce ) ) + '|'
            else:
                sub = self.subICE
                customEmoticons = self.conversation.customEmoticons.get( self.username )
                for ce in customEmoticons:
                    cePattern += re.escape( escape( ce ) ) + '|'
        
        for item in self.parser.parse( self.text ):
            if type( item ) == str or type( item ) == unicode:
                item = escape( item )
                if cePattern!= '': item = re.sub( '('+cePattern[:-1]+')', sub, item )
                data += item
            elif item.type() == 'url':
                if urls:
                    data += item.getHtml()
                else:
                    data += str( item )
            elif item.type() == 'image':
                if smileys:
                    data += item.getHtml()
                else:
                    data += escape( str( item ) )
                    
        return data
        
    def setConversation( self, conversation ):
        self.conversation = conversation
        
    def setInk( self, value ):
        self.ink = True
        
    def setUser( self, username ):
        if username != None: self.username = username.lower()
            
    def getDataType( self ):
        return 'conversation'

class Smiley(object):
    def __init__( self, smiley, theme ):
        self.smiley = smiley
        self.theme = theme
    
    def getHtml( self ):
        if self.theme.getSmileyPath( self.smiley ) == None:
            print 'Can\'t find the smiley %s'%self.smiley
            return escape( self.smiley )
        
        path = self.theme.getSmileyPath( self.smiley )
        if path.startswith('/'):    
            url = 'file://' + path
        else:
            url = 'file:///' + path # Windows
            
        return '<img src="%s" alt="%s"/>'%(url, escape( self.smiley ))
        
    def getPixbuf( self ):
        return self.theme.getSmiley( self.smiley )
        
    def __str__( self ):
        return self.smiley
        
    def __repr__(self):
        return '<type \'smiley\'>'
    
    def type( self ):
        return 'image'
        
class Url(object):
    def __init__( self, url ):
        self.url = url

    def getHtml( self ):
        href = self.url
        if self.url[:4] == 'www.':
            href  = 'http://'+self.url
        elif self.url[:4] == 'ftp.':
            href  = 'ftp://'+self.url
        return '<a href="'+escape(href)+'">'+escape(self.url)+'</a>'
    
    def getPango( self ):
        return escape( self.url )
        
    def __str__( self ):
        return escape( self.url )
        
    def __repr__( self ):
        return '<type \'URL\'>'
    
    def type( self ):
        return 'url'
    
class UnifiedParser( gobject.GObject ):
    __gsignals__ = {
        'preparse' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, [gobject.TYPE_PYOBJECT] ),
        'postparse' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, [gobject.TYPE_PYOBJECT]),
        }
    
    handlers = {}
    patterns = []
    
    def __init__( self, theme ):
        gobject.GObject.__init__( self )
        self.theme = theme
        self.urlsPattern = "(?P<url>(?<![A-Za-z0-9])((?:https?\:\/\/|ftp\:\/\/|www\.|ftp\.)(?:\S*[;:~]?[^\s\W])))"
        self.smileyPattern=''
        for smiley in theme.getSmileysList():
            self.smileyPattern += re.escape( smiley ) + '|'
        self.smileyPattern = '(?P<smiley>'+self.smileyPattern[:-1]+')'
        
        self.recompileRegex()
        
    def getTheme( self ):
        return self.theme
    
    def getParser( self, text, dataType=DefaultDataType ):
        return dataType( text, self )
    
    def parse( self, text ):
        list = []
        def handler( data ):
            gruop = data.groupdict()

            if gruop['smiley'] != None:
                list.append( Smiley(gruop['smiley'], self.theme) )
            elif gruop['url'] != None:
                list.append( Url( gruop['url'] ) )
            elif gruop['other'] != None:
                i = len(list)-1
                if i >-1 and type( list[i] ) == type('str'):
                    list[i] += gruop['other']
                else:
                    list.append( gruop['other'] )
                    
        self.reALL.sub( handler, text )
        return list
        

    def recompileRegex( self ):
        customPatterns=''
        for customPattern in self.patterns:
            customPatterns = customPattern
        pattern = self.urlsPattern+'|'+self.smileyPattern+customPatterns+'|(?P<other>.)'
        self.reALL = re.compile( pattern,re.DOTALL )
    
    def do_preparse( self, parser ):
        pass
    
    def do_postparse( self ):
        pass
        
        
if __name__=='__main__':
    import Theme
    
    theme = Theme.Theme('')
    parser = UnifiedParser( theme )
    data =  ':D :o :P http://someurl.com/~das/:Dx/index.od?asd=asd&asd test :D <:o)(G)(@)")'
    dataObject = parser.getParser( data, UserListDataType )
    
    print data
    print 'Test 1: parser.parse( data, UserListDataType )'
    print dataObject.get()

