# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


import gtk
import pango
from emesenecommon import *
import Theme
import htmltextview
import gobject

class PreferenceWindow( gtk.Window ):
    '''This class display a windows where the user
    can modify the behaviour of emesene'''
    
    def __init__( self, controller, config, parent ):
        '''Contructor'''
        
        gtk.Window.__init__( self )
        
        self.controller = controller
        self.set_default_size( 300, 250 )
        self.set_title( _( 'Preferences' ) )
        
        self.config = config
        self.set_transient_for( parent )
        self.set_type_hint( gtk.gdk.WINDOW_TYPE_HINT_DIALOG )
        
        self.set_border_width( 12 )
        self.set_resizable( False )
        vbox = gtk.VBox()
        vbox.set_spacing( 12 )
        hButBox = gtk.HButtonBox()
        hButBox.set_spacing( 6 )
        hButBox.set_layout( gtk.BUTTONBOX_END )
        notebook = gtk.Notebook()
        bClose = gtk.Button( stock=gtk.STOCK_CLOSE )
        
        vbox.pack_start( notebook, False, False)
        hButBox.pack_start( bClose )
        vbox.pack_start( hButBox, False, False )
   
        self.connectionTab = ConnectionTab( self.config )    
        notebook.append_page( self.connectionTab, gtk.Label( _( 'Connection' ) ) )
        
        if self.controller.connected:
            self.interfaceTab = InterfaceTab( self.controller )
            self.themesTab = ThemesTab( self.controller )
            self.loggingTab = LoggingTab( self.controller )
            self.colorSchemeTab = ColorSchemeTab(self.controller)
            notebook.append_page( self.interfaceTab, gtk.Label( _( 'Interface' ) ) )
            notebook.append_page( self.themesTab, gtk.Label( _( 'Theme' ) ) )
            notebook.append_page( self.colorSchemeTab, gtk.Label(_('Color scheme')))
            notebook.append_page( self.loggingTab, gtk.Label( _( 'Logs' ) ) )
        
        vbox.show_all()
        self.add( vbox )
        
        self.connect( 'delete-event', self.close )
        bClose.connect( 'clicked', self.close )

    def close( self, *args ):
        '''
        Close the window
        '''
        
        self.hide()
        self.save()
        self.controller.emit( 'preferences-changed' )
        self.destroy()
        
    def save( self ):
        '''
        save the actual setting
        '''
        if self.controller.connected:
            self.interfaceTab.save()
            self.themesTab.save()
            self.loggingTab.save()
            self.colorSchemeTab.save()
        self.connectionTab.save()

class ColorSchemeItem:
    def __init__(self, tab, configColor,text):
        self.configColor=configColor
        self.button=gtk.ColorButton(gtk.gdk.color_parse(tab.config.getUserValue(configColor)))
        self.button.connect("color-set", tab.save)
        alignment = gtk.Alignment(0, 0.5, 0, 0)
        alignment.add(gtk.Label(text))
        self.hbox = gtk.HBox()
        self.hbox.pack_start(alignment, True, True, 3)
        self.hbox.pack_start(self.button, False, False, 3)


class ColorSchemeTab(gtk.VBox):
    '''
    This class represent the tab that contain all the configuration about the colors
    '''
    def __init__(self, controller):
        gtk.VBox.__init__(self)
        
        self.config = controller.getConfig()
        self.controller = controller
        
        self.items={}
        self.items["typingColor"] = ColorSchemeItem(self, 'typingColor', _("Contact typing"))
        self.items["messageWaitingColor"] = ColorSchemeItem(self, 'messageWaitingColor', _("Message waiting"))
        self.items["personalMessageColor"] = ColorSchemeItem(self,'personalMessageColor', _("Personal msn message"))
        self.items["nickColor"] = ColorSchemeItem(self,'nickColor', _("Contact's list"))
        
        vbox = gtk.VBox()
        
        for i in self.items.keys():
            vbox.pack_start(self.items[i].hbox, False, False, 3)
        
        scroll = gtk.ScrolledWindow()
        scroll.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        scroll.add_with_viewport(vbox)
        
        self.pack_start(scroll, True, True, 0)        
        self.pack_start(gtk.HSeparator(), False, True, 2)
        
        restoreButton = gtk.Button(stock = gtk.STOCK_CLEAR)
        restoreButton.connect("clicked", self.restoreColors)
        
        alignment = gtk.Alignment(0.5, 0.5, 0.5, 0.5)
        alignment.add(restoreButton)
        self.pack_start(alignment, False, True, 2)
                        
        self.show_all()
    
    def restoreColors(self, *args):
        for i in self.items.keys():
            self.items[i].button.set_color(gtk.gdk.color_parse(self.config.getUserDefaultValue(self.items[i].configColor))) 
        self.controller.themeChanged()
    
    def save(self, *args):
        for i in self.items.keys():
            self.config.setUserValue(self.items[i].configColor, rgbToHexa(self.items[i].button.get_color()))
        self.controller.themeChanged()
        
class InterfaceTab( gtk.VBox ):
    '''This class represent the tab that contain all the configuration about the GUI'''
    
    def __init__( self, controller ):
        '''Contructor'''
        
        gtk.VBox.__init__( self )
        
        self.controller = controller
        self.config = controller.getConfig()
        
        self.set_spacing( 4 )
        self.set_border_width( 12 )
        
        # main window checkboxes

        self.showUserPanel = gtk.CheckButton(_('Show _user panel' ))
        self.showUserPanel.set_active( bool( int( self.config.getUserValue( 'showUserPanel' ) ) ) )
        self.showSearchEntry = gtk.CheckButton(_('Show _search entry' ))
        self.showSearchEntry.set_active( bool( int( self.config.getUserValue( 'showSearchEntry' ) ) ) )
        self.showStatusCombo = gtk.CheckButton(_('Show status _combo' ))
        self.showStatusCombo.set_active( bool( int( self.config.getUserValue( 'showStatusCombo' ) ) ) )
        self.avatarsInUserList = gtk.CheckButton(_('Show _avatars in userlist' ))
        self.avatarsInUserList.set_active( bool( int( self.config.getUserValue( 'avatarsInUserList' ) ) ) )
                        
        self.showUserPanel.connect( 'toggled', self.onToggled, 'showUserPanel' )
        self.showSearchEntry.connect( 'toggled', self.onToggled, 'showSearchEntry' )
        self.showStatusCombo.connect( 'toggled', self.onToggled, 'showStatusCombo' )
        self.avatarsInUserList.connect( 'toggled', self.onToggled, 'avatarsInUserList' )
                
        self.pack_start(self.avatarsInUserList, expand=True, fill=True, padding=0)
        self.pack_start(self.showUserPanel, expand=True, fill=True, padding=0)
        self.pack_start(self.showSearchEntry, expand=True, fill=True, padding=0)
        self.pack_start(self.showStatusCombo, expand=True, fill=True, padding=0)
                
        # conversation panel
        
        self.conversationPanel = ConversationPanel( self.controller, self.config )
        
        frame = gtk.Frame( _( 'Conversation' ) )
        frame.add( self.conversationPanel )
        
        self.pack_start( frame )
        self.show_all()
        
    def onToggled( self, radio, option ):
        self.config.setUserValue(option, str(int(radio.get_active())) )
        self.controller.emit( 'preferences-changed' )
        
        
    def save( self ):
        '''save the actual setting'''
        
        self.conversationPanel.save()


class LoggingTab( gtk.VBox ):
    '''This class represent the tab to configure logging'''
    
    def __init__( self, controller ):
        '''
        Contructor
        '''
        
        gtk.VBox.__init__( self )
        
        self.config = controller.config
        
        self.set_spacing( 12 )     # just to make it look bigger than it is
        self.set_border_width( 12 ) 
        
        self.saveLogs = gtk.CheckButton( _( 'Save logs automatically' ) )
        self.saveLogs.set_active( bool( int( self.config.getUserValue('saveLogsAutomatically', 0) ) ) )
        
        formats = int( self.config.getUserValue( 'logFormats' ) )
        formatsVbox = gtk.VBox()
        formatsVbox.set_spacing( 12 )
        formatsVbox.set_border_width( 12 )
        self.CheckText = gtk.CheckButton( _( 'Text' ) )
        self.CheckText.set_active( formats & 1 )
        self.CheckHTML = gtk.CheckButton( 'HTML' )
        self.CheckHTML.set_active( formats & 2 )
        formatsVbox.pack_start( self.CheckText, False, False )
        formatsVbox.pack_start( self.CheckHTML, False, False )
        formatsFrame = gtk.Frame( _( 'Log Formats' ) )
        formatsFrame.add( formatsVbox )
        formatsFrameAlig = gtk.Alignment( xscale=1 )
        formatsFrameAlig.set_padding( 24, 24, 24, 24 )
        formatsFrameAlig.add( formatsFrame )
        
        self.logEntry = gtk.Entry()  #TODO: add something to browse paths
        self.logEntry.set_text( self.config.getLogPath() )
        logLabel = gtk.Label( _( '_Log path:' ) )
        logLabel.set_use_underline( True )
        logLabel.set_mnemonic_widget( self.logEntry )
        pathHbox = gtk.HBox()
        pathHbox.pack_start(logLabel, True, True, 0)
        pathHbox.pack_start(self.logEntry, True, True, 0)
        
        self.pack_start(self.saveLogs, False, False)
        self.pack_start(formatsFrameAlig, False, False)
        self.pack_start(pathHbox, False, False)
        
        self.show_all()
        
    def save( self ):
        '''save the actual setting'''
        formats = int(self.CheckText.get_active()) + (int(self.CheckHTML.get_active()) << 1)
        
        self.config.setUserValue( 'saveLogsAutomatically', str(int(self.saveLogs.get_active())) )
        self.config.setUserValue( 'logFormats', str(int(formats)) )
        self.config.setUserValue( 'logPath', self.logEntry.get_text() )


class ConnectionTab( gtk.VBox ):
    '''This class represent the tab that contains the connection options'''
    
    def __init__( self, config ):
        '''
        Contructor
        '''
        
        gtk.VBox.__init__( self )

        self.config = config

        self.set_spacing( 18 )
        self.set_border_width( 12 ) 

        self.debug = gtk.CheckButton( _( '_Show debug in console' ) )
        self.debug.set_active( bool( int( self.config.getGlobalValue( 'debug' ) ) ) )
        self.nodns = gtk.CheckButton( _( '_No DNS mode' ) )
        self.nodns.set_active( bool( int( self.config.getGlobalValue( 'nodns' ) ) ) )
        
        self.useProxy = gtk.CheckButton( _( '_Use proxy' ) + ' ' + _( '(Experimental)' ) )
        self.useProxy.set_active( bool( int( self.config.getGlobalValue( 'useProxy' ) ) ) )
        
        self.proxySettings = ProxySettings( self.config )
        # Proxy fields status
        if self.useProxy.get_active():
            self.proxySettings.enable()
        else:
            self.proxySettings.disable()
        
        frame = gtk.Frame( _( 'Proxy settings' ) )
        frame.add( self.proxySettings )
        
        alig = gtk.Alignment( xscale=1 )
        alig.set_padding( 0, 0, 24, 0 )
        alig.add( frame )

        self.pack_start( self.debug, False, False )
        self.pack_start( self.nodns, False, False )
        
        proxyBox = gtk.VBox( spacing=12 )
        proxyBox.pack_start( self.useProxy, False, False )
        proxyBox.pack_start( alig, False, False )
        
        self.pack_start( proxyBox, False, False )

        self.useProxy.connect( 'toggled', self.useProxyToggled )

        self.show_all()
    
    def useProxyToggled( self, *args ):
        '''callback for the toggled signal'''
        
        if self.useProxy.get_active():
            self.proxySettings.enable()
        else:
            self.proxySettings.disable()
    
    def save( self ):
        '''save the actual setting'''
    
        self.config.setGlobalValue( 'debug', str( int( self.debug.get_active() ) ) )
        self.config.setGlobalValue( 'nodns', str( int( self.nodns.get_active() ) ) )
        self.config.setGlobalValue( 'useProxy', str( int( self.useProxy.get_active() ) ) )
        self.proxySettings.save()
        

class ThemesTab( gtk.VBox ):
    '''This class represent the tab that ...'''
    
    def __init__( self, controller ):
        '''Constructor'''
        
        gtk.VBox.__init__( self )
        
        self.controller = controller
        self.config = controller.getConfig()
        self.theme = controller.getTheme()
        self.set_spacing( 4 )
        self.set_border_width( 12 ) 
        
        self.smallIcons = gtk.CheckButton(_('_Use small icons' ))
        self.smallIcons.set_active( bool( int( self.config.getUserValue( 'smallIcons' ) ) ) )
        self.parseSmilies = gtk.CheckButton(_('Display _smiles' ))
        self.parseSmilies.set_active( bool( int( self.config.getUserValue( 'parseSmilies' ) ) ) )
        
        self.pack_start(self.smallIcons, False, False)
        self.pack_start(self.parseSmilies, False, False)

        fields = []
        
        fields.append({'name': 'theme',
                       'label': _( 'Theme' ),
                       'values': self.config.getThemes(),
                       'config': None,
                       'sample': self.getThemeSample,
                       })
        
        fields.append({'name': 'smilieTheme',
                       'label': _( 'Smilie theme' ),
                       'values': self.config.getSmilieThemes(),
                       'config': None,
                       'sample': self.getSmilieSample,
                       })
        
        fields.append({'name': 'conversationLayout',
                       'label': _( 'Conversation Layout' ),
                       'values': self.controller.getConversationLayoutManager().listAvailableThemes(),
                       'config': self.configConversationLayout,
                       'sample': None,
                       })
        
        self.fields = fields
        
        table = gtk.Table(len(fields), 3, False)
        table.set_row_spacings( 8 )
        table.set_col_spacings( 8 )
        row = 0
        for field in fields:
            label = gtk.Label( field['label'] )
            label.set_alignment( 0.0, 0.5 )
            label.set_use_underline( True )
            table.attach( label, 0,1, row,row+1 )
            
            comboModel = gtk.ListStore( str, gtk.gdk.Pixbuf )
            combo = gtk.ComboBox( comboModel )
            cellRenderer = gtk.CellRendererText()
            sampleImage = gtk.CellRendererPixbuf()
            sampleImage.set_property( 'xalign', 1.0 )

            combo.pack_start( cellRenderer, False )
            combo.add_attribute( cellRenderer, 'markup', 0)
            combo.pack_start( sampleImage, True )
            combo.add_attribute( sampleImage, 'pixbuf', 1)

            self.fields[row]['combo'] = combo
            label.set_mnemonic_widget( combo )
            combo.connect( 'changed', self.save, field['name'] )
            
            default = self.config.getUserValue(field['name'])
            done = False
            count = 0
            for current in field['values']:
                sample = None
                if field['sample']:
                    sample = field['sample'](field['name'], current)
                if not done:
                    if current == default: done = True
                    else: count += 1
                comboModel.append( [ current, sample ] )
            combo.set_active( count )
            table.attach( combo, 1,2, row,row+1 )
            
            button = gtk.Button( _( 'Edit...' ) )
            
            if field['config'] != None:
                table.attach( button, 2,3, row,row+1 )
                button.connect( 'clicked', field['config'] )
            #else:
            #    button.set_sensitive( False )
            
            row += 1
        self.pack_start(table, False, False, 3)
        self.show_all()
        
    # Config dialogs
    def configConversationLayout( self, *args ):
        dialog = gtk.Dialog( _( "Conversation Layout" ),
                             None,
                             gtk.DIALOG_MODAL,
                             (gtk.STOCK_OK, gtk.RESPONSE_ACCEPT) )
        conversationLayoutTab = ConversationLayoutTab(self.controller)
        dialog.vbox.add(conversationLayoutTab)
        dialog.show_all()
        response = dialog.run()
        dialog.destroy()
        # do something with the changes here
    
    # Samples
    def getThemeSample( self, label, item ):
        tempTheme = Theme.Theme( None, item )
        icon = tempTheme.getImage("icon")
        del tempTheme
        return icon
    
    def getSmilieSample( self, label, item ):
        tempTheme = Theme.SmilieTheme( None, item )
        try:
            icon = tempTheme.getSmiley(":D").get_static_image()
            del tempTheme
            return icon
        except:
            pass
    
    # Other functions
    def save( self, widget = None, name = 'all' ):
        self.config.setUserValue('parseSmilies', str(int(self.parseSmilies.get_active())) )
        self.config.setUserValue('smallIcons', str(int(self.smallIcons.get_active())) )
        try:
            for field in self.fields:
                self.config.setUserValue( field['name'], field['combo'].get_active_text() )
        except:
            pass
        
        if name in ('all','conversationLayout'): 
            layout = self.config.getUserValue( 'conversationLayout' )
            self.controller.getConversationLayoutManager().load(layout)
        
        self.controller.themeChanged()

#TODO: this is not a tab
class ConversationLayoutTab( gtk.VBox ):
    '''This class represents the conversation layout config'''
    
    def __init__( self, controller ):
        '''Constructor'''
        
        gtk.VBox.__init__( self )
        
        self.controller = controller
        self.config = controller.getConfig()
        
        self.set_spacing( 4 )
        self.set_border_width( 12 ) 
        
        # Preview field
        self.labelName = gtk.Label()
        self.labelName.set_alignment( 0.0, 0.5 )
        self.labelDescription = gtk.Label()
        self.labelDescription.set_alignment( 0.0, 0.5 )
        self.labelAuthor = gtk.Label()
        self.labelAuthor.set_alignment( 0.0, 0.5 )
        self.labelWebsite = gtk.Label()
        self.labelWebsite.set_alignment( 0.0, 0.5 )
        self.previewScroll = gtk.ScrolledWindow()
        self.previewScroll.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        self.preview = htmltextview.HtmlTextView()
        self.previewScroll.add(self.preview)
        self.pack_start(self.labelName, False, False, 3)
        self.pack_start(self.labelDescription, False, False, 3)
        self.pack_start(self.labelAuthor, False, False, 3)
        self.pack_start(self.labelWebsite, False, False, 3)
        self.pack_start(self.previewScroll, True, True, 3)
        self.updateInfosFrame()

        self.show_all()
        
    def updateInfosFrame( self ):
        "Updates the htmltextview preview."
        
        self.labelName.set_markup('<b>' + _('Name:') + '</b> ' + self.controller.getConversationLayoutManager().getName())
        self.labelDescription.set_markup('<b>' + _('Description:') + '</b> ' + self.controller.getConversationLayoutManager().getDescription())
        self.labelAuthor.set_markup('<b>' + _('Author:') + '</b> ' + self.controller.getConversationLayoutManager().getAuthor())
        self.labelWebsite.set_markup('<b>' + _('Website:') + '</b> ' + self.controller.getConversationLayoutManager().getWebsite())
        
        textbuff = self.preview.get_buffer()
        start, end = textbuff.get_bounds()
        textbuff.delete(start,end)
 
        html = self.controller.getConversationLayoutManager().getPreview()       
        self.preview.display_html("<body xmlns='http://www.w3.org/1999/xhtml'>%s</body>" % html)

class ConversationPanel( gtk.VBox ):
    '''This class represent the panel to configure the conversation inside the interface tab'''
        
    def __init__( self, controller, config ):
        '''
        Contructor
        '''
        
        gtk.VBox.__init__( self )      

        self.controller = controller
        self.config = config
        
        self.set_spacing( 6 )
        self.set_border_width( 12 )

        #windows check button
        self.windows = gtk.CheckButton( _( 'Use multiple windows' ) )
        self.windows.set_active( bool( int( self.config.getUserValue( 'windows' ))))

        self.parsePlusMarkup = gtk.CheckButton(_('_Display MSN+ colors' ))
        self.parsePlusMarkup.set_active( bool( int( self.config.getUserValue( 'parsePlusMarkup' ) ) ) )

        self.showHeader = gtk.CheckButton(_('Show conversation _header' ))
        self.showHeader.set_active( bool( int( self.config.getUserValue( 'showHeader' ) ) ) )
        self.showToolbar = gtk.CheckButton(_('Show conversation _toolbar' ))
        self.showToolbar.set_active( bool( int( self.config.getUserValue( 'showToolbar' ) ) ) )
        self.showOwnAvatar = gtk.CheckButton(_('Show own _avatar' ))
        self.showOwnAvatar.set_active( bool( int( self.config.getUserValue( 'showMyAvatar' ) ) ) )
        self.showOthersAvatar = gtk.CheckButton(_('Show _friend _avatar' ))
        self.showOthersAvatar.set_active( bool( int( self.config.getUserValue( 'avatarDual' ) ) ) )
        
        self.showHeader.connect( 'toggled', self.onToggled, 'showHeader' )
        self.showToolbar.connect( 'toggled', self.onToggled, 'showToolbar' )
        self.showOwnAvatar.connect( 'toggled', self.onToggled, 'showMyAvatar' )
        self.showOthersAvatar.connect( 'toggled', self.onToggled, 'avatarDual' )
                
        self.pack_start(self.windows, expand=True, fill=True, padding=0)
        self.pack_start(self.parsePlusMarkup, expand=True, fill=True, padding=0)
        self.pack_start(self.showHeader, expand=True, fill=True, padding=0)
        self.pack_start(self.showToolbar, expand=True, fill=True, padding=0)
        self.pack_start(self.showOwnAvatar, expand=True, fill=True, padding=0)
        self.pack_start(self.showOthersAvatar, expand=True, fill=True, padding=0)
        
        self.show_all()

    def onToggled( self, radio, option ):
        self.config.setUserValue(option, str(int(radio.get_active())) )
        self.controller.emit( 'preferences-changed' )
        
    def save( self ):
        '''save the actual setting'''
        
        self.config.setUserValue('windows', str(int( self.windows.get_active() )) )
        self.config.setUserValue('parsePlusMarkup', str(int(self.parsePlusMarkup.get_active())) )

        self.config.setUserValue('showHeader', str(int(self.showHeader.get_active())) )
        self.config.setUserValue('showToolbar', str(int(self.showToolbar.get_active())) )
        self.config.setUserValue('showMyAvatar', str(int(self.showOwnAvatar.get_active())) )
        self.config.setUserValue('avatarDual', str(int(self.showOthersAvatar.get_active())) )
        
class ProxySettings( gtk.VBox ):
    '''
    This class represent the panel with the proxy variables in the config file
    '''

    def __init__( self, config ):
        '''
        Contructor
        '''
        
        gtk.VBox.__init__( self )   

        self.config = config
        
        self.set_spacing( 6 )
        self.set_border_width( 12 )

        self.host = gtk.Entry()
        self.host.set_text( self.config.getGlobalValue( 'proxyHost' ) )
        self.port = gtk.Entry()
        self.port.set_text( self.config.getGlobalValue( 'proxyPort' ) )

        table = gtk.Table( 2, 2 )
        table.set_row_spacings( 6 )
        table.set_col_spacings( 12 )
        
        host = gtk.Label( _( '_Host:' ) )
        host.set_alignment( 0.0, 0.5 )
        host.set_use_underline( True )
        host.set_mnemonic_widget( self.host )
        port = gtk.Label( _( '_Port:' ) )
        port.set_alignment( 0.0, 0.5 )
        port.set_use_underline( True )
        port.set_mnemonic_widget( self.port )
        table.attach( host, 0, 1, 0, 1 )
        table.attach( port , 0, 1, 1, 2 )
        
        table.attach( self.host, 1, 2, 0, 1 )
        table.attach( self.port, 1, 2, 1, 2 )
        
        self.pack_start( table )

        self.show_all()
    
    def save( self ):
        '''
        save the actual setting
        '''
        
        host = self.host.get_text()
        if host.startswith( "http://" ): host = host.split( "http://" )[1]
        if host.find( "/" ) != -1: host = host.split( "/" )[0]

        self.config.setGlobalValue( 'proxyHost', host )
        self.config.setGlobalValue( 'proxyPort', self.port.get_text() )
    
    def enable( self ):
        '''
        enable the entries for editing
        '''
        
        self.host.set_editable( True )
        self.port.set_editable( True )
        self.set_sensitive( True )

    def disable( self ):
        '''
        disable the entries for editing
        '''
        
        self.host.set_editable( False )
        self.port.set_editable( False )
        self.set_sensitive( False )


if __name__ == "__main__":
    print "sorry, this is totally broken"
