# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import gtk
import Theme
from Theme import resizePixbuf
from gobject import timeout_add, source_remove
from ImageFileChooser import ImageFileChooserDialog

class SmilieWindow( gtk.Window ):
    """
    this is the window that opens when you press the smilie button on the
    conversation window
    """

    def __init__( self, controller , callback, parent=None ):

        gtk.Window.__init__( self )
        self.set_decorated( False )
        self.set_type_hint( gtk.gdk.WINDOW_TYPE_HINT_DIALOG )
#        self.set_modal( True ) # causes weird enter/leave events
        if parent:
            self.set_transient_for( parent )
        self.set_position( gtk.WIN_POS_MOUSE )
        self.set_resizable( False )
        self.set_title( _("Smilies") )
        self.set_border_width( 2 )

        self.callback = callback
        
        self.vbox = gtk.VBox()
        table1 = gtk.Table(10)
        
        self.theme = controller.getTheme()
        
        self.customEmoticons = controller.customEmoticons
        self.config = controller.config
        
        self.closed = True

        x = 0
        y = 0

        for i in self.theme.getSingleSmileysList():
            if x == 10:
                x = 0
                y += 1
            try:
                button = gtk.Button()
                #button.set_relief(gtk.RELIEF_NONE)
                smilieImage = gtk.Image()
                if self.theme.getSmiley( i ).is_static_image():
                    smilieImage.set_from_pixbuf( self.theme.getSmiley( i ).get_static_image() )
                else:
                    smilieImage.set_from_animation( self.theme.getSmiley( i ) )
                button.set_image( smilieImage )
                button.connect( "clicked" , self.clicked , i )
                button.connect( 'enter-notify-event' , self.on_enter_notify_event )
                table1.attach( button , x , x+1 , y , y+1 )
            except:
                x -= 1
            x += 1

        ceLabel = gtk.Label('Custom Emoticons')
        self.vbox.pack_start( table1 )
        self.vbox.pack_start( ceLabel )
        self.fillCETable()
        
        self.add( self.vbox )
        self.vbox.show_all()
        self.connect( 'delete-event' , self.on_delete_event )
        self.connect( 'leave-notify-event' , self.on_leave_notify_event )
        self.connect( 'enter-notify-event' , self.on_enter_notify_event )

        self.tag = None

    def on_leave_notify_event(self, *args):
        if not self.tag and not self.closed:
            self.tag = timeout_add( 500, self.hide )

    def on_enter_notify_event(self, *args):
        if ( self.tag ):
            source_remove( self.tag )
            self.tag = None

    def on_delete_event( self , *args ):
        self.hide()
        self.closed = True
        return True

    def clicked( self , button , smilie ):
        self.hide()
        self.callback( smilie )
        
    def clickedAdd( self, button, *args ):
        self.hide()

    def show( self ):
        gtk.Window.show( self )
        self.tag = None
        self.closed = False

    def hide( self ):
        gtk.Window.hide( self )
        if ( self.tag ):
            source_remove( self.tag )
            self.tag = None
        self.closed = True
        
    def fillCETable( self ):
        try:
            self.vbox.remove( self.ceTable )
        except: pass
        self.ceTable = gtk.Table()
        x = 0
        y = 0
        
        for shortcut, filename in self.customEmoticons.list.iteritems():
            if x == 10:
                x = 0
                y += 1
            try:
                pixbuf = gtk.gdk.pixbuf_new_from_file( filename )
                pixbuf = resizePixbuf( pixbuf, 24, 24 )
                button = gtk.Button( )
                smilieImage = gtk.Image()
                smilieImage.set_from_pixbuf( pixbuf )
                button.set_image( smilieImage )
                button.connect( "clicked" , self.clicked, shortcut )
                button.connect( 'enter-notify-event' , self.on_enter_notify_event )
                self.ceTable.attach( button, x , x+1 , y , y+1 )
            except:
                print 'Error in smiley', shortcut, filename
                x -= 1
            x += 1
            
        button = gtk.Button( 'add' )
        button.connect( 'clicked', self.addClicked )
        button.connect( 'enter-notify-event' , self.on_enter_notify_event )
        self.ceTable.attach( button, x , x+1 , y , y+1 )
        
        self.vbox.pack_start( self.ceTable )
        self.vbox.show_all()
        
    def addClicked( self, button ):
        self.hide()
        emo = AddCustomEmoticon( self, self.config ).run()
        if emo != None:
            self.customEmoticons.create( emo[0], emo[1] )
            self.fillCETable()


class AddCustomEmoticon( gtk.Dialog ):
    '''
    a dialog to add a custom emoticon
    '''    
    
    def __init__( self , parent, config ):
        gtk.Dialog.__init__( self , _("Add a Custom Emoticon"), parent , \
                gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT , \
                (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT, \
                gtk.STOCK_ADD, gtk.RESPONSE_ACCEPT) )

        self.set_geometry_hints( self, 400 )
        
        self.config = config

        self.set_resizable( False )
        self.set_has_separator( False )
        self.set_border_width( 6 )

        self.set_default_response( gtk.RESPONSE_ACCEPT )
        
        self.hbox = gtk.HBox()
        
        self.image = gtk.Image()
        
        label1 = gtk.Label( _('shortcut:') )
        self.shortcut = gtk.Entry()
        self.chooseButton = gtk.Button( _("Choose") )
        
        
        self.hbox.pack_start( self.image )
        
        self.hbox.pack_start( label1 )
        self.hbox.pack_start( self.shortcut )
        self.hbox.pack_start( self.chooseButton )
        
        self.chooseButton.connect( 'clicked', self.chooseClicked )
        
        self.vbox.pack_start( self.hbox )
        self.vbox.show_all()


    def run( self ):
        "runs the dialog and return the string it doesnt make any control"
        
        option = gtk.Dialog.run( self )
        if option == gtk.RESPONSE_ACCEPT:
            shortcut = self.shortcut.get_text()
            if shortcut == None:
                return None
            if self.filename == None:
                return None
            self.destroy()
            return ( shortcut, self.filename )
        else:
            self.destroy()
            return None
            
    def chooseClicked( self, button ):
        self.filename = ImageFileChooserDialog( self.config ).run()[1]
        if self.filename != None:
            pixbuf = gtk.gdk.pixbuf_new_from_file( self.filename )
            pixbuf = resizePixbuf( pixbuf, 24,24 )
            self.image.set_from_pixbuf( pixbuf )
