# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import gtk

import UserMenu
import GroupMenu
import TreeViewTooltips

import pango
import re
import md5 # TODO: another way to get a pixbuf identifier?
from gobject import PARAM_READWRITE
from msnPlusMarkup import msnPlusMarkup
from msnPlusMarkup import getCustomStatus

from Parser import UserListDataType, IMG_TAG_OTHER_RE, extractImgAlt
from emesenecommon import unescape
from emesenecommon import OS_NAME
I_WANT_TO_SEE_FANCY_SMILEYS_IN_USERLIST = True

reverse_status = {
    'NLN':        0,
    'AWY':        1,
    'BSY':        2,
    'BRB':        3,
    'PHN':        4,
    'LUN':        5,
    'HDN':        6,
    'IDL':        7,
    'FLN':        8,
}
if I_WANT_TO_SEE_FANCY_SMILEYS_IN_USERLIST:
    NOPARSE_TAG = '<noparse />'
else:
    NOPARSE_TAG = ''

class UserList( gtk.TreeView ):
    '''This class represent the user list in the main window'''

    def __init__( self , controller ):
        '''Constructor'''

        gtk.TreeView.__init__( self )
        self.scroll = None
        self.controller = controller
        self.config = self.controller.getConfig()
        self.contactManager = self.controller.getContactManager()
        self.theme = self.controller.getTheme()
        self.groupState = {}
        self.dictEmailIterator = {}
        self.cacheDict = {}

        self.unifiedParser = controller.getUnifiedParser()
        
        if (I_WANT_TO_SEE_FANCY_SMILEYS_IN_USERLIST):
            self.cellRendererText = SmileyRenderer( self.config, self.cacheDict )
        else:
            self.cellRendererText = gtk.CellRendererText()
        
        self.cellRendererPixbuf = gtk.CellRendererPixbuf()
        self.cellRendererPixbufSpace = gtk.CellRendererPixbuf()
        
        nickColor = self.config.getUserValue('nickColor')
        self.cellRendererText.set_property("foreground-gdk",\
            gtk.gdk.color_parse(nickColor))
        self.cellRendererText.set_property( 'xalign', 0.0 )
        self.cellRendererPixbuf.set_property( 'xalign', 0.0 )
        self.cellRendererPixbufSpace.set_property( 'xalign', 0.0 )
        self.cellRendererPixbufSpace.set_property( 'visible', False )
        self.cellRendererPixbuf.set_property( 'visible', False )

        self.columnType = [ gtk.gdk.Pixbuf, gtk.gdk.Pixbuf, str, str, str, bool ]
        column = gtk.TreeViewColumn()
        column.set_title('Icons & Text')
        column.set_expand( True )
        expColumn = gtk.TreeViewColumn()
        expColumn.set_max_width(16)       
        self.append_column(expColumn)
        self.append_column( column )
        self.set_expander_column(expColumn)

        column.pack_start(self.cellRendererPixbufSpace, False)
        column.pack_start(self.cellRendererPixbuf, False)
        column.pack_start(self.cellRendererText )
        column.add_attribute(self.cellRendererPixbuf, 'pixbuf', 1)
        column.add_attribute(self.cellRendererPixbufSpace, 'pixbuf', 0)
        column.add_attribute(self.cellRendererPixbuf, 'visible', 5)
        column.add_attribute(self.cellRendererPixbufSpace, 'visible', 5)
        #column.add_attribute(self.cellRendererText, 'text', 2)
        column.set_attributes(self.cellRendererText, markup=2)
        self.cellRendererText.set_property( 'ellipsize', pango.ELLIPSIZE_END )
        self.filterText = ''

        self.treeStore = gtk.TreeStore( *self.columnType )
        self.treeModelFilter = self.treeStore.filter_new()
        self.treeModelFilter.set_visible_func( self.visible_func )
        self.set_model( self.treeModelFilter )
        self.set_search_column( 2 )
        self.set_headers_visible( False )
        self.set_reorderable( True )

        self.connect( "row_activated" , self.row_activated )
        self.connect( "button-press-event" , self.button_press_event )
        self.connect( "row-expanded" , self.row_expanded )
        self.connect( "row-collapsed" , self.row_collapsed)

        #self.connect( "motion-notify-event", self.mne )
        #self.connect( "leave-notify-event" , self.clearStatus )
        #self.set_events( gtk.gdk.POINTER_MOTION_MASK )

        self.showByNick = bool(int(self.controller.config.getUserValue("showByNick")))
        self.showOffline = bool(int(self.controller.config.getUserValue("showOffline")))
        self.showByStatus = bool(int(self.controller.config.getUserValue("showByStatus")))
        self.showEmptyGroups = bool(int(self.controller.config.getUserValue("showEmptyGroups")))
        self.avatarsInUserList = bool(int(self.controller.config.getUserValue("avatarsInUserList")))

        self.tooltips = TreeViewTooltips.TreeViewTooltips(self, info_col=2, mail_col=3, is_user_col=5)
        
        self.plusMarkup = msnPlusMarkup()

        self.show()
        
        self.controller.connect( 'preferences-changed', self.preferencesChanged )
        self.controller.getMSN().connect( 'display-picture-changed', self.displayPictureChanged )

    def displayPictureChanged(self, msnp, email):
        if self.avatarsInUserList:
            self.refresh()

    def preferencesChanged( self, controller ):
        if self.avatarsInUserList != bool(int(self.controller.config.getUserValue("avatarsInUserList"))):
            self.refresh()
        del self.cacheDict
        self.cacheDict = {}

    def visible_func( self, model, iter):
        email = str(model.get( iter, 3 )[ 0 ])
        if self.filterText == "":
            return True
        elif email.lower().find( self.filterText.lower() ) != -1\
        or self.contactManager.getContactNick( email ).lower().find( self.filterText.lower() ) != -1\
        or self.contactManager.getContactAlias( email ).lower().find( self.filterText.lower() ) != -1:
            return True
        elif str(model.get( iter, 4 )[ 0 ]) == 'group':
            return True

        return False

    def setFilterText( self, text ):
        self.filterText = text
        self.treeModelFilter.refilter()

    def add( self , values , iterator = None ):
        return self.treeStore.append( iterator, values )

    def setShowByNick( self , value ):
        self.showByNick = value

    def getShowByNick( self ):
        return self.showByNick

    def setShowOffline( self , value ):
        self.showOffline = value

    def getShowOffline( self ):
        return self.showOffline

    def setShowEmptyGroups( self , value ):
        self.showEmptyGroups = value

    def getShowEmptyGroups( self ):
        return self.showEmptyGroups

    def setShowByStatus( self , value ):
        self.showByStatus = value

    def getShowByStatus( self ):
        return self.showByStatus

    def orderByStatus( self, x ):
    	if x in self.controller.status_ordered[ 1 ]:
    		return self.controller.status_ordered[ 1 ].index(x)
    	else:
            # this is only supposed to happen when not showByStatus
            ret = x.lower()
            if ret == 'no group':
                return 'zzzzzzzzz'    # TODO sth better
            else:
                return ret

    def refresh( self ):
        self.treeStore.clear()
        
        nickColor = self.config.getUserValue('nickColor')
        self.cellRendererText.set_property("foreground-gdk", gtk.gdk.color_parse(nickColor))

        self.showByNick = bool(int(self.controller.config.getUserValue("showByNick")))
        self.showOffline = bool(int(self.controller.config.getUserValue("showOffline")))
        self.showByStatus = bool(int(self.controller.config.getUserValue("showByStatus")))
        self.showEmptyGroups = bool(int(self.controller.config.getUserValue( "showEmptyGroups" )))
        self.groupByStatus = bool(int(self.controller.config.getUserValue( "orderByStatus" )))
        self.avatarsInUserList = bool(int(self.controller.config.getUserValue("avatarsInUserList")))
        
        contactListDict = self.contactManager.\
                        getContactList( self.showOffline, self.showEmptyGroups, self.groupByStatus )

        nodes = contactListDict.keys()
        nodes.sort( key=lambda x: self.orderByStatus( x ) )
        
        for i in nodes:
            iterator = None
            #a dict to count users size per status
            statusDictCount = {}
            iterator = self.addListNode(i, len( contactListDict[ i ] ) )

            for j in self.orderContacts(contactListDict[i]):
                self.addContactToList(j, iterator)
            
            if self.groupByStatus:
                # if the key isnt in the dict we add it to True (expanded) except
                # for offline status
                if not self.groupState.has_key(i):
                    if i == "offline":
                        self.groupState[i] = False
                    else:
                        self.groupState[i] = True
                        self.expand_row(self.treeStore.get_path(iterator), True)
                else:
                    if self.groupState[i]:
                        self.expand_row(self.treeStore.get_path(iterator), True)
            else:
                #if the key isnt in the dict we add it as expanded
                if not self.groupState.has_key(i) or self.groupState[i]:
                    self.groupState[i] = True
                    self.expand_row(self.treeStore.get_path(iterator) , True)

    def addListNode(self, node, size=0 ):
        """this method a note to user list: status if showByStatus is True, group name otherwise"""

        if self.groupByStatus:
            if node == 'offline':
                #: in plural
                groupName = _( 'Offline' )
            else:
                #: in plural
                groupName = _( 'Online' )
                
            iterator = self.add( [ None, None, NOPARSE_TAG + '<b>' + groupName  \
                                  + ' (' + str( size ) + ')' + '</b>',\
                                  node, "group", False ], None )
                
        else:

            if node == 'No group':
                userRelation = self.contactManager.getOnOffUsersRelationByGroup('No group')
                userRelationStr = " (" +str(userRelation[0])+ "/" + str(userRelation[1])+ ")"

                iterator = self.add( [ None, None, NOPARSE_TAG + '<b>' + _('No group') + \
                    userRelationStr  + '</b>', node, "no group", False ], None )
            else:
                userRelation = self.contactManager.getOnOffUsersRelationByGroup(node)
                userRelationStr = " (" +str(userRelation[0])+ "/" + str(userRelation[1])+ ")"
                
                iterator = self.add( [ None, None, NOPARSE_TAG + '<b>' + node + \
                    userRelationStr + '</b>', node, "group", False ], None )
                
        return iterator
    
    def addContactToList(self, contactEmail, iterator):
        """ This method add a contact to user list, according to node (iterator)"""

        if self.contactManager.getContactIsAllowed(contactEmail) or\
            self.contactManager.getContactIsBlocked(contactEmail):
                
            small = bool( int( self.config.getUserValue('smallIcons','0') ) )
            personalMessageColor = self.config.getUserValue('personalMessageColor')
            parsePlusMarkup = int(self.controller.config.getUserValue('parsePlusMarkup', '0'))
            canParseSmileys = int(self.config.getUserValue("parseSmilies","1"))==1
            contactStatus = self.controller.getContactStatus(contactEmail)
            
            space = None
            self.dictEmailIterator[contactEmail] = iterator
            
            if self.contactManager.getContactHasSpace(contactEmail):
                space = self.theme.getImage('space')
            
            if not self.contactManager.getContact(contactEmail).reverse:
                space = self.theme.getImage('reverse')

            if self.contactManager.getContactIsBlocked(contactEmail):
                space = self.theme.getImage('status-blocked')

            if contactStatus == 'FLN' and\
                    self.contactManager.getContactHasMobile(contactEmail):
                status = self.theme.getImage('mobile')
            elif self.avatarsInUserList and\
                 self.theme.hasUserDisplayPicture( contactEmail ):
                if small:
                    dpsize = 16
                else:
                    dpsize = 32
                status = self.theme.getUserDisplayPicture( contactEmail, dpsize, dpsize )
            else:
                status = self.theme.statusToPixbuf(\
                    self.contactManager.getContactStatus(contactEmail) )
            
            displayName, displayStatus = getCustomStatus( self.contactManager.getContactNameToDisplay(contactEmail) )
            if not displayStatus and contactStatus != 'NLN':
                displayStatus = self.controller.status_ordered[2][reverse_status[contactStatus]]
            personalMessage = self.contactManager.getContactPersonalMessage(contactEmail, True)
            
            displayName = self.unifiedParser.getParser( unescape( displayName ), UserListDataType ).get()
            personalMessage = self.unifiedParser.getParser( unescape( personalMessage ), UserListDataType ).get()
            
            if I_WANT_TO_SEE_FANCY_SMILEYS_IN_USERLIST != True or canParseSmileys != True:
                displayName = extractImgAlt( displayName )
                personalMessage = extractImgAlt( personalMessage )
            
            if personalMessage != '':
                if contactStatus == 'FLN' or not parsePlusMarkup:
                    personalMessage = self.plusMarkup.removeMarkup(personalMessage)
                else:
                    personalMessage = self.plusMarkup.replaceMarkup(personalMessage, False)
            
            if self.showByNick:
                if contactStatus == 'FLN' or not parsePlusMarkup:
                    name = self.plusMarkup.removeMarkup(displayName)
                else:
                    name = self.plusMarkup.replaceMarkup(displayName, False)
            else:
                name = contactEmail

            # all this mess for two '-' :D
            if small:
                if displayStatus != '':
                    displayStatus = '(%s)' % displayStatus
                
                template = '%s - <span foreground="%s">%s %s</span>'
            else:
                template = '%s\n<span size="small" foreground="%s">%s %s</span>'
                
            if personalMessage != '' and displayStatus != '':
                    displayStatus += ' -'
            
            text = template % (name, personalMessageColor, displayStatus, personalMessage)
            self.add([space, status, text, contactEmail, "user", True], iterator )
    
    def mne( self , widget , e ):
        _list = widget.get_path_at_pos( int(e.x), int(e.y) )

        if _list != None:
            iterator = widget.get_model().get_iter( _list[ 0 ] )
            value = widget.get_model().get_value( iterator , 3 )

            self.controller.showUserData( value )

    def clearStatus( self , *args ):
        self.controller.showUserData( "" )

    def row_activated( self , treeview , path , view_column  ):
        iterator = treeview.get_model().get_iter( path )

        mail = treeview.get_model().get_value( iterator , 3 )

        typeOfRow = treeview.get_model().get_value( iterator , 4 )

        if typeOfRow == "user":
            self.controller.newConversation( None, mail, None, True )

    def button_press_event( self , treeview , event ):
        
        self.tooltips.hideTooltip()
                
        if event.button == 3:

            _list = treeview.get_path_at_pos( int(event.x), int(event.y) )

            if _list == None:
                return

            iterator = treeview.get_model().get_iter( _list[ 0 ] )
            rowType = treeview.get_model().get_value( iterator , 4 )
            time = event.time

            if rowType == 'group':
                if not bool(int(self.controller.config.getUserValue( "orderByStatus" ))):
                    value = treeview.get_model().get_value( iterator , 3 )
                    value = unicode( value )
                    menu = GroupMenu.GroupMenu( self.controller , value )
                    menu.popup( None, None, None, event.button, time )
            elif rowType == 'user':
                value = treeview.get_model().get_value( iterator , 3 )

                groupIter = treeview.get_model().iter_parent( iterator )
                groupName = treeview.get_model().get_value( groupIter , 3 )
                groupName = unicode( groupName )
                
                menu = UserMenu.UserMenu( self.controller , value, groupName )
                menu.popup( None, None, None, event.button, time )

    def row_expanded(self,treeview, iterator, path):
        """
        Callback for a node (usergroup) when expanded, it adds the node
        path to a list of expanded nodes if this one is not allready.
        """

        value = treeview.get_model().get_value( iterator , 3 )
        self.groupState[ value ] = True

    def row_collapsed(self,treeview, iterator, path):
        """
        Callback for a node (usergroup) when collapsed, it removes
        a node path from a list of expanded nodes if present.
        """

        value = treeview.get_model().get_value( iterator , 3 )
        self.groupState[ value ] = False
        
    def getMenuData( self ):
        '''Returns useful data for building menus in the format
        ( <selected item type>, [ [ <item name> ], <item's group> ] )'''
        try:
            model = self.treeStore
            iterator = self.get_selection().get_selected()[ 1 ]

            if not model.iter_is_valid(iterator):
                print "iter is invalid"
                return ( '', )
                
            l = model.get( iterator , 4 )
            rowType = l[0]

            if rowType == 'group':
                if not bool(int(self.controller.config.getUserValue( "orderByStatus" ))):
                    value = unicode( model.get_value( iterator , 3 ) )
                    return ( 'group', value )
                else:
                    return ( '', )
            elif rowType == 'user':
                value = model.get_value( iterator , 3 )
    
                groupIter = model.iter_parent( iterator )
                groupName = unicode( model.get_value( groupIter , 3 ) )
    
                return ( 'user', value, groupName )
            else:
                return ( '', )
        except:
            return ( '', )

    def getUserSelected( self ):
        try:
            l = self.treeStore.get( self.get_selection().get_selected()[ 1 ] ,\
                                     3, 4 )
            if l[ 1 ] == "user":
                retval = l[ 0 ]
            else:
                retval = ""
        except:
            retval = ""

        return retval

    def getGroupSelected( self ):
        try:
            l = self.treeStore.get( self.get_selection().get_selected()[ 1 ] ,\
                 2 , 3 , 4 )
            if l[ 2 ] == "group":
                retval = l[ 1 ]
            else:
                retval = ""
        except:
            retval = ""

        return retval

    def getGroupState( self ):
        return self.groupState

    def setGroupState( self , groupState ):
        self.groupState = groupState

    def orderContacts( self, contacts ):
        '''receive a dict of contacts and order acording to thei status
        return a list of keys'''
        contactByStatus = {}
        returnList = []
        
        if self.showByStatus:
            statusOrdered = self.controller.status_ordered[0]
        else:
            statusOrdered = [ 'noStatus', 'FLN' ]
        
        for i in statusOrdered:
            contactByStatus[i] = []
        
        for i in contacts.iterkeys():
            contactStatus = self.contactManager.getContactStatus(i)
            if not self.showByStatus and not contactStatus == 'FLN':
                contactStatus = 'noStatus'
            contactByStatus[contactStatus].append(i)

        for i in statusOrdered:
            if self.showByNick and len(contactByStatus[i]) > 0:
                # order contacts alphabetically (case ins) by their display name
                contactByStatus[i].sort( key=lambda x: str.lower(\
                            str( self.contactManager.getContactNick ( x ) ) ) )
            else:
                contactByStatus[i].sort()

            returnList += contactByStatus[i]

        return returnList

class SmileyRenderer(gtk.GenericCellRenderer):
    """Cellrenderer with smiley support. Based on 
    http://mail.gnome.org/archives/gtk-perl-list/2004-April/pl00001.pl
    and code.google.com, because there isn't any documentation.
    """

    __gproperties__ = {
        'markup':
            (str, 'Text', 'The text to show', '', PARAM_READWRITE),
        'foreground-gdk':
            (str, 'Foreground color', '', '', PARAM_READWRITE),
        'ellipsize':
            #(pango.EllipsizeMode, 'ellipsize', '', '', PARAM_READWRITE),
            (str, 'ellipsize', '', '', PARAM_READWRITE),
    }


    def __init__( self, config, cacheDict ):
        self.__gobject_init__()

        self.config = config

        self.debug_string = "[DEBUG SMILEYS]" # contacts with this string are debugged
        self.smiley_margin = 2
        self.xpad = 2
        self.ypad = 1
        
        self.lastTextHeight = 0
        self.cache = cacheDict

    def canParseSmileys(self):
        '''Returns True if user has smileys enabled and NOPARSE_TAG isn't found'''
        if self.markup.find(NOPARSE_TAG)!=-1:
            return False
        canParse = int(self.config.getUserValue("parseSmilies","1"))==1
        return canParse

    def canDebug(self):
        '''Return True if the user has debug_string in the nickname'''

        return self.markup.find(self.debug_string) != -1

    def getLayouts(self, markup, parseSmileys):
        '''Convert markup with smileys into a list with fixed markup'''

        lines = markup.replace(NOPARSE_TAG,'').split("\n")
        def lineToList(x):
            if parseSmileys:
                list = self.parseImgTags( x )
                list = FixMarkup(list).List
                if self.canDebug(): print "+++", list
                return list
            return [x]
        ret= [lineToList(x) for x in lines]
        if self.canDebug(): print "***", ret
        return ret

    def parseImgTags( self, data ):
        list = []
        def sub( tag ):
            group = tag.groupdict()
            if group['src'] != None:
                uri = group['src'].replace('file:///', '')
                
                if OS_NAME == 'posix':
                    uri = '/' + uri
                    
                list.append( gtk.gdk.pixbuf_new_from_file( uri ) )
            elif group['other'] != None:
                i=len(list)-1
                if i >-1 and type( list[i] ) == type('str'):
                    list[i] += group['other']
                else:
                    list.append( group['other'] )
                
        IMG_TAG_OTHER_RE.sub(sub, data)
        return list

    def renderPath(self, mode, cell_area, widget, win=None, background_area=None, expose_area=None, flags=None):
        '''Draw and/or get the size, depending of the mode.
        Calculates positions based on width and height'''
        if mode not in ['render','size']: return
        render = (mode=='render')

        start_xy = (0,0)
        if render: start_xy = (cell_area.x + self.xpad, cell_area.y + self.ypad)

        # start_x is used in the for line in lines
        y = start_xy[1]  # start
        size_x = highersize_x = 0  # per-item
        size_y = highersize_y = 0  # per-line
        total_y = 0  # end

        parseSmileys = self.canParseSmileys()
        lines = self.getLayouts(self.markup, parseSmileys)

        for line in lines:
            if render and self.canDebug(): print "render line:",line
            
            # init line
            x = start_xy[0]
            y += highersize_y
            size_y = highersize_y = 0
            
            for item in line:
                if render and self.canDebug(): print item

                # draw the block and/or get the size
                size_x, size_y = self.draw(not render, item, x,y, win, \
                                           widget, cell_area, flags)

                # add the width now, save the height for the next line
                x += size_x
                if size_y > highersize_y: highersize_y = size_y

            # add the total height, and set highersize_x if gt
            total_y += highersize_y
            if (x-start_xy[0]) > highersize_x:
                highersize_x = x-start_xy[0]
        width, height = highersize_x, (total_y + self.ypad)
        return width,height

    def on_render(self, win, widget, background_area, cell_area, expose_area, flags):
        '''Called by gtk to render the cell.'''

        self.renderPath('render', cell_area, widget, win, \
                        background_area, expose_area, flags)

    def on_get_size(self, widget, cell_area):
        '''Returns the size of the cellrenderer'''

        x_offset = y_offset = 0
        width, height = self.renderPath('size', cell_area, widget)

        if self.canDebug(): print "get size:", x_offset, y_offset, width, height
        # workaround for ellipsize
        return x_offset, y_offset, 0, height


    def draw(self, dummy, item, x,y, win, widget = None, cell_area = None, flags = None):
        '''minimal abstraction between pixbuf and text
        - If item is an animation, get pixbuf
        - If item is a pixbuf, call win.draw_pixbuf
        - If item is a string, call self.paint_text
        Then save start and end points. Returns size_xy'''
        if type(item) == gtk.gdk.PixbufAnimation:
            item = item.get_static_image()

        if type(item)==gtk.gdk.Pixbuf:
            if self.lastTextHeight == 0:
                tempLayout = widget.create_pango_layout("")
                self.lastTextHeight = tempLayout.get_pixel_size()[1]
            SM = self.smiley_margin
            size = self.lastTextHeight + self.smiley_margin
            item = self.getScaled(item, size)
            if not dummy: win.draw_pixbuf(None, item, 0, 0, int(x), int(y-SM), -1, -1)
            size_xy = (item.get_width() + SM, item.get_height())
        elif type(item)==str:
            if not dummy:
                size_xy = self.paint_text(item, x, y, win, widget, cell_area, flags)
            else:
                layout = widget.create_pango_layout("")
                layout.set_markup(item)
                size_xy = layout.get_pixel_size()
            self.lastTextHeight = size_xy[1]  # for smileys
        return size_xy


    def paint_text(self, markup, x, y, win, widget, cell_area, flags):
        layout = widget.create_pango_layout("")
        layout.set_markup(markup)
        if type(win) == gtk.gdk.Window:
            widget.style.paint_layout(win, flags, gtk.SHADOW_IN, 
                                      cell_area, widget, "cellrenderertext",
                                      x, y, layout)
        elif type(win) == gtk.gdk.Pixmap:
            gc = widget.style.fg_gc[gtk.STATE_NORMAL]
            win.draw_layout(gc, x, y, layout)
            
            
        size_x, size_y = layout.get_pixel_size()
        return size_x,size_y
 
    def getScaled( self, pixbuf, size ):
        '''Returns a scaled version of pixbuf (identified with repr() in the cache)'''
        hash = md5.new( str(pixbuf.get_pixels_array()) ).hexdigest()

        if hash in self.cache:
            return self.cache[hash]
        else:
            newpixbuf = pixbuf.scale_simple(size,size,gtk.gdk.INTERP_BILINEAR)
            self.cache[hash] = newpixbuf
            return newpixbuf

    def do_get_property(self, prop):
        return getattr(self, prop.name)

    def do_set_property(self, prop, value):
        setattr(self, prop.name, value)

class FixMarkup:
    "Closes tags before smileys and reopens them after"

    def __init__(self, list):
        self.regex = re.compile("<span([^>]*)>(.*)$")
        self.tagsopen = []
        self.List = [self.process(i) for i in list]

    def process(self, data):
        if type(data)==str:
            pre = ''.join(self.tagsopen)
            self.tagsopen=[]
            data = self.regex.sub(self.handler, pre+data)
            closedTags = data.count('</span>')
            if closedTags > 0:
                self.tagsopen = self.tagsopen[:-closedTags]
            data = data + '</span>' * len(self.tagsopen)
        return data

    def handler(self, data):
        group0, group1 = data.groups()
        group1 = self.regex.sub(self.handler, group1)
        self.tagsopen.append('<span%s>' % group0)
        return '<span%s>%s' % (group0,group1)

if __name__=='__main__':
    print FixMarkup(['<span 1><span 2>',0xDEADBEEF,'</span>','</span>']).List
