# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Foobar is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


import gtk

class UserMenu( gtk.Menu ):
    '''This class represent the popup menu that is displayed when you right clivk a user on
    the userList'''

    def __init__( self , controller , userName, groupName ):

        gtk.Menu.__init__( self )

        self.controller = controller
        self.contactManager = self.controller.getContactManager()

        openConversationMenuItem = self.newImageMenuItem( _( "_Open Conversation" ), gtk.STOCK_OPEN )
        self.add( openConversationMenuItem )
        openConversationMenuItem.show()


        renameUserMenuItem = self.newImageMenuItem( _( "_Set contact alias..." ), gtk.STOCK_EDIT )
        self.add( renameUserMenuItem )
        renameUserMenuItem.show()
        
        deleteAlias = gtk.MenuItem( _( '_Delete alias' ) )
        alias = self.controller.getContactAlias( userName )
        deleteAlias.set_sensitive( not alias == '')
        deleteAlias.connect( 'activate', self.on_delete_alias_activate )
        self.add( deleteAlias )
        
        self.add( gtk.SeparatorMenuItem() )

        if not self.contactManager.getContactIsBlocked( userName ):
            blockUserMenuItem = self.newImageMenuItem( _( "_Block" ), gtk.STOCK_STOP )
            self.add( blockUserMenuItem )
            blockUserMenuItem.show()
            blockUserMenuItem.connect( "activate" , self.on_block_user_activate )
        else:
            unblockUserMenuItem = self.newImageMenuItem( _( "_Unblock" ) )
            self.add( unblockUserMenuItem )
            unblockUserMenuItem.show()
            unblockUserMenuItem.connect( "activate" , self.on_unblock_user_activate )

        # uncomment this when we find a better way to know the space URL
        if self.contactManager.getContactHasSpace( userName ):
            self.add( gtk.SeparatorMenuItem() )
            spaceUserMenuItem = self.newImageMenuItem( _( "Visit MSN Space" ) , gtk.STOCK_NETWORK )
            self.add( spaceUserMenuItem )
            spaceUserMenuItem.show()
            spaceUserMenuItem.connect( "activate" , self.on_space_user_activate )

        deleteUserMenuItem = self.newImageMenuItem( _( "_Remove contact" ), gtk.STOCK_DELETE )
        self.add( deleteUserMenuItem )
        deleteUserMenuItem.show()

        self.add( gtk.SeparatorMenuItem() )

        moveMenuItem = self.newImageMenuItem( _( "M_ove to group" ), gtk.STOCK_REDO )
        moveMenu = gtk.Menu()

        for i in self.controller.getGroupNames():
            menuItem = self.newImageMenuItem ( i )
            moveMenu.add( menuItem )
            menuItem.connect( "activate" , self.moveToActivate, i )

        moveMenuItem.set_submenu( moveMenu )
        moveMenuItem.show_all()
        self.add( moveMenuItem )

        copyMenuItem = self.newImageMenuItem( _( "_Copy to group" ), gtk.STOCK_COPY )
        copyMenu = gtk.Menu()

        for i in self.controller.getGroupNames():
            menuItem = self.newImageMenuItem ( i )
            copyMenu.add( menuItem )
            menuItem.connect( "activate" , self.copyToActivate, i )

        copyMenuItem.set_submenu( copyMenu )
        copyMenuItem.show_all()
        self.add( copyMenuItem )

        removeFromGroupMenuItem = self.newImageMenuItem( _( "R_emove from group" ), gtk.STOCK_DELETE )
        self.add( removeFromGroupMenuItem )
        removeFromGroupMenuItem.show()
        removeFromGroupMenuItem.connect( "activate" , self.removeFromActivate )

        self.add( gtk.SeparatorMenuItem() )

        addUserMenuItem = self.newImageMenuItem( _( "_Add contact..." ) , gtk.STOCK_ADD )
        self.add( addUserMenuItem )
        addUserMenuItem.show()

        self.show_all()

        renameUserMenuItem.connect( "activate" , self.on_rename_user_activate )
        addUserMenuItem.connect( "activate" , self.on_add_user_activate )
        deleteUserMenuItem.connect( "activate" , self.on_delete_user_activate )
        openConversationMenuItem.connect( "activate" , self.on_open_conversation_activate )

        self.userName = userName
        self.groupName = groupName

    def newImageMenuItem( self , label , stock = None , img = None ):
        mi = gtk.ImageMenuItem( _( label ) )

        if stock != None:
            mi.set_image(gtk.image_new_from_stock(stock, gtk.ICON_SIZE_MENU))
        elif img != None:
            image = gtk.Image()
            image.set_from_pixbuf( img )
            mi.set_image( image )
        return mi

    def moveToActivate( self, menuItem, group ):
        self.controller.moveUserToGroup( self.userName, self.groupName, group )

    def copyToActivate( self, menuItem, group ):
        self.controller.addUserToGroup( self.userName, group )

    def removeFromActivate( self, menuItem ):
        self.controller.removeUserFromGroup( self.userName, self.groupName )
        
    def on_delete_alias_activate( self, *args ):
        self.controller.changeNick( '', self.userName )

    def on_add_user_activate( self , *args ):
        self.controller.addUserDialog()

    def on_delete_user_activate( self , *args ):
        self.controller.removeUser( self.userName )

    def on_rename_user_activate( self , *args ):
        self.controller.changeNickDialog( self.userName, False )

    def on_block_user_activate( self , *args ):
        self.controller.blockUser( self.userName )

    def on_unblock_user_activate( self , *args ):
        self.controller.unblockUser( self.userName )

    def on_space_user_activate( self , *args ):
        self.controller.seeSpace( self.userName )

    def on_open_conversation_activate( self, *args):
        mail = self.userName
        self.controller.newConversation( None, mail, None, True )

