# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

# FOR INFORMATION ABOUT THIS PROTOCOL GO TO:
# msnpiki.msnfanatic.com
# siebe.bot2k3.net/docs/

import os
import sys
import struct
import random
import base64
import time
import gobject

from exceptions import IndexError

import Msnobj

class P2PManager( object ):
    
    def __init__( self, switchboard ):
        self.switchboard = switchboard
        
        self.senders = []
        self.receivers = []
        
    def isNewInvite( self, message ):
        if ( message.find( 'INVITE MSNMSGR:' ) != -1 ) and \
        ( message.find( 'Content-Type: application/x-msnmsgr-sessionreqbody' ) != -1 ):
            return True
        else:
            return False
            
    def getContext( self, message ):
        return message.split( 'Context: ' )[1].split('\r')[0]
        
    def getMsnObj( self, message ):
        context = self.getContext( message )
        string = base64.b64decode( context )
        return Msnobj.createFromString( string, 0 )
                
    def process( self, message, mail ):
        response = None
        
        eraseList = []
        user = self.switchboard.msn.getUser()
        
        for i in self.senders:
            if ( self.isNewInvite( message ) ) and ( i.getFrom() == user ):
                i.setStatus( Base.STATUS_END )
                break
            if i.process( message ):
                break
        
        for i in self.senders + self.receivers:
            if i.getStatus() == Base.STATUS_END:
                eraseList.append( i )
            elif i.isMessage( message ):
                response = i.process( message )
                break
               
        for i in eraseList:
            if i in self.senders:
                self.senders.remove( i )
            elif i in self.receivers:
                self.receivers.remove( i )
            else:
                print "?"
                 
        if response == None and self.isNewInvite( message ):
            msnOM = self.switchboard.msn.msnObjectsManager
            context = self.getContext( message ) 
            msnobj = msnOM.getByContext( context )
            if msnobj != None:
                picture = Picture( msnobj.filename, Picture.OPEN )
            else:
                print 'Msnobject not found', context
                print self.getMsnObj( message )
                return
                
            print 'Sending', msnobj
            s = Sender( self.switchboard.msn.getUser(), mail, message, picture, self.switchboard )
            r = s.process( s.message )
            self.senders.append( s )
            
        elif response != None:
            for j in response:
                self.switchboard.sendMessage(j, '', 'application/x-msnmsgrp2p', 'D', 'P2P-Dest: ' + mail )
    
    def newReceiver( self, To, From, msnobj, filename, appID = 1 ):
        receiver = Receiver( To, From, msnobj, filename, self.switchboard.msn.emit, appID ) 
        self.receivers.append( receiver )
        return receiver
            
class Base( object ):
    # this status means the status we are before processing the current message
    # you dont have to pass through all the states
    STATUS_CREATED = 0 # when just created
    STATUS_INVITE = 1 # we are on invite waiting the base id
    STATUS_BASEID = 2 # we received the base id and we are waiting the 200ok
    STATUS_200OK = 3
    STATUS_200OK_ACK = 4
    STATUS_PREPARATION = 5
    STATUS_PREPARATION_ACK = 6
    STATUS_SENDING = 7
    STATUS_RECEIVING = 8
    STATUS_DATA_ACK = 9
    STATUS_BYE = 10
    STATUS_BYE_ACK = 11
    STATUS_FINISH = 12
    STATUS_END = 13 # when all the stuff is finished
    
    CONTENT_LIMIT = 1202
    
    CONTENT_INVITE = 'application/x-msnmsgr-sessionreqbody'
    CONTENT_BYE = 'application/x-msnmsgr-sessionclosebody'
    CONTENT_CLOSE = 'application/x-msnmsgr-sessionclosebody'
    
    #flags

    #* 0x0 when no flags are specified
    NO_FLAG = int( 0x0 )
    #* 0x2 if it's an acknowledgement
    ACK_FLAG = int( 0x2 )
    #* 0x8 if there was an error on the binary level
    BINERROR_FLAG = int( 0x8 )
    #* 0x20 when the message content is data for display pictures or custom emoticons
    PIC_OR_EMO_FLAG = int( 0x20 )
    #* 0x40 Bye ack flag
    BYE_ACK_FLAG = int( 0x40 )
    #* 0x01000030 if it's data of a file 
    FILE_FLAG = int( 0x01000030 )

    EUFGUID_DISPLAY = '{A4268EEC-FEC5-49E5-95C3-F126696BDBF6}'
    EUFGUID_FILE = '{5D3E02AB-6190-11D3-BBBB-00C04F795683}'
    #EUFGUID_WEBCAM = '{4BD96FC0-AB17-4425-A14A-439185962DC8}'
    
    FOOTER = '\0\0\0\0'
    DP_FOOTER = '\0\0\0\1'
    FILE_FOOTER = '\0\0\0\2'
    INK_FOOTER = '\0\0\0\3'
    CE_FOOTER = '\0\0\0\x0c'
    
    def __init__( self ):
        self.status = Base.STATUS_CREATED
        
        self.From = ''
        self.To = ''
        self.Via = ''
        self.CSeq = ''
        self.CallID = ''
        self.MaxForwards = ''
        self.ContentType = ''
        self.ContentLength = ''
        self.EUFGUID = ''
        self.SessionID = ''
        self.AppID = ''
        self.Context = ''
        self.myLastHeader = None
        self.msnobj = None
        
        self.picture = None
        self.identifier = self.generateRandomNumber( 50000 )
        self.baseID = self.identifier # baseID will be modified
        self.alreadyChangedBaseID = False
        # we keep identifier because we need to skip the current value
                
    def setPicture( self, value ):
        self.picture = value

    def getPicture( self ):
        return self.picture
    
    def setMyLastHeader( self, value ):
        self.myLastHeader = value

    def getMyLastHeader( self ):
        return self.myLastHeader

    def setMsnobj( self, value ):
        self.msnobj = value

    def getMsnobj( self ):
        return self.msnobj
        
    def setFrom( self, value ):
        self.From = value

    def getFrom( self ):
        return self.From

    def setTo( self, value ):
        self.To = value

    def getTo( self ):
        return self.To

    def setVia( self, value ):
        self.Via = value

    def getVia( self ):
        return self.Via

    def setCSeq( self, value ):
        self.CSeq = value

    def getCSeq( self ):
        return self.CSeq

    def setCallID( self, value ):
        self.CallID = value

    def getCallID( self ):
        return self.CallID

    def setMaxForwards( self, value ):
        self.MaxForwards = value

    def getMaxForwards( self ):
        return self.MaxForwards

    def setContentType( self, value ):
        self.ContentType = value

    def getContentType( self ):
        return self.ContentType

    def setContentLength( self, value ):
        self.ContentLength = value

    def getContentLength( self ):
        return self.ContentLength

    def setEUFGUID( self, value ):
        self.EUFGUID = value

    def getEUFGUID( self ):
        return self.EUFGUID

    def setSessionID( self, value ):
        self.SessionID = value

    def getSessionID( self ):
        return self.SessionID

    def setAppID( self, value ):
        self.AppID = value

    def getAppID( self ):
        return self.AppID

    def setContext( self, value ):
        self.Context = value

    def getContext( self ):
        return self.Context
        
    def setStatus( self, status ):
        self.status = status
        
    def getStatus( self ):
        return self.status
    
    def nextBaseID( self ):
        if (self.baseID == self.identifier) and (not self.alreadyChangedBaseID):
            self.baseID -= 3
            self.alreadyChangedBaseID = True
        else:
            self.baseID += 1
            
            if self.baseID == self.identifier:
                self.baseID += 1
    
    # we use the max from sys.maxint in a 32 bit machine because
    # in a 64 bit machine it is bigger and can give some troubles
    def generateRandomNumber( self, minimum = 0, maximum = 2147483647 ):
        return random.randint( minimum, maximum )
     
    def randomid( self ):
        id = '%04X' % random.randint( 0x1111 , 0xAAFF )
        id += '%04X' % random.randint( 0x1111 , 0xAAFF ) + "-"
        id += '%04X' % random.randint( 0x1111 , 0xAAFF ) + "-"
        id += '%04X' % random.randint( 0x1111 , 0xAAFF ) + "-"
        id += '%04X' % random.randint( 0x1111 , 0xAAFF ) + "-"
        id += '%04X' % random.randint( 0x1111 , 0xAAFF )
        id += '%04X' % random.randint( 0x1111 , 0xAAFF )
        id += '%04X' % random.randint( 0x1111 , 0xAAFF )
        
        return id
    
    def getParameters( self, s ):
        '''parse a message and create and return a dict with the values'''
        d = {}
        key = lambda x: x.split(': ')[0]
        value = lambda x: ': '.join(x.split(': ')[1:])
        d = dict([ (key(i),value(i)) for i in s.split('\r\n') ]) 
        try:
            header = s.split( '\r\n\r\n' )[ 1 ][ :48 ]
            d[ 'header' ] = header
        except:
            pass
        return d
    
    def contain( self, string, substring ):
        return string.find( substring ) != -1
        
class Sender( Base ):

    def __init__( self, To, From,inviteMessage, picture, switchboard ):
        Base.__init__( self )
        
        self.setTo( To )
        self.setFrom( From )
        self.picture = picture
        self.message = inviteMessage
        self.switchboard = switchboard
        self.lastReceivedTime = int (time.time())
        
        if inviteMessage.find( "INVITE MSNMSGR:" ) == -1:
            print "Not an invite message"
            self.setStatus( Base.STATUS_FINISH )
        
        via = self.extractViaBranch()
        if via == "":
            print "Malformed invitation finishing, no Via ID"
            self.setStatus( Base.STATUS_FINISH )
        else:
            self.Via = via
            
        callID = self.extractCallID()
        
        if callID == "":
            print "Malformed invitation finishing, no Call-ID"
            self.setStatus( Base.STATUS_FINISH )
        else:
            self.setCallID( callID )
            
        sid = self.extractSessionID()
        
        if sid == "":
            print "Malformed invitation finishing, no SessionID"
            self.setStatus( Base.STATUS_FINISH )
        else:
            self.setSessionID( sid )
            print "SessionID in __init__ = " + str(self.SessionID) + "\n"
            
            
    #Via: MSNSLP/1.0/TLP ;branch={D7B851BF-97F2-47C4-8009-11FAFABDE1A9}
    def extractViaBranch( self ):
        """try to extract the via id from the invite message return empty string otherwise """        
        try:
            return self.message.split( "Via: MSNSLP/1.0/TLP ;branch={" )[ 1 ].split( "}" )[ 0 ]
        except:
            return ""
            
    def extractCallID( self ):
        """try to extract the callID from the invite message return empty string otherwise """        
        try:
            return self.message.split( "Call-ID: {" )[ 1 ].split( "}" )[ 0 ]
        except:
            return ""
            
     #SessionID: \r\n
    def extractSessionID( self ):
        """try to extract the sessionID from the invite message return an empty string otherwise"""
        try:
            return self.message.split( "SessionID: " )[ 1 ].split( "\r\n" )[ 0 ]
        except:
            return ""
            
    def updateTime( self ):
        self.lastReceivedTime = int ( time.time() )
        print "lastReceivedTime = " + str(self.lastReceivedTime)
        gobject.timeout_add( 30 * 1000, self.timerCallback )
        
    def timerCallback( self ):
        if int( time.time() ) - self.lastReceivedTime >= 30:
            print "more than 30 secs passed - abandoning the connection"
            self.setStatus( Base.STATUS_END )
     
    def process( self, message ):

        params = self.getParameters( message )
        
        try:
            header = BinHeader( params[ 'header' ] )
        except:
            print "NO HEADER IN MESSAGE!"
            return False
        
        header.printFields()
        
        if self.getStatus() == Base.STATUS_CREATED:
            self.setStatus( Base.STATUS_INVITE )
            #we got an invitation, send a baseid message, and a 200ok
            try:
                #return self.sendBaseID( header, params ) + self.send200OK() + \
                #self.sendDataPreparation() + self.sendData()
                #print "\nInvitation header:\n"
                #header.printFields()
                self.sendBaseID( header, params )
                self.send200OK()
                self.updateTime()
                self.setStatus( Base.STATUS_200OK )
            except Exception, e:
                print str( e )
                #return []
            return True
        elif self.getStatus() == Base.STATUS_200OK:
            #we expect a 200ok ack message. if we got one, we move to the next step -
            #sending a data preparation message
            #we can also get an invitation for a direct connection from WLM clients. 
            if self.is200OKAck( header ):
                try:
                    print "got a 200OKAck message - send data prep"
                    self.sendDataPreparation()
                    self.updateTime()
                    self.setStatus( Base.STATUS_PREPARATION )
                except Exception, e:
                    print str( e )
                return True
            elif self.isDirectConnectionInvitation( message ):
                try:
                    self.sendDirectConnectionRefusal( header, message )
                    self.updateTime()
                    self.setStatus( Base.STATUS_PREPARATION )
                except Exception, e:
                    print str( e )
                return True
            else:
                return False
        elif self.getStatus() == Base.STATUS_PREPARATION:
            #we expect a data prep ack message. if we got one, we send the data
            if self.isDataPrepAck( header ):
                try:
                    print "got a data prep ack - send data"
                    self.sendData()
                    self.updateTime()
                    self.setStatus( Base.STATUS_FINISH )
                except Exception, e:
                    print str( e )
                return True
            else:
                return False
        elif self.getStatus() == Base.STATUS_FINISH:
            #we expect a bye message. if received, send a bye ack message and mark
            #the sender class as a removal candidate
            if self.isByeMessage( header, message ):
                try:
                    print "got a bye message - acknowledge"
                    self.sendByeAck( header.getTotalDataSize(), header.getIdentifier(), header.getAcknowledgedIdentifier() )
                    self.updateTime()
                    self.setStatus( Base.STATUS_END )
                except Exception, e:
                    print str( e )
                return True
            else:
                return False
        #else:
            #return []
            
    def is200OKAck( self, header ):
        ''' fields to check:
        * his Identifier = his BaseID - 3
        * his Total data size == his Acknowledged data size == my Total data size (from 200OK)
        * Flags == 2
        * his Acknowledged identifier == my Identifier (from 200OK)
        * his Acknowledged unique ID == Acknowledged identifier (from 200OK)
        * all others == 0'''
        
        #if ( self.myLastHeader.getTotalDataSize() == header.getTotalDataSize() ) and \
        #( self.myLastHeader.getTotalDataSize() == header.getAcknowledgedDataSize() ) and \
        #( header.getFlag() == 2 ) and \
        #( self.myLastHeader.getAcknowledgedIdentifier() == header.getAcknowledgedUniqueID() ):
        
        if header.getTotalDataSize() == self.myLastHeader.getTotalDataSize() and\
           header.getAcknowledgedDataSize() == self.myLastHeader.getTotalDataSize() and\
           header.getFlag() == Base.ACK_FLAG and\
           header.getAcknowledgedIdentifier() == self.myLastHeader.getIdentifier() and\
           header.getAcknowledgedUniqueID() == self.myLastHeader.getAcknowledgedIdentifier():
            return True
        else:
            return False
            
    def isDataPrepAck( self, header ):
        ''' fields to check:
        * his SessionID from invite in his SessionID
        * his BadeID - 2 in his Identifier
        * his Total data size == Acknowledged data size == 4
        * his flags == 2
        * his Acknowledged identifier == my Identifier (from dataprep)'''

        if header.getTotalDataSize() == 4 and\
           header.getAcknowledgedDataSize() == 4 and\
           header.getFlag() == 2 and\
           header.getAcknowledgedIdentifier() == self.myLastHeader.getIdentifier() and\
           header.getAcknowledgedUniqueID() == self.myLastHeader.getAcknowledgedIdentifier():
            return True
        else:
            return False
            
    def isDirectConnectionInvitation( self, message ):
        #TODO - verify this function really works - I couldn't find documentation of the
        #direct connection invitation anywhere (The Albatross)
        if message.find( 'INVITE MSNMSGR' ) > -1 and \
        message.find( 'Content-Type: application/x-msnmsgr-transreqbody' ) > -1 and\
        message.find( 'Conn-Type:' ) > -1:
            return True
        else:
            return False
    
    def isByeMessage( self, header, message ):
        if ( message.find( 'BYE MSNMSGR' ) > -1 ) and \
        ( message.split( "Call-ID: {" )[ 1 ].split( "}" )[ 0 ] == self.getCallID() ):
            return True
        else:
            return False
        
    def isMessage( self, message ):
        return False
    
    def sendBaseID( self, bin, params ):
        '''things to send:
        my Identifier = random BaseID
        my Total data size = his last Total data size
        my flags = 2
        my Acknowledged identifier = his Identifier
        my Acknowledged unique ID = his Acknowledged identifier
        my Acknowledged data size = his Total data size
        All other fields are 0.'''
        
        try:
            self.SessionID = int( params[ 'SessionID' ] )
            print "SessionID in sendBaseID = " + str(self.SessionID) + "\n"
        except:
            raise Exception( 'Could not get sessionID from invite' )
        self.hisBaseID = bin.getIdentifier()
            
        header = BinHeader()
        header.setIdentifier( self.baseID )
        self.nextBaseID()
        header.setTotalDataSize(bin.getTotalDataSize())
        header.setFlag(Base.ACK_FLAG)
        header.setAcknowledgedIdentifier(bin.getIdentifier())
        header.setAcknowledgedUniqueID(bin.getAcknowledgedIdentifier())
        header.setAcknowledgedDataSize(bin.getTotalDataSize())
                
        ack = header.getData() + Base.FOOTER
        self.myLastHeader = header
        self.switchboard.sendMessage(ack, '', 'application/x-msnmsgrp2p', 'D', 'P2P-Dest: ' + self.getFrom() )
        #return [ ack ]
    
    def send200OK( self ):
        content = 'SessionID: ' + str( self.SessionID ) + '\r\n\r\n\0'
            
        s = 'MSNSLP/1.0 200 OK\r\n'
        s += 'To: <msnmsgr:' + self.getFrom() + '>\r\n'
        s += 'From: <msnmsgr:' + self.getTo() + '>\r\n'
        s += 'Via: MSNSLP/1.0/TLP ;branch={' + self.Via + '}\r\n'
        s += 'CSeq: 1 \r\n'
        s += 'Call-ID: {' + self.getCallID() + '}\r\n'
        s += 'Max-Forwards: 0\r\n'
        s += 'Content-Type: ' + Base.CONTENT_INVITE + '\r\n'
        s += 'Content-Length: ' + str( len( content ) ) + '\r\n'
        
        s += '\r\n'
        s += content
        s += Base.FOOTER
        
        header = BinHeader()
        header.setIdentifier( self.baseID )
        self.nextBaseID()
        header.setTotalDataSize( len( s ) - 4 )
        header.setMessageLength( len( s ) - 4 )
        header.setAcknowledgedIdentifier( self.generateRandomNumber() )
    
        self.myLastHeader = header
        ack200 = header.getData() + s
    
        #self.setStatus(Base.STATUS_INVITE)
        
        print "\n200OK Headers:\n"
        header.printFields()
        print "\nOriginal Base ID: " + str(self.identifier) + "\n"
        
        self.switchboard.sendMessage(ack200, '', 'application/x-msnmsgrp2p', 'D', 'P2P-Dest: ' + self.getFrom() )
        #return [ ack200 ]
        
    def sendDirectConnectionRefusal( self, receivedHeader, message ):
        #we should acknowledge the invitation, and then send a refusal
        header = BinHeader()
        header.setSessionID( self.getSessionID() )
        header.setIdentifier( self.baseID )
        self.nextBaseID()
        header.setFlag(Base.ACK_FLAG)
        header.setAcknowledgedIdentifier( self.generateRandomNumber() )
        header.setAcknowledgedDataSize( receivedHeader.getTotalDataSize() )
        header.setAcknowledgedUniqueID( receivedHeader.getAcknowledgedDataSize() )
        
        self.switchboard.sendMessage(header.getData() + Base.FOOTER, '', 'application/x-msnmsgrp2p', 'D', 'P2P-Dest: ' + self.getFrom() )
        
        header = BinHeader()
        header.setSessionID( self.getSessionID() )
        header.setIdentifier( self.baseID )
        self.nextBaseID()
        
        content = 'Bridge: TCPv1\r\n'
        content += 'Listening: false\r\n'
        content += 'Hashed-Nonce: {00000000-0000-0000-0000-000000000000}\r\n'
        content += '\r\n'
            
        s = 'MSNSLP/1.0 200 OK\r\n'
        s += 'To: <msnmsgr:' + self.getFrom() + '>\r\n'
        s += 'From: <msnmsgr:' + self.getTo() + '>\r\n'
        s += 'Via: MSNSLP/1.0/TLP ;branch={' + self.Via + '}\r\n'
        s += 'CSeq: 2 \r\n'
        s += 'Call-ID: {' + self.getCallID() + '}\r\n'
        s += 'Max-Forwards: 0\r\n'
        s += 'Content-Type: ' + Base.CONTENT_INVITE + '\r\n'
        s += 'Content-Length: ' + str( len( content ) ) + '\r\n'
        
        s += '\r\n'
        s += content
        s += Base.FOOTER
        
        header = BinHeader()
        header.setIdentifier( self.baseID )
        self.nextBaseID()
        header.setTotalDataSize( len( s ) - 2 )
        header.setMessageLength( len( s ) - 2 )
        header.setAcknowledgedIdentifier( self.generateRandomNumber() )
    
        self.myLastHeader = header
        refusal = header.getData() + s
        
        self.switchboard.sendMessage(refusal, '', 'application/x-msnmsgrp2p', 'D', 'P2P-Dest: ' + self.getFrom() )
    
    def sendDataPreparation( self ):
        header = BinHeader()
        header.setSessionID( self.getSessionID() )
        header.setIdentifier( self.baseID )
        self.nextBaseID()
        header.setTotalDataSize(4)
        header.setMessageLength(4)
        header.setAcknowledgedIdentifier( self.generateRandomNumber() )
        
        self.myLastHeader = header
        
        print "\nData Prep Headers:\n"
        header.printFields()
        #print "\nOriginal Base ID: " + str(self.identifier) + "\n"
        
        self.switchboard.sendMessage(header.getData() + '\0\0\0\0' + Base.DP_FOOTER, '', 'application/x-msnmsgrp2p', 'D', 'P2P-Dest: ' + self.getFrom() )
        #return [ header.getData() + '\0\0\0\0' + Base.FOOTER ]
        
    
    def sendData( self ):
        if not self.picture or self.picture.getSize() == 0:
            raise Exception( 'picture size 0 aborting' )

        moreData = True
        self.picture.setPointer( 0 )
        l = []
        print "avatar: " + self.picture.path
        
        while moreData:
            header = BinHeader()
            header.setSessionID( self.SessionID )
            header.setIdentifier( self.baseID )
            header.setDataOffset( self.picture.getPointer() )
            header.setTotalDataSize( self.picture.getSize() )
            header.setFlag( Base.PIC_OR_EMO_FLAG )
            header.setAcknowledgedIdentifier( self.generateRandomNumber() )
            
            if (self.picture.getSize() - self.picture.getPointer() ) > Base.CONTENT_LIMIT:
                header.setMessageLength( Base.CONTENT_LIMIT )
                chunk = self.picture.getData()[ self.picture.getPointer():self.picture.getPointer() + Base.CONTENT_LIMIT ] 
                self.picture.setPointer( self.picture.getPointer() + Base.CONTENT_LIMIT ) 
            else:
                header.setMessageLength( (self.picture.getSize() - self.picture.getPointer()) )
                chunk = self.picture.getData()[ self.picture.getPointer(): ]
                self.picture.setPointer( self.picture.getSize() )
                self.nextBaseID()
                moreData = False
            
            self.myLastHeader = header
            print "data segment"
            header.printFields()
            #l.append( header.getData() + chunk + Base.DP_FOOTER )
            self.switchboard.sendMessage( header.getData() + chunk + Base.DP_FOOTER, '', 'application/x-msnmsgrp2p', 'D', 'P2P-Dest: ' + self.getFrom() )
            
        #return l
        
    def sendByeAck( self, sizeField, HisBaseIdentifier, HisBaseID ):
        header = BinHeader()
        header.setSessionID( 0 )
        header.setIdentifier( self.identifier + 1 )
        self.nextBaseID()
        header.setFlag( 0x40 )
        header.setTotalDataSize( sizeField )
        header.setMessageLength( 0 )
        header.setAcknowledgedIdentifier( HisBaseIdentifier )
        header.setAcknowledgedDataSize( sizeField )
        header.setAcknowledgedUniqueID( HisBaseID )
        
        self.myLastHeader = header
        
        print "Bye ack message"
        header.printFields()
        
        self.switchboard.sendMessage(header.getData() + Base.FOOTER, '', 'application/x-msnmsgrp2p', 'D', 'P2P-Dest: ' + self.getFrom() )
        
        
class Receiver( Base ):

    def __init__( self, To, From, msnobj, filename, emitCb, AppID = 1 ):
        Base.__init__( self )
        
        self.setSessionID( self.generateRandomNumber( minimum=50000, maximum=99999 ) )
        self.setTo( To )
        self.setFrom( From ) # from its me
        self.setMsnobj( msnobj )
        self.setVia( self.randomid() )
        self.setCallID( self.randomid() )
        self.setPicture( Picture( filename, Picture.CREATE ) )
        
        self.AppID = AppID
        
        self.emitCb = emitCb
        
        self.hisBaseID = 0
        self.hisLastHeader = None
        self.nextMessageIdWaited = 0
        
        self.doDebug = False
    
    def debug( self, text ):
        if self.doDebug:
            print text
    
    def process( self, message ):
        params = self.getParameters( message )
        
        if not params.has_key( 'header' ):
            self.debug( '\n' + self.To + ' no header in process!\n' )
            return []
        
        header = BinHeader( params[ 'header' ] )
        
        if self.contain( message, "Conn-Type:" ) and self.contain( message, 'INVITE MSNMSGR:' ):
            self.debug( "direct connect, sending decline" )
            return self.decline()
            
        if self.isBaseID(message, header, params):
            print "is baseID"
            self.setStatus( Base.STATUS_BASEID )
            self.nextMessageIdWaited = header.getIdentifier() - 3
            return []
        if self.is200Ok(message, header, params):
            print "is 200ok"
            self.nextMessageIdWaited = header.getIdentifier() + 1
            self.hisLastHeader = header
            return self.ack200OK( header )    
        elif self.isPreparation(message, header, params):
            print "is preparation"
            self.hisBaseID = header.getIdentifier()
            self.hisLastHeader = header
            self.nextMessageIdWaited = header.getIdentifier() + 1
            return self.DpmAck( header )
        elif self.isData(message, header, params):
            print "is data"
            self.hisLastHeader = header
            return self.receiveData( message, header )
        elif self.isByeAck( message, header, params ):
            print "bye ack"
            self.setStatus( Base.STATUS_END )
        else:
            self.debug( 'Unknown message on Receiver.process, closing' )
            #self.setStatus( Base.STATUS_END )
            return []

    def isMessage( self, message ):
        params = self.getParameters( message )
        
        if not params.has_key( 'header' ):
            self.debug( '\n' + self.To + ' NO HEADER IN PROCESS!\n' )
            return False
        
        header = BinHeader( params[ 'header' ] )
        
        # TODO: restrict this to all kind of not switchboard connection
        if message.find( "Conn-Type:" ) != -1 and \
        message.find( 'INVITE MSNMSGR:' ) != -1:
            return True
        
        if self.getStatus() == Base.STATUS_CREATED:
            self.debug( "\njust created return false\n" )
            return False
            
        if self.isBaseID( message, header, params ):
            self.debug( "identified baseID " + str( self ) )
            return True
        elif self.is200Ok( message, header, params ):
            self.debug( "identified 200Ok " + str( self ) )
            return True
        elif self.isPreparation( message, header, params ):
            self.debug( "identified preparation " + str( self ) )
            return True
        elif self.isData( message, header, params ):
            self.debug( "identified data " + str( self ) )
            return True
        elif self.isByeAck( message, header, params ):
            self.debug( "bye ack " + str( self ) )
            return True
        else:
            self.debug( "not from " + str( self ) )
            return False
        
    def inviteMessage( self ):
        self.setStatus(Base.STATUS_INVITE)
        
        # a message from gaim 2.0.6 to WLM that works
        '''
        INVITE MSNMSGR:xmxsxn@hotmail.com MSNSLP/1.0
        To: <msnmsgr:xmxsxn@hotmail.com>
        From: <msnmsgr:luismarianoguerra@gmail.com>
        Via: MSNSLP/1.0/TLP ;branch={CE68F97B-2977-4899-AAA0-7277B172B781}
        CSeq: 0 
        Call-ID: {89BCFC9D-408B-41C2-823C-E60601891369}
        Max-Forwards: 0
        Content-Type: application/x-msnmsgr-sessionreqbody
        Content-Length: 344

        EUF-GUID: {A4268EEC-FEC5-49E5-95C3-F126696BDBF6}
        SessionID: 158288
        SChannelState: 0
        AppID: 12
        Context: PG1zbm9iaiBDcmVhdG9yPSJ4bXhzeG5AaG90bWFpbC5jb20iIFNpemU9IjE1MzUxIiBUeXBlPSIzIiBMb2NhdGlvbj0iVEZSMkMyLnRtcCIgRnJpZW5kbHk9IkFBQT0iIFNIQTFEPSJ6V0VlNlB3T0JBSDBnQ0lxbVZVUjJycHpOR2M9IiBTSEExQz0iT3pqdEdQZHNuUkJqVVpSaDBIT3V5T0RvbENBPSIvPgA=

        \0
        '''
        
        content = 'EUF-GUID: {A4268EEC-FEC5-49E5-95C3-F126696BDBF6}\r\n'
        content += 'SessionID: ' + str( self.getSessionID() ) + '\r\n'
        content += 'SChannelState: 0\r\nAppID: ' + str( self.AppID ) + '\r\n'
        content += 'Context: ' + base64.b64encode( str( self.msnobj  ) + '\0' ) + '\r\n\r\n\0'
        
        s = 'INVITE MSNMSGR:' + self.getTo() + ' MSNSLP/1.0\r\n'
        s += 'To: <msnmsgr:' + self.getTo() + '>\r\n'
        s += 'From: <msnmsgr:' + self.getFrom() + '>\r\n'
        s += 'Via: MSNSLP/1.0/TLP ;branch={' + self.getVia() + '}\r\n'
        s += 'CSeq: 0 \r\n'
        s += 'Call-ID: {' + self.getCallID() + '}\r\n'
        s += 'Max-Forwards: 0\r\n'
        s += 'Content-Type: ' + Base.CONTENT_INVITE + '\r\n'
        s += 'Content-Length: ' + str( len( content ) ) + '\r\n'
        
        s += '\r\n'
        s += content
        s += Base.FOOTER
        
        header = BinHeader()
        header.setIdentifier( self.baseID )
        self.nextBaseID()
        header.setTotalDataSize( len( s ) - 4 )
        header.setMessageLength( len( s ) - 4 )
        header.setAcknowledgedIdentifier( self.generateRandomNumber() )
        
        self.myLastHeader = header
        
        return header.getData() + s
    
    def decline( self ):
        content = 'SessionID: ' + str( self.SessionID ) + '\r\n\r\n\0'
        
        s = 'MSNSLP/1.0 603 Decline\r\n'
        s += 'To: <msnmsgr:' + self.getTo() + '>\r\n'
        s += 'From: <msnmsgr:' + self.getFrom() + '>\r\n'
        s += 'Via: MSNSLP/1.0/TLP ;branch={' + self.Via + '}\r\n'
        s += 'CSeq: 1 \r\n'
        s += 'Call-ID: {' + self.getCallID() + '}\r\n'
        s += 'Max-Forwards: 0\r\n'
        s += 'Content-Type: application/x-msnmsgr-sessionreqbody\r\n'
        s += 'Content-Length: ' + str( len( content ) ) + '\r\n'
        
        s += '\r\n'
        s += content
        s += Base.FOOTER
        
        header = BinHeader()
        header.setIdentifier( self.baseID )
        self.nextBaseID()
        header.setTotalDataSize( len( s ) - 4 )
        header.setMessageLength( len( s ) - 4 )
        header.setAcknowledgedIdentifier( self.generateRandomNumber() )
    
        self.myLastHeader = header
            
        self.setStatus(Base.STATUS_PREPARATION_ACK)
        return [ header.getData() + s ]
        
    def basicAck( self, bin ):
        '''return a basic ack header'''
        
        header = BinHeader()
        header.setIdentifier( self.baseID )
        self.nextBaseID()
        header.setTotalDataSize(bin.getTotalDataSize())
        header.setAcknowledgedDataSize(bin.getTotalDataSize())
        header.setFlag(Base.ACK_FLAG)
        header.setAcknowledgedIdentifier(bin.getIdentifier())
        header.setAcknowledgedUniqueID(bin.getAcknowledgedIdentifier())
        
        return header
        
    def ack200OK( self, bin ):
        self.setStatus( Base.STATUS_200OK_ACK )
        header = self.basicAck( bin )
        self.myLastHeader = header
        return [header.getData() + Base.FOOTER]
        
    def DpmAck( self, bin ):
        self.setStatus(Base.STATUS_PREPARATION_ACK)
        header = self.basicAck( bin )
        self.myLastHeader = header
        return [header.getData() + Base.FOOTER]
        
    def receiveData( self, message, bin ):
        self.picture.setPointer( bin.getDataOffset() + bin.getMessageLength() )
        self.picture.appendData( message.split( '\r\n\r\n' )[ 1 ][ 48:48 + bin.getMessageLength() ] )
            
        self.debug( 'actual ' + str( self.picture.getPointer() ) + ' size ' +  str( bin.getTotalDataSize() ) )
            
        if self.picture.getPointer() == bin.getTotalDataSize():
            self.debug( 'writing file..' )
            self.picture.write()
            
            if self.msnobj.getType() == "3":
                self.emitCb( 'display-picture-changed', self.To )
            elif self.msnobj.getType() == "2":
                self.emitCb( 'custom-emoticon-transfered', self.To, self.msnobj, self.picture.getPath() )
                
            self.setStatus( Base.STATUS_BYE )
            
            return self.ackData( bin ) + self.bye( bin )
        
        return []
            
    def ackData( self, bin ):
        header = self.basicAck( bin )
        header.setSessionID( self.getSessionID() )
        self.myLastHeader = header
        return [header.getData() + Base.FOOTER]
        
    def bye( self, bin ):
        s = 'BYE MSNMSGR:' + self.getTo() + ' MSNSLP/1.0\r\n'
        s += 'To: <msnmsgr:' + self.getTo() + '>\r\n'
        s += 'From: <msnmsgr:' + self.getFrom() + '>\r\n'
        s += 'Via: MSNSLP/1.0/TLP ;branch={' + self.randomid() + '}\r\n'
        s += 'CSeq: 0 \r\n'
        s += 'Call-ID: {' + self.getCallID() + '}\r\n'
        s += 'Max-Forwards: 0\r\n'
        s += 'Content-Type: application/x-msnmsgr-sessionclosebody\r\n'
        s += 'Content-Length: 3\r\n'
        s += '\r\n\r\n\0\0\0\0\0'
        
        header = BinHeader()
        header.setIdentifier( self.baseID )
        self.nextBaseID()
        header.setTotalDataSize( len( s ) - 4 )
        header.setMessageLength( len( s ) - 4 )
        header.setFlag( 0x80 )
        header.setAcknowledgedIdentifier( self.generateRandomNumber() )
        
        self.myLastHeader = header

        self.nextMessageIdWaited += 2
         
        return [ header.getData() + s ]
        
    def isBaseID( self, message, header, params ):
        '''control:
        (Alice)                   (Bob)
        Total data size         = Total data size
        Flag                   == 2
        identifier              = Acknowledged identifier
        Acknowledged identifier = Acknowledged unique ID (im not checking this)
        Total data size         = Acknowledged data size

        other fields             = 0'''
        
        #print "checking if message is baseID"
        #print "status should be " + str( Base.STATUS_INVITE ) + " and is " + str( self.getStatus() )
        #print "total data size should be " + str( self.myLastHeader.getTotalDataSize() ) + " and is " + str( header.getTotalDataSize() )
        #print "flag should be " + str( Base.ACK_FLAG ) + " and is " + str( header.getFlag() )
        #print "acknowledged identifier should be " + str( self.identifier )  + " and is " + str( header.getAcknowledgedIdentifier() )
        #print
        
        # some clients send this one as 0
        #self.myLastHeader.getTotalDataSize() == header.getTotalDataSize() and
        if self.getStatus() == Base.STATUS_INVITE and \
           header.getFlag() == Base.ACK_FLAG and \
           self.identifier == header.getAcknowledgedIdentifier() and \
           self.myLastHeader.getTotalDataSize() == header.getAcknowledgedDataSize():
           
           return True
        
        return False
        
    def is200Ok( self, message, header, params  ):
        '''control:
        (Alice)                   (Bob)
                          Identifier == his BaseID - 3
                          Total data size == length of the MSNSLP data
                          Message length  == length of the MSNSLP data
                          Acknowledged identifier == random

        other fields              = 0'''
        
        #print "checking if message is 200 ok"
        #print "status should be " + str( Base.STATUS_BASEID ) + " and is " + str( self.getStatus() )
        #print "identifier should be " + str( self.nextMessageIdWaited )  + " and is " + str( header.getIdentifier() )
        #print
        
        if self.getStatus() == Base.STATUS_BASEID and \
          self.contain( message, 'MSNSLP/1.0 200 OK' ) and \
          header.getIdentifier() == self.nextMessageIdWaited:
            return True

        return False
        
    def isPreparation( self, message, header, params ):
        '''control:
        (Alice)                   (Bob)
        SessionID               = SessionID (Field 1)
                                Identifier == his BaseID - 2
                                Total data size == 4
                                Message length == 4
                                Acknowledged identifier == random

        other fields              = 0'''

        #print "checking if message is preparation"
        #print "status should be " + str( Base.STATUS_200OK_ACK ) + " and is " + str( self.getStatus() )
        #print "identifier should be " + str( self.nextMessageIdWaited )  + " and is " + str( header.getIdentifier() )
        #print "TotalDataSize should be 4 and is " + str( header.getTotalDataSize() )
        #print "MessageLength should be 4 and is " + str( header.getMessageLength() )
        #print
        
        # self.nextMessageIdWaited is set on self.process on if self.is200Ok
        if self.getStatus() == Base.STATUS_200OK_ACK and \
          header.getIdentifier() == self.nextMessageIdWaited and \
          header.getTotalDataSize() == 4 and \
          header.getMessageLength() == 4:
            return True
        
        return False
        
    def isData( self, message, header, params ):
        '''control:
        (Alice)                   (Bob)
        SessionID               = SessionID (Field 1)
                          Identifier == his BaseID - 2
                          Flag == 0x20
                          Acknowledged identifier == random
                          Acknowledged unique ID == 0
                          Acknowledged data size == 0'''
        
        #print "checking if message is data"
        #print "status should be " + str( Base.STATUS_PREPARATION_ACK ) + " and is " + str( self.getStatus() )
        #print "identifier should be " + str( self.nextMessageIdWaited )  + " and is " + str( header.getIdentifier() )
        #print "SessionID should be " + str( self.getSessionID() )  + " and is " + str( header.getSessionID() )
        #print "Flag should be " + str( Base.PIC_OR_EMO_FLAG )  + " and is " + str( header.getFlag() )
        #print

        # when we come from a direct connect this field is 0        
        #header.getIdentifier() == self.nextMessageIdWaited and \
        # if someone is sending data to this session and we
        # skipped some step it doesnt matherm we receive it
        #if self.getStatus() == Base.STATUS_PREPARATION_ACK and \
        
        if self.getSessionID() == header.getSessionID() and \
        header.getFlag() == Base.PIC_OR_EMO_FLAG:
            self.setStatus(Base.STATUS_PREPARATION_ACK)
            return True
            
        return False
        
    def isByeAck( self, message, header, params ):
        '''control:
        (Alice)                   (Bob)
                                  Identifier == his BaseID + 1
        Total data size         = Total data size
        Total data size         = Acknowledged data size
                                  Flag == 0x40
        Identifier              = Acknowledged identifier
        Acknowledged identifier = Acknowledged unique ID

        other fields              = 0'''
        
        #print "checking if message is bye ack"
        #print "status should be " + str( Base.STATUS_BYE ) + " and is " + str( self.getStatus() )
        #print "identifier should be " + str( self.nextMessageIdWaited )  + " and is " + str( header.getIdentifier() )
        #print "TotalDataSize should be " + str( self.myLastHeader.getTotalDataSize() )  + " and is " + str( header.getTotalDataSize() )
        #print "Flag should be " + str( Base.BYE_ACK_FLAG )  + " and is " + str( header.getFlag() )
        #print
        
        # if we lost some package and he send us a bye ack we identify it to close
        # the session
        #if self.getStatus() == Base.STATUS_BYE and \
        
        if self.myLastHeader.getTotalDataSize() == header.getTotalDataSize() and \
        header.getIdentifier() == self.nextMessageIdWaited and \
        header.getFlag() == Base.BYE_ACK_FLAG:
            return True
            
        return False
        

class Picture( object ):
    '''a class that contains data about our display picture'''
    
    OPEN = 1
    CREATE = 2
    
    def __init__( self, path, action ):
        '''Contructor'''
        
        self.path = path
        
        if action == Picture.OPEN:
            f = file( path, 'r' )
            if f:
                self.data = f.read()
                f.close()
            else:
                self.data = ''
        else:
            self.data = ''
        
        self.size = len( self.data )
        #point to the actual point where the data is
        #being readed
        self.pointer = 0
        
    def write( self ):
        f = file( self.path, 'wb' )
        f.write( self.data )
        f.close()
        print "wrote!"
        
    def getData(self):
        return self.data
    
    def setData( self, data ):
        self.data = data
        
    def appendData( self, data ):
        self.data += str( data )
        
    def getPointer( self ):
        return self.pointer
    
    def setPointer( self, position ):
        self.pointer = position
        
    def getSize( self ):
        return self.size
    
    def setSize( self, size ):
        self.size = size
        
    def getPath( self ):
        return self.path
    
    def setPath( self, path ):
        self.path = path
       
class ByteArray( object ):
    '''a class that represent a byte array (little endian)'''
    
    def __init__( self, size ):
        '''create a byte array with size=size and initialize with 0'''
        
        self.fmt = '<' + str( size ) + 'B'
        self.data = struct.pack( self.fmt, *[ 0 for i in range( size ) ] )
        self.valueList = list( struct.unpack( self.fmt, self.data ) )

    def getData( self ):
        return self.data

    def __setitem__( self, pos, value ):
        '''set the value in pos'''
        
        if len( self.valueList ) > pos:
            self.valueList[ pos ] = value
            self.data = struct.pack( self.fmt, *self.valueList )
            
    def __getitem__( self, pos ):
            return self.valueList[ pos ]
        
    def setDword( self, pos, value ):
        # less significative
        byte1 = self.__extractByte( value, 0 )
        byte2 = self.__extractByte( value, 1 )
        byte3 = self.__extractByte( value, 2 )
        byte4 = self.__extractByte( value, 3 )
        
        self[ pos ] = byte1
        self[ pos + 1 ] = byte2
        self[ pos + 2 ] = byte3
        self[ pos + 3 ] = byte4
        
    def getDword( self, pos ):
        byte1 = self[ pos ]
        byte2 = self[ pos + 1 ]
        byte3 = self[ pos + 2 ]
        byte4 = self[ pos + 3 ]
        
        return (byte4<<24) + (byte3<<16) + (byte2<<8) + byte1
        
    def setQword( self, pos, value ):
        byte1 = self.__extractByte( value, 0 )
        byte2 = self.__extractByte( value, 1 )
        byte3 = self.__extractByte( value, 2 )
        byte4 = self.__extractByte( value, 3 )
        
        byte5 = self.__extractByte( value, 4 )
        byte6 = self.__extractByte( value, 5 )
        byte7 = self.__extractByte( value, 6 )
        byte8 = self.__extractByte( value, 7 )
        
        self[ pos ] = byte1
        self[ pos + 1 ] = byte2
        self[ pos + 2 ] = byte3
        self[ pos + 3 ] = byte4
        
        self[ pos + 4 ] = byte5
        self[ pos + 5 ] = byte6
        self[ pos + 6 ] = byte7
        self[ pos + 7 ] = byte8
        
    def getQword( self, start ):
        return (self.getDword( start + 4 ) << 32) + self.getDword( start )
    
    def __extractByte( self , number , byte ):
        '''
        extract a given byte of a number and return it
        0 -> first byte ( less significative )
        '''
        
        return ( number & ( 255 << byte * 8 )) >> ( byte * 8 )
        
    def __repr__( self ):
        return self.data
    
    def __len__(self):
        return len( self.valueList )

       
class BinHeader( ByteArray ):
    '''this class represent the bin header'''
    
    def __init__( self, data = None ):
        '''Contructor'''
        
        ByteArray.__init__( self, 48 )
        
        if data != None:
            self.fill( data )
            
    def setSessionID( self, dword ):
        self.setDword( 0, dword )
        
    def getSessionID( self ):
        return self.getDword( 0 )
    
    def setIdentifier( self, dword ):
        self.setDword( 4, dword )
        
    def getIdentifier( self ):
        return self.getDword( 4 )
    
    def setDataOffset( self, qword ):
        self.setQword( 8, qword )
        
    def getDataOffset( self ):
        return self.getQword( 8 )
    
    def setTotalDataSize( self, qword ):
        self.setQword( 16, qword )
        
    def getTotalDataSize( self ):
        return self.getQword( 16 )
    
    def setMessageLength( self, dword ):
        self.setDword( 24, dword )
        
    def getMessageLength( self ):
        return self.getDword( 24 )
    
    def setFlag( self, dword ):
        self.setDword( 28, dword )
        
    def getFlag( self ):
        return self.getDword( 28 )
    
    def setAcknowledgedIdentifier( self, dword ):
        self.setDword( 32, dword )
        
    def getAcknowledgedIdentifier( self ):
        return self.getDword( 32  )
    
    def setAcknowledgedUniqueID( self, dword ):
        self.setDword( 36, dword )
        
    def getAcknowledgedUniqueID( self ):
        return self.getDword( 36 )
    
    def setAcknowledgedDataSize( self, qword ):
        self.setQword( 40, qword )
        
    def getAcknowledgedDataSize( self ):
        return self.getQword( 40 )
    
    def printNumbers( self ):
        count = 0
        for i in self.data:
            print hex( ord( i ) ),
            
            # :)
            if len( str( hex( ord( i ) ) ) ) == 3:
                print '  ',
            else:
                print ' ',
                
            count += 1
            if count > 15:
                print
                count = 0
                
        print
        
    def printFields( self ):
        return
        print '-------------------------------------------\n'
        print 'SessionID:               ' + str( self.getSessionID() )
        print 'Identifier:              ' + str( self.getIdentifier() )
        print 'Data offset:             ' + str( self.getDataOffset() )
        print 'Total data size:         ' + str( self.getTotalDataSize() )
        print 'Message length:          ' + str( self.getMessageLength() )
        print 'Flag:                    ' + str( self.getFlag() )
        print 'Acknowledged identifier: ' + str( self.getAcknowledgedIdentifier() )
        print 'Acknowledged unique ID:  ' + str( self.getAcknowledgedUniqueID() )
        print 'Acknowledged data size:  ' + str( self.getAcknowledgedDataSize() )
        print '-------------------------------------------\n'
        
    def fill( self, data ):
        self.setSessionID(self.getDwordFromData(data, 0))
        self.setIdentifier(self.getDwordFromData(data, 4))
        self.setDataOffset(self.getQwordFromData(data, 8))
        self.setTotalDataSize(self.getQwordFromData(data, 16))
        self.setMessageLength(self.getDwordFromData(data, 24))
        self.setFlag(self.getDwordFromData(data, 28))
        self.setAcknowledgedIdentifier(self.getDwordFromData(data, 32))
        self.setAcknowledgedUniqueID(self.getDwordFromData(data, 36))
        self.setAcknowledgedDataSize(self.getQwordFromData(data, 40))
        self.printFields()
    
    def getDwordFromData( self, header, start ):
        '''extract 4 bytes from header starting from start
        and return it as a number'''
    
        b1 = ord( header[ start ] )
        b2 = ord( header[ start + 1 ] )
        b3 = ord( header[ start + 2 ] ) 
        b4 = ord( header[ start + 3 ] )
    
        return (b4<<24) + (b3<<16) + (b2<<8) + b1

    def getQwordFromData( self, header, start ):
        # i think its like this
        return (self.getDwordFromData( header, start + 4) << 32) + self.getDwordFromData( header, start )

