# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


import Queue
import threading
import httplib

requestQueue = Queue.Queue( 0 )
responseQueue = Queue.Queue( 0 )

def put( request ):
    global requestQueue

    requestQueue.put( request )
    
def get():
    global responseQueue

    return responseQueue.get()

def getNoBlock():
    global responseQueue

    return responseQueue.get(True, 0.01)

def process():
    '''run the callbacks if some response is in responseQueue'''
    global responseQueue
    
    while True:
        try:
            response = getNoBlock()
            
            if response.getCallback() != None:
                response.getCallback()( response, *response.getArgs() )
        except Queue.Empty:
                break

class SoapRequest( object ):
    '''This class represent a soapRequest'''
    
    def __init__( self, action, host, port, path, body, callback = None, args = () ):
        '''Contructor'''
        
        self._action = action
        self._host = host
        self._port = port
        self._path = path
        self._body = body
        self._callback = callback
        self._args = args
        self._extraCookie = ''
        self._errorCount = 0
        self._status = ( 0 , '' )

    def getAction( self ):
        return self._action
        
    def setAction( self, action ):
        self._action = action

    def getErrorCount( self ):
        return self._errorCount
    
    def setErrorCount( self, count ):    
        self._errorCount = count
    
    def getExtraCookie( self ):
        return self._extraCookie
    
    def setExtraCookie( self, cookie ):
        self._extraCookie = cookie
    
    def getHost( self ):
        return self._host
        
    def setHost( self, host ):
        self._host = host
        
    def getPort( self ):
        return self._port
        
    def setPort( self, port ):
        self._port = port

    def getPath( self ):
        return self._path

    def setPath( self, path ):
        self._path = path
        
    def setBody( self, body ):
        self._body = body

    def getBody( self ):
        return self._body
        
    def getCallback( self ):
        return self._callback
        
    def setCallback( self, callback ):
        self._callback = callback

    def setStatus( self, status ):
        self._status = status

    def getStatus( self ):
        return self._status
        
    def setArgs( self, args ):
        self._args = args

    def getArgs( self ):
        return self._args
        

class SoapManager( threading.Thread ):
    '''This class is a thread that wait for soapRequest in the requestQueue and make
    the request, then add the response to the responseQueue'''
       
    def __init__( self, MSPAuth, MSPProf ):
        threading.Thread.__init__( self )
        
        self.MSPAuth = MSPAuth
        self.MSPProf = MSPProf
        
    def run( self ):
        global requestQueue, responseQueue
        
        while True:
            req = requestQueue.get()
            
            if req == 'quit':
                break
            else:
                try:
                    responseQueue.put( self.makeSoapRequest( req ) )
                except Exception, e:
                    print e
                    if req.getErrorCount() < 2:
                        req.setErrorCount( req.getErrorCount() + 1 )
                        requestQueue.put( req )
                
        return False
        
    def makeSoapRequest( self, soapRequest ):
    
        headers = { \
        "SOAPAction" : soapRequest.getAction() , \
        "Content-Type" : "text/xml; charset=utf-8" , \
        "Cookie" : "MSPAuth=" + self.MSPAuth + ";MSPProf=" + self.MSPProf + soapRequest.getExtraCookie() , \
        "Host" : soapRequest.getHost() , \
        "Content-Length" : str( len( soapRequest.getBody() ) ) , \
        "User-Agent" : "MSN Explorer/9.0 (MSN 8.0; TmstmpExt)",\
        "Connection" : "Keep-Alive",\
        "Cache-Control" : "no-cache",\
        }
        
        conn = None
        response = None
        
        if soapRequest.getPort() == 443:
            conn = httplib.HTTPSConnection( soapRequest.getHost() + ':' + str( soapRequest.getPort() ) )  
        else:
            conn = httplib.HTTPConnection( soapRequest.getHost() + ':' + str( soapRequest.getPort() ) ) 
             
        conn.request( "POST", soapRequest.getPath(), soapRequest.getBody(), headers )
        response = conn.getresponse()

        soapResponse = SoapRequest( soapRequest.getAction(), soapRequest.getHost(), soapRequest.getPort(), soapRequest.getPath(), response.read(), soapRequest.getCallback(), soapRequest.getArgs() )            
        soapResponse.setStatus( ( response.status, response.reason ) )
        
        return soapResponse               
