# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import os
import socket
import urllib
import select
import gobject
import base64

import Msnobj
import Socket # class from core
import MsnP2P

class Switchboard( gobject.GObject ):
    '''This class represent a switchboard connection with the notification server.'''
    
    __gsignals__ = {
        'typing' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,)),

        'user-join' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,)),
        'user-leave' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,)),
        'user-offline' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,)),
        'user-online' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,)),

        'message' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            #tid, nick, body, format, charset
            (gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING, 
             gobject.TYPE_STRING, gobject.TYPE_STRING)),
        'action-message' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING, gobject.TYPE_STRING)),
            #tid, data
        'ink-message' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING, gobject.TYPE_STRING)),
            #tid, filename

        'nudge' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,)),
        'status-change' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
            
        'custom-emoticon-received' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,gobject.TYPE_PYOBJECT,)),

    } 

    def __init__( self , id, msn, type, status = 'pending' ):
        '''class contructor
        type can be 'requested' or 'invited', its 'requested' if we request the
        switchboard and guess what invited means? :P.
        status is pending initialy, then when we process the connection string is
        connected and is stablished when someone join the conversation, 
        if something goes wrong the status is closed'''

        gobject.GObject.__init__(self)
        self.id = id
        
        self._status = status
        self._validStatus = [ 'pending', 'connected', 'established', 'closed', 'error' ]
        self._invalidTransitions = { 'connected' : [ 'pending' ], 'established' : [ 'pending', 'connected' ] }
        self._connectionString = ''
            
        self.msn = msn

        self._user = self.msn._user
        self._debug = self.msn._debug
        self._proxy = self.msn._proxy
        
        self._host = ''
        self._port = 0
        self._authenticationType = ''
        self._authenticationString = ''
        self._sessionID = ''
        self._command = ''
        self._socket = None

        self.members = {}
        self.messageQueue = [] # a list that holds the messages sent when the status was 'pending
                               # its a list of dicts like this { 'msg' : msg, 'format' : format, 'contentType' : contentType }'
        self._invitationQueue = [] # a list of mails to invite when we go to established status
        self.inkSessions = {} # session:dict{'file','total','actual'}
        
        
        self.p2p = MsnP2P.P2PManager( self )
        self.userOfflineId = self.msn.connect( 'user-offline', self.userOffline )
        self.userOnlineId = self.msn.connect( 'user-online', self.userOnline )
        self.msnobjChangedId = self.msn.connect( 'msnobj-changed', self.msnobjChanged )
        
        self.firstMessage = False
        self.firstUser = ''
        
        self.error = 0
   
    def connect( self, signal, *args ):
        print 'connection signal ' + signal
        
        return gobject.GObject.connect( self, signal, *args )
        
    def __repr__( self ):
        return '<Switchboard users: ' + str( self.members.keys() ) + '>' 
   
    def parse( self ):
        '''parse the connection string to fill the attributes'''
    
        if self._connectionString == '':
            return        

        if self._connectionString.split()[ 0 ] == 'XFR':
            # example
            # XFR 15 SB 207.46.108.37:1863 CKI 17262740.1050826919.32308\r\n
            ( command, tid, sb, host, authenticationType, authenticationString, u, server) = self._connectionString.split()
        else:
            # example
            # RNG 11752013 207.46.108.38:1863 CKI 849102291.520491113 example@passport.com Example%20Name
            ( command, sessionID, host, authenticationType, authenticationString, user, userName, u, server ) = self._connectionString.split()
            self._sessionID = sessionID
             
            self.addMember( user, userName )

        self._command = command
        self._host = host.split( ':' )[ 0 ]
        self._port = int( host.split( ':' )[ 1 ] )
        self._authenticationType = authenticationType
        self._authenticationString = authenticationString
        
    def leaveChat( self ):
        '''leave the conversation'''
        
        self.setStatus( 'closed' )
        try:
            self._socket.sendText( 'OUT\r\n' )
        except Exception, e:
            pass
        
    def splitMsg( self, message ):
        '''return header(dict), body(str)'''
        part = message.split( '\r\n\r\n' )
        def htuple(x):
            parts = x.split(': ')
            return ( parts[0], ': '.join(parts[1:]) )
        header = dict([ htuple(i) for i in part[0].split('\r\n') ])
        body = '\r\n\r\n'.join(part[1:])
        return header,body
        
    def process( self ):
        '''read the info from Socket and process the info'''
        
        if self._socket == None: return
        
        fds = self._socket.select()
        if len( fds[ 0 ] ) < 1 or len( fds[ 1 ] ) < 1: return
        
        try:
            (command, tid, params) = self._socket.receiveCommand()
        except:
            print str( self ) + ' closed'
            self.setStatus('closed')
            return

        if command == 'MSG':
            try:
                message = self._socket.receivePayload( int( params.split()[ 1 ] ) )
            except Exception, e:
                # raise e
                self._socket.debug( 'socket error on switchboard, closing switchboard' )
                self._socket.debug( str( e ) )
                self.setStatus( 'closed' )
                return
                
            nick = params.split()[ 0 ]
            self._socket.debug( message )
            
            header, body = self.splitMsg( message )
            Type = ''
            if "Content-Type" in header:
                Type = header["Content-Type"]
            
            if Type.startswith('text/plain'):
                # MSG god@gmail.com God%20http://www.400monkeys.com/God/ 111\r\n
                # MIME-Version: 1.0
                # Content-Type: text/plain; charset=UTF-8
                # X-MMS-IM-Format: FN=Sans; EF=; CO=000000; PF=0
                # 
                # a
                
                if not self.firstMessage:
                    self.firstMessage = True
                    self.msn.emit( 'new-conversation', self.firstUser, self )
                
                format = ''
                if 'X-MMS-IM-Format' in header:
                    format = header['X-MMS-IM-Format']
                
                charset = Type.split('text/plain; charset=')[1]
                
                self.emit( 'message', tid, nick, body, format, charset )
                self.msn.emit( 'message-received', tid )    
                
            elif Type == 'text/x-msnmsgr-datacast':
                # datacasts: nudges and actions
                if body.find('ID: 1') != -1:
                    # nudges
                    if not self.firstMessage:
                        self.firstMessage = True
                        self.msn.emit( 'new-conversation', self.firstUser, self )
                    
                    self.msn.emit( 'nudge-received', tid )
                    self.emit( 'nudge', tid )

                elif body.find('ID: 4') != -1:
                    # actions
                    if not self.firstMessage:
                        self.firstMessage = True
                        self.msn.emit( 'new-conversation', self.firstUser, self )
                    data = body.split("Data: ")[1] 
                    self.emit( 'action-message', tid, data )
                    self.msn.emit( 'message-received', tid )
            
            elif Type == 'text/x-msmsgscontrol':
                self.emit( 'typing', tid )

            elif Type == 'text/x-mms-emoticon' or \
                 Type == 'text/x-mms-animemoticon':
                if not self.firstMessage:
                    self.firstMessage = True
                    self.msn.emit( 'new-conversation', self.firstUser, self )
                self.parseCustomEmoticon( message )

            elif Type == 'application/x-msnmsgrp2p' and \
                 header['P2P-Dest'].lower() == self.getUser().lower():
                if body.startswith("@\x00\x00\x00"):
                    try:
                        self.parseInkMessage( message, tid )
                    except:
                        pass
                else:
                    self.p2p.process( message, tid )
            
            elif Type == 'image/gif':
                try:
                    self.parseInkMessage( message, tid )
                except:
                    pass
            elif 'Message-ID' in header:
                try:
                    if header['Message-ID'] in self.inkSessions.keys():
                        self.parseInkMessage( message, tid )
                except:
                    pass
            else:
                self._socket.debug("Unhandled content type: " + str(header))
                
        elif command == 'USR':
            if params.split( ' ' )[ 0 ] == 'OK':
                self.setStatus( 'connected' )
            else:
                raise 'SwitchboardException'

        elif command == 'IRO':
            #IRO 1 1 1 god@gmail.com God%20http://www.400monkeys.com/God/ 1615642660\r\n
            ( currentNumber, totalNumber, mail, nick, clientID ) = params.split( ' ' )
            self.addMember( mail.lower(), nick )
            
            self.emit( 'user-join', mail.lower() )
                
            self.setStatus( 'established' ) 
                
        elif command == 'JOI':
            # JOI god@gmail.com God%20http://www.400monkeys.com/God/ 1615642660\r\n
            ( nick, clientID ) = params.split( ' ' )
            self.addMember( tid, nick )
            
            self.emit( 'user-join', tid )
                
            self.setStatus( 'established' ) 
                
        elif command == 'ANS':
            if params == 'OK':
                self.setStatus( 'connected' )                
            else:
                raise 'SwitchboardException'
        elif command == 'BYE':
            self.leave( tid )
            
            # TODO, check when a bye means an OUT
            # from the other side.
            if params == '1':
                self.emit( 'user-leave', tid )
        try:
            error=int(command)
            self._socket.debug( 'server error %d, closing switchboard' % error)
            self.error = error
            self.setStatus( 'error' )
        except ValueError:
            pass

    def parseInkMessage( self, message, tid ):
        header, body = self.splitMsg( message )
        if body[0] == "@":
            # p2p ink on private conversation
            self._socket.debug("p2p ink message")
            p2p = True
            
            # get info
            binary = MsnP2P.BinHeader(body[:48])
            identifier = binary.getIdentifier()
            size = binary.getTotalDataSize()
            offset = binary.getDataOffset()
            length = binary.getMessageLength()
            
            # get content
            content = body[48:-4].replace("\x00","") # TODO: find a better way
            
            if offset == 0:  # first message
                cheader, cbody = self.splitMsg( content )
                content = cbody[7:]
        else:
            # group chat, but it can be received on any situation
            # this is more efficient than the other method because there
            # is no utf-16 encoding (and that means double size)
            self._socket.debug("group chat ink")
            p2p = False
            
            if "Content-Type" in header :
                if body[0:7] != "base64:":
                    self._socket.debug("base64 sig not found")
                    return
                content = body[7:]
            else: 
                content = body
            length = 1
            offset = 0
            if "Message-ID" in header:
                # multipart
                identifier = header["Message-ID"]
                if "Content-Type" in header and "Chunks" in header:
                    size = int(header["Chunks"])
                else:
                    offset = int(header["Chunk"])
            else:
                # only one message, no sessions needed
                identifier = ''
                size = 1
                
        # save the session data
        if offset==0:
            uniqueName = identifier or len(content)
            inkData = content
            if identifier:
                self.inkSessions[identifier] = { 'data': inkData, 'total': size, 'actual': length }
                current = self.inkSessions[identifier]
                self._socket.debug("ink: %s / %s" % (current['actual'], current['total']))
        else:
            current = self.inkSessions[identifier]
            current['actual'] += length
            current['data'] += content
            self._socket.debug("ink: %s / %s" % (current['actual'], current['total']))
            
        # check if transfer has finished
        if not identifier or (current['actual'] == current['total']):
            self._socket.debug("saving...")
            if identifier: inkData = current['data']
            inkData = base64.decodestring(inkData)
            
            filename = "ink%s_%s.gif" % (urllib.quote(tid), identifier)
            completeFileName = self.msn.getCacheDir() + os.sep + filename
            
            rc = open(completeFileName, "w")
            rc.write(inkData)
            rc.close()
            
            self.emit('ink-message', tid, completeFileName)
            
            if p2p:
                binary.setFlag(MsnP2P.Base.ACK_FLAG)
                binary.setAcknowledgedDataSize(binary.getTotalDataSize())
                binary.setAcknowledgedIdentifier(binary.getIdentifier())
                binary.setAcknowledgedUniqueID(binary.getAcknowledgedIdentifier())
                reply = binary.getData() + MsnP2P.Base.FOOTER
                self.sendMessage(reply, '', 'application/x-msnmsgrp2p', 'D', 'P2P-Dest: ' + tid )
                print "sent acknowledge"
                
            if identifier: del self.inkSessions[identifier]

    def parseCustomEmoticon( self, message ):
        body = message.split( '\r\n\r\n' )[ 1 ]
        l = body.split( '\t' )
        d = {}
      
        while len( l ) > 0:
            if len( l ) < 2:
                break
            shortcut = l.pop(0)
            msnobjString = l.pop(0)
            msnobj = Msnobj.createFromString( msnobjString, False )
           
            if msnobj != None:
                self.emit( 'custom-emoticon-received', shortcut, msnobj )
                filename = urllib.quote( shortcut + '_' + msnobj.sha1d + '.tmp').replace('/', '_')
                completeFileName = self.msn.getCacheDir() + os.sep + filename
                
                if not os.path.isfile( completeFileName ):
                    self.getCustomEmoticon(msnobj.getCreator(), msnobj, completeFileName)
                else:
                    self.msn.emit( 'custom-emoticon-transfered', None, msnobj, completeFileName )
            

    def sendMessage( self, msg = '',  format = '' , contentType = 'text/plain; charset=UTF-8', acknowledgeType = 'A', extraHeader = '' ):
        if extraHeader != "":
            header = "MIME-Version: 1.0\r\nContent-Type: " + contentType + "\r\n" + extraHeader + '\r\n'
        else:
            header = "MIME-Version: 1.0\r\nContent-Type: " + contentType + "\r\n"
            
        if contentType == 'text/plain; charset=UTF-8':
            msg = msg[ :1100 ]   #TODO: use something like MAX_MESSAGE_LENGTH
        
        if self._status != 'established':
            self.messageQueue.append( { 'msg' : msg, 'format' : format, 'contentType' : contentType, 'acknowledgeType' : acknowledgeType, 'extraHeader' : extraHeader } )                    

        elif self._status == 'established':
            if format != "":
                header += format + "\r\n\r\n"
            else:
                header += "\r\n"
                
            try:
                self._socket.sendPayloadCommand( 'MSG', acknowledgeType, header + msg )
            except Exception, e:
                #raise e
                print 'socket error on switchboard, closing switchboard'
                self._socket.debug( str( e ) )
                self.setStatus( 'closed' )

    def flushMessageQueue( self ):
        '''send all the unsended messages'''
        
        if self._status == 'established':

            # flush the queue
            if len( self.messageQueue ) > 0:
                for i in self.messageQueue:
                    if i['extraHeader'] != "":
                        header = "MIME-Version: 1.0\r\nContent-Type: " + i[ 'contentType' ] + "\r\n" + i['extraHeader'] + '\r\n'
                    else:
                        header = "MIME-Version: 1.0\r\nContent-Type: " + i[ 'contentType' ] + "\r\n"
                    
                    if i[ 'format' ] != "":
                        header += i[ 'format' ] + "\r\n\r\n"
                    else:
                        header += "\r\n"
                
                    try:
                        self._socket.sendPayloadCommand( 'MSG', i[ 'acknowledgeType' ], header + i[ 'msg' ] )
                    except Exception, e:
                        self._socket.debug( 'socket error on switchboard, closing switchboard' )
                        self._socket.debug( str( e ) )
                        self.setStatus( 'closed' )
                # flushed
                self.messageQueue = []
                self._socket.debug( 'message queue flushed' )

    def sendNudge( self ):
        '''a easy method to send a nudge'''

        self.sendMessage( 'ID: 1\r\n\r\n', '', 'text/x-msnmsgr-datacast' )

    def sendAction( self, data ):
        '''a easy method to send an action message - this works with MSNC6+'''

        self.sendMessage( 'ID: 4\r\nData: %s\r\n' % data, '', 'text/x-msnmsgr-datacast' )

    def sendIsTyping( self ):
        '''a easy method to send the is typing message'''

        self.sendMessage( '\r\n', '', 'text/x-msmsgscontrol\r\nTypingUser: ' + self._user )

    def invite( self, mail ):
        '''invite somebody to the switchboard conversation'''

        if self.firstUser == '':
            self.firstUser = mail.lower()

        if ( self._status == 'connected' or self._status == 'established' ) and not mail in self.members.keys():
            try:
                self._socket.sendCommand( 'CAL', mail )
            except Exception, e:
                #raise e
                self._socket.debug( 'socket error on switchboard, closing switchboard' )
                self._socket.debug( str( e ) )
                self.setStatus( 'closed' )
                return
            
            if mail in self._invitationQueue:
                self._invitationQueue.pop( self._invitationQueue.index( mail ) )
        elif not mail in self._invitationQueue:
            self._invitationQueue.append( mail.lower() )

    def leave( self, mail ):
        '''remove the mail from the members list'''
        
        if mail.lower() in self.members.keys():
            if len( self.members ) == 1:
                self.setStatus('closed')
            else:
                del self.members[ mail.lower() ]
    
    def setConnectionString( self, connectionString ):
        '''we received the conecction string (the first response of a XFR or the RNG)
        and here we process it'''
        
        self._connectionString = connectionString        
        self.parse()
        self.connectSocket()
        
    def connectSocket( self ):
        '''connect the socket'''
        
        if self._proxy != None:
            self._socket = Socket.PSocket( self._host, self._port, \
                self._debug, self._proxy, 'SB' )
        else:
            self._socket = Socket.Socket( self._host, self._port, \
                self._debug )
            try:
                self._socket.connect()
            except socket.error, e:
                self._socket.debug( "can't connect to switchboard, socket error" )
                self._socket.debug( str( e ) )
                self.setStatus( 'closed' )
                return
        
        if self._command == 'XFR':
            try:
                self._socket.sendCommand( "USR" , self._user + " " + self._authenticationString )
            except Exception, e:
                #raise e
                self._socket.debug( 'socket error on switchboard, closing switchboard' )
                self._socket.debug( str( e ) )
                self.setStatus( 'closed' )
            
        else: #RNG
            # ANS 1 alice@passport.com 1056411141.26158 17342299\r\n
            try:
                self._socket.sendCommand( "ANS" , self._user + " " + self._authenticationString + " " + self._sessionID )
            except Exception, e:
                #raise e
                self._socket.debug( 'socket error on switchboard, closing switchboard' )
                self._socket.debug( str( e ) )
                self.setStatus( 'closed' )
                        
    def addMember( self, mail, nick ):
        '''add a member to the members dict'''
        
        if self.firstUser == '':
            self.firstUser = mail.lower()
        self.members[ mail.lower() ] = nick.replace( '%20', ' ' )
        self.getDisplayPicture( mail.lower() )

    def setStatus( self, status ):
        '''set the status, the status can be 'pending', 'stablished' or 'closed'
        PLEASE change the status here and not directly because we may want to do thing when a status
        is changed'''
        
        if status in self._validStatus and self._status != status:
            if self._invalidTransitions.has_key( self._status ) and status in self._invalidTransitions[ self._status ]:
                return

            self._status = status

            if status == "closed":
                self.msn.disconnect( self.userOfflineId )
                self.msn.disconnect( self.userOnlineId )
                self.msn.disconnect( self.msnobjChangedId )
            if status == "error":
                try:
                    self._socket.sendText( 'OUT\r\n' )
                except Exception, e:
                    pass
                self._socket = None
            if status == 'connected':
                for i in self._invitationQueue:
                    self.invite( i )
                    
                self._invitationQueue = []
            elif self._status == 'established':
                self.flushMessageQueue()
            
            self.emit( 'status-change' )
            
    def getMSN(self):
        return self.msn
        
    def getStatus( self ):
        '''return the status'''
        return self._status
    
    def getUser( self ):
        '''return the user mail'''
        
        return self._user
    
    def getId( self ):
        '''return the id of the switchboard, this is a unique identification of the switchboars.
        The value is the value of the Trid of the command that created the switchboard or the
        sessionid if the other user started the conversation, the value doesnt mather, what mather is
        that its unique.'''
        
        return self.id
    
    def getOnlineUsers( self ):
        '''This method returns a list ol mails of the contacts who are not offline'''
        
        return self.msn.contactManager.getOnlineUsers()
    
    def getMembers( self ):
        '''return a list of the members in the switchboard'''
        
        return self.members.keys()

    def isGroupChat( self ):
        return (len( self.members ) > 1)
        
    def getFirstUser( self ):
        return self.firstUser

    def getInvitedMembers( self ):
        '''return a list of the members invited but not joined'''
        
        return self._invitationQueue
        
    def getUserDisplayName( self, mail ):
        '''return the user display name ot just the mail if it cant be found'''
        
        alias = self.msn.contactManager.getContactAlias( mail.lower() )
        
        if alias:
            return alias
        else:
            return self.msn.contactManager.getContactNick( mail.lower() )

    def userOffline( self, msnp, email ):
        '''Method called when a user get offline'''
        email = email.lower()
        if email in self.members.keys():
            self.emit( 'user-offline', email )
            self.leave( email )

    def userOnline( self, msnp, email, oldStatus ):
        '''Method called when a user get online'''
        email = email.lower()
        if email in self.members.keys() and oldStatus == 'FLN':
            self.emit( 'user-online', email )
        
    def getNick( self, mail ):
        '''return the nick of the user mail'''
        
        return self.msn.contactManager.getContactNick( mail.lower() )
    
    def getDisplayPicture( self, email ):
        '''start a P2P session to get the display picture'''
        email = email.lower()
        contact = self.msn.contactManager.getContact( email )
        
        if contact == None:
            print 'contact (' + email + ') not found in getDisplayPicture'
            return
        
        msnobj = contact.msnobj
        
        if msnobj == None:
            print email + ' has no msnobj in getDisplayPicture'
            return
        
        filename = self.msn.getCacheDir() + os.sep + email.split( '@' )[ 0 ] + '.tmp'
        receiver = self.p2p.newReceiver( email, self.getUser(), msnobj, filename )
        message = receiver.inviteMessage()
        self.sendMessage(message, '', 'application/x-msnmsgrp2p', 'D', 'P2P-Dest: ' + email )
        # TODO
        #~ filename = self.msn.getCacheDir() + os.sep + email.split( '@' )[ 0 ] + msnobj.sha1c +'.png'
        #~ if not os.path.isfile( filename ):
            #~ receiver = self.p2p.newReceiver( email, self.getUser(), msnobj, filename )
            #~ message = receiver.inviteMessage()
            #~ self.sendMessage(message, '', 'application/x-msnmsgrp2p', 'D', 'P2P-Dest: ' + email )
            
    def msnobjChanged( self, msnp, msnobj, wasOffline ):
        '''callback called when the user change his msnobj'''
        
        if msnobj != None and msnobj.getCreator() == self.firstUser:
            self.getDisplayPicture( msnobj.getCreator() )
            
    def getCustomEmoticon( self, email, msnobj, filename ):
        email = email.lower()
        receiver = self.p2p.newReceiver( email, self.getUser(), msnobj, filename, 12 )
        message = receiver.inviteMessage()
        self.sendMessage(message, '', 'application/x-msnmsgrp2p', 'D', 'P2P-Dest: ' + email )
        
    def sendCustomEmoticons( self, message ):
        msnObjs = []
        msnObj=''
        i=0
        msnOM = self.msn.getMsnObjectsManager()
        for CE in msnOM.getIds():
            if i == 4:
                i=0
                msnObjs.append( msnObj )
                msnObj = ''
            if message.find(CE) != -1:
                msnObj += CE+'\t'+str( msnOM.getById(CE) )+'\t'
                i +=1
        if msnObj != '': msnObjs.append( msnObj )
        for msnObj in msnObjs:
            self.sendMessage( msnObj, contentType='text/x-mms-animemoticon' )

gobject.type_register(Switchboard)
