# -*- coding: utf-8 -*-

import re
import os
import time
import socket
import string
import urllib
import httplib
import select
import thread
import threading

import Socket
import base64
import MsnOIM
import Msnobj
import MsnP2P
import ContactData
import SoapManager
import XmlTemplates
import Switchboard
import DefaultSignalHandler
import gobject

import xml.sax.saxutils

status_table = {
    'online':       'NLN',
    'away':         'AWY',
    'busy':         'BSY',
    'brb':          'BRB',
    'phone':        'PHN',
    'lunch':        'LUN',
    'invisible':    'HDN',
    'idle':         'IDL',
    'offline':      'FLN',
}

reverse_status = {
    'NLN':        'online',
    'AWY':        'away',
    'BSY':        'busy',
    'BRB':        'brb',
    'PHN':        'phone',
    'LUN':        'lunch',
    'HDN':        'invisible',
    'IDL':        'idle',
    'FLN':        'offline',
}

class Msnp( gobject.GObject ):
    '''This class give support to the MSNP13 protocol to use the MSN Messenger network'''

    __gsignals__ = {
        'user-list-change' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
        # when we receive a message, the parameter is the user mail
        'message-received' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_STRING,)),
        'connection-problem' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
        # disconnected happen when someone login in the same account
        'disconnected' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
        # disconnect is when a socket error happens
        'connection-closed' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
        'challenge' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,gobject.TYPE_STRING,gobject.TYPE_STRING)),
        'initial-status-change' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,gobject.TYPE_STRING,gobject.TYPE_STRING)),
        # command, tid, params
        'status-change' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,gobject.TYPE_STRING,gobject.TYPE_STRING)),
        # command, tid, params
        'status-online' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,gobject.TYPE_STRING,gobject.TYPE_STRING)),
        # command, tid, params
        'status-offline' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,gobject.TYPE_STRING,gobject.TYPE_STRING)),
        'switchboard-invitation' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,gobject.TYPE_STRING,gobject.TYPE_STRING)),
        'add-notification' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,gobject.TYPE_STRING,gobject.TYPE_STRING,gobject.TYPE_STRING)),
        'remove-notification' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,gobject.TYPE_STRING,gobject.TYPE_STRING,gobject.TYPE_STRING)),
        'personal-message-changed' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,gobject.TYPE_STRING)),
        'server-message' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,gobject.TYPE_STRING,gobject.TYPE_STRING,gobject.TYPE_STRING)),
        'user-disconnected' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,gobject.TYPE_STRING)),
        'logout' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
        # signal emited when we change our nick, the parameters are the old and
        # the new nickname
        'self-nick-changed' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,gobject.TYPE_STRING)),
        'self-status-changed' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,)),
        'self-personal-message-changed' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,gobject.TYPE_STRING)),
        'self-current-media-changed' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,gobject.TYPE_STRING)),
        
        'new-conversation' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,gobject.TYPE_PYOBJECT)),
        'nudge-received' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,)),
            
        'initial-mail-notification' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            ()),
        # From, FromAddr, Subject, MessageURL , PostURL, id
        'new-mail-notification' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,gobject.TYPE_STRING,gobject.TYPE_STRING,gobject.TYPE_STRING,gobject.TYPE_STRING,gobject.TYPE_STRING)),
        'mail-movement-notification' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            ()),
            
        'nick-changed' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,gobject.TYPE_STRING)),
        # email, status
        'contact-status-change' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,gobject.TYPE_STRING)),
         # the bolean value on msnobj-changed indicate if he was offline
         # to not request all the avatars when we come online
        'msnobj-changed' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_PYOBJECT,gobject.TYPE_BOOLEAN)),
        'user-online' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,gobject.TYPE_STRING)),
        'user-offline' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,)),
        'send-message-error' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,gobject.TYPE_STRING,gobject.TYPE_STRING)),
            
        'offline-message-waiting' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_PYOBJECT,)),
        'offline-message-received' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_PYOBJECT,)),
        
        'display-picture-changed' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,)),
        'custom-emoticon-transfered' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,gobject.TYPE_PYOBJECT,gobject.TYPE_STRING)),
    }

    def __init__( self, host, port, user, password, \
        userConfigDir, debug = False, proxy = None ):
        '''Contructor, intialize variables.'''
        gobject.GObject.__init__(self)
        
        if proxy == None:
            self.socket = Socket.Socket( host, port, debug )
        else:
            self.socket = Socket.PSocket( host, port, debug, proxy )

        self._user = user.lower()
        self._password = password

        self._debug = debug
        self._cacheDir = userConfigDir+os.sep+'cache'
        
        self._userConfigDir = userConfigDir

        self._proxy = proxy
        self._callbacks = None
        self.nick = ''

        self.entourage = None
        self.status = 'FLN'

        self.switchboards = {} # the switchboard dict
        
        self.demographics = {}

        self.membershipListXML = ''
        self.addressBookXML = ''
        self.dynamicItemsXML = ''

        self.contactManager = ContactData.ContactList( {} )

        self.msnOIM = MsnOIM.MsnOIM( self )

        self.currentMedia = ''
        self.personalMessage = ''
        
        self.msnobj = None # the msnobj for the display picture
        self.picture = None
        
        # p2p features support number
        self.CLIENT_ID = 0x60000000 | 0x4
        
        self.msnObjectsManager = Msnobj.MsnObjectsManager( user )

        self.inboxMessages = 0
        self.inboxUnreadMessages = 0
        self.otherMessages = 0
        self.otherMessagesUnreaded = 0
        
    def _presentation( self ):
        '''this method implement the first stage of the login code because we have 
        to do it two times, return the command, tid and params from the USR command to
        see what we have to do if it is XFR we have to connect to another server if the
        response is USR then whe continue the login'''

        #we connect to the server
        self.socket.connect()

        # send the protocol version we speak
        self.socket.sendCommand( 'VER' , 'MSNP13 CVR0' )
        (command, tid, params) = self.socket.receiveCommand()

        # if we received VER we check if the server wants to speak the same version
        if command == 'VER':
            # if he doesnt want to speak version 13 we raise an exception
            if not params.startswith( 'MSNP13' ):
                raise ProtocolError , params
        else:
            # if the server didnt send us VER then something is wrong
            raise ProtocolError , command

        # we introduce ourselves
        self.socket.sendCommand( "CVR" , \
            "0x0c0a winnt 5.1 i386 MSNMSGR 8.0.0792 msmsgs " + self._user )
        (command, tid, params) = self.socket.receiveCommand()

        # read the response
        if command != 'CVR':
            raise ProtocolError , command

        # introduce the user
        self.socket.sendCommand( "USR" , "TWN I " + self._user )
        return self.socket.receiveCommand()

    def login( self ):
        '''login to the server, after succesfully login you have to change your
            status to appear online'''
        (command, tid, params) = None, None, None
        
        # Redirection
        # We accept as much as 5 redirections
        succeeded = False
        for i in range( 5 ):
            (command, tid, params) = self._presentation()

            if command == 'XFR' and params[ :2 ] == 'NS':
                # the above command tell us where do we have to connect
                #example: XFR 3 NS 207.46.24.39:1863 U D

                self.socket.close()
                try:
                    ns = params.split()[ 1 ].split( ":" ) # 207.46.24.39:1863
                    host = ns[ 0 ]        # 207.46.24.39
                    port = int( ns[ 1 ] ) # int( 1863 )
                except:
                    raise RedirectionError, 'Incorrect parameters: ' + params
                debug = self.socket._debugFlag

                if self._proxy == None:
                    self.socket = Socket.Socket( host, port, self._debug )
                else:
                    self.socket = Socket.PSocket( host, port, self._debug, self._proxy )

                # and do all over again
            else:
                succeeded = True
                break
            
        if not succeeded:
            raise RedirectionError, 'Too many redirections'
        
        
        # Recieve GCF payload if any
        if command == 'GCF':
            self.socket.receivePayload( int( params ) )

        (command, tid, params) = self.socket.receiveCommand()

        # if we didnt get a USR response..
        if command != 'USR':
            raise AuthError, str( command ) + ' received. Expecting USR - TWN'

        # we get the hash
        try:
            hash = params.split( "TWN S " )[ 1 ]
        except Exception, e:
            print e
            raise AuthError, 'Incorrect USR response parameters: ' + params
        # start the passport authentication
        self.passportid = self.passportAuth( hash )
        try:
            self.t = self.passportid.split( '&p=')[ 0 ]
            self.t = self.t[2:]
            self.MSPProf = self.passportid.split( '&p=')[ 1 ]
        except:
            raise AuthError, 'Incorrect passportid: ' + passportid

        # we introduce ourselves again
        self.socket.sendCommand( "USR" , "TWN S " + self.passportid )
        (command, tid, params) = self.socket.receiveCommand()
        # if something is wrong raise an exception
        try:
            if command != "USR" and params[ 0:2 ] != "OK":
                raise AuthError
        except:
            raise AuthError, '"%s - %s" received. Expecting USR - OK' % (command, params)

        data = self.socket.receive()        
        if self._proxy == None:
            while data != '':
                data = self.socket.receive()
                if data.startswith( "MSPAuth" ):
                    try:
                        self.MSPAuth = data.split( "MSPAuth: " )[ 1 ]
                    except:
                        raise AuthError, 'Incorrect MSPAuth data: ' + data
                else:
                    try:
                        key, value =  data.split( ":" )
                        self.demographics.update( {key.strip():value.strip()} )
                    except:
                        pass
        else:
            # if we are using the HTTP Method all the data comes
            # in one package (nice :)
            try:
                payload = self.socket.receive()
                self.MSPAuth = payload.split( "MSPAuth: " ) \
                    [ 1 ].split( '\r\n' )[ 0 ]
            except:
                raise AuthError, 'Incorrect MSPAuth data: ' + data
        self.socket.onLogin = False
        
        # ------ logged in --------
        self.soapManager = SoapManager.SoapManager( self.MSPAuth, self.MSPProf )
        self.soapManager.start()
        # Get the lists
        mlCache = self.getCacheFileData( self._user + "_ml.xml" )
        diCache = self.getCacheFileData( self._user + "_di.xml" )
        
        if mlCache and diCache:
            self.membershipListXML = mlCache
            self.dynamicItemsXML = diCache
            self.contactManager.setMembershipListXml( self.membershipListXML  )
            self.contactManager.setDynamicItemsXml( self.dynamicItemsXML )
            self.getNickFromDynamicItems()
            self.emit( 'user-list-change' )
        else:
            if not self.getMembershipListSync():
                raise LoginError, 'Couldn\'t retrieve membership list'
            if not self.getDynamicItemsSync():
                raise LoginError, 'Couldn\'t retrieve dynamic items'

        # send privacy status
        self.socket.sendCommand( "BLP" , "BL" )
        
        # send our contacts (as of rev 411 only FL contacts are sent)
        for i in self.createADL():
            self.socket.sendPayloadCommand( "ADL", '', i )
        
        # change the personal message
        self.changePersonalMessage('')
        
        # change the nickname
        self.changeNick( self.nick )

        # connect some callbacks
        self._callbacks = DefaultSignalHandler.DefaultSignalHandler( self )
        self.connect( 'challenge' , self._callbacks.challenge )
        self.connect( 'status-change' , self._callbacks.statusChange )
        self.connect( 'initial-status-change' , self._callbacks.statusChange )
        self.connect( 'status-online' , self._callbacks.statusOnline )
        self.connect( 'status-offline' , self._callbacks.statusOffline )
        self.connect( 'switchboard-invitation' , self._callbacks.switchboardInvitation )
        self.connect( 'server-message' , self._callbacks.serverMessage )
        
        if mlCache and diCache:
            self.getMembershipListAsync()
            self.getDynamicItemsAsync()

    def passportAuth( self , hash ):
        '''
        do the passport authenticaton, this is done connecting to loginnet.passport.com:443
        and sending a XML message described on XmlTemplates.passport
        '''

        self.socket.debug('PASSPORT begin')

        user = self._user
        password = self._password
        # unquote the hash
        ticket = urllib.unquote( hash ).replace( "," , "&amp;" )
        # replace the %s in the string
        body = XmlTemplates.passport%( user, password, ticket )

        #create the headers
        headers = { \
        "Accept" :  "text/*" , \
        "User-Agent" : "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1)" , \
        "Host" : "loginnet.passport.com" , \
        "Content-Length" : str( len( body ) ) , \
        "Connection" : "Keep-Alive" , \
        "Cache-Control" : "no-cache" , \
        }

        _server="loginnet.passport.com"
        _url="/RST.srf"
        
        succeeded = False
        for i in range( 5 ):
            # send the SOAP request
            for i in range( 3 ):
                try:
                    conn = httplib.HTTPSConnection(_server,443)
                    conn.request("POST", _url, body, headers)
                    response = conn.getresponse()
                    self.socket.debug( str( response.status ) + " " + response.reason )
                    break
                except Exception, e:
                    #print str( e )
                    #raise e
                    pass

            data = response.read()

            if data.find( '<faultcode>psf:Redirect</faultcode>' ) > 0:
                from urlparse import urlparse
                _url=urlparse( data.split( '<psf:redirectUrl>' )[ 1 ].split( '</psf:redirectUrl>' )[ 0 ] )
                # fixed according to http://docs.python.org/lib/module-urlparse.html
                _server=_url[ 1 ]
                _url=_url[ 2 ]
                self.socket.debug( 'Redirect to: %s %s' % ( _server, _url ) )
            else:
                succeeded = True
                break
        
        if not succeeded:
            raise AuthError, 'Too many redirections'

        # try to get the ticket from the received data
        try:
            twnticket = data.split( "<wsse:BinarySecurityToken Id=\"PPToken1\">" )\
                            [ 1 ].split( "</wsse:BinarySecurityToken>" )[ 0 ]
            twnticket = twnticket.replace( "&amp;" , "&" )
        except Exception, e:
            debugmsg( e )
            debugmsg( data )
            # try to get the faultstring
            try:
                faultstring = data.split( "<faultstring>" )\
                    [ 1 ].split( "</faultstring>" )[ 0 ]
            except:
                faultstring = ''
            raise AuthError, faultstring

        return twnticket

    def ping( self ):
        '''send a ping to the server and register the timestamp of the command'''
        try:
            self.socket.ping()
        except (IOError, socket.error):
            self.emit( 'connection-closed' )
            
    def checkConnection( self ):
        '''return true if the time between the last ping and the response is
        lower than MAX_PING_DELAY'''
        
        if (self.socket.lastqng < self.socket.lastpng) and\
        ( int( time.time() ) - self.socket.lastpng ) > self.socket.MAX_PING_DELAY:
            return False
        
        return True

    def getMembershipListSync( self ):
        '''request the membership list dont return until we get it'''

        self.socket.debug( 'request Membership list' )

        request = SoapManager.SoapRequest( 'http://www.msn.com/webservices/AddressBook/FindMembership',\
                                'contacts.msn.com', 443,\
                                '/abservice/SharingService.asmx' , XmlTemplates.membershipList )

        SoapManager.put( request )
        response = SoapManager.get()

        return self.onGetMembershipList( response )

    def getMembershipListAsync( self ):
        '''request the membership list return and put a callback'''

        self.socket.debug( 'request Membership list' )

        request = SoapManager.SoapRequest( 'http://www.msn.com/webservices/AddressBook/FindMembership',\
                                'contacts.msn.com', 443, '/abservice/SharingService.asmx' , \
                                XmlTemplates.membershipList, self.onGetMembershipList )

        SoapManager.put( request )

    def onGetMembershipList( self, response ):
        '''method called when we receive the membership list'''

        self.socket.debug( str( response.getStatus() ) )

        if response.getStatus()[ 0 ] == 200:
            self.membershipListXML = response.getBody()
            self.contactManager.setMembershipListXml( self.membershipListXML  )
            self.newCacheFile( self._user + '_ml.xml', self.membershipListXML )
            self.emit( 'user-list-change' )
            
            return True
        else:
            return False

    def getAddressBookSync( self ):
        '''request the address book return and put a callback'''
    
        # TODO: do we really need to retrieve the AB??
        return
    
        self.socket.debug( 'request Address book' )

        request = SoapManager.SoapRequest( 'http://www.msn.com/webservices/AddressBook/ABFindAll',\
                                'contacts.msn.com', 443,\
                                '/abservice/abservice.asmx' , XmlTemplates.addressBook )

        SoapManager.put( request )
        response = SoapManager.get()

        self.socket.debug( str( response.getStatus() ) )

        if response.getStatus()[ 0 ] == 200:
            self.addressBookXML = response.getBody()
            self.newCacheFile( self._user + '_ab.xml', self.addressBookXML )

    def getAddressBookAsync( self ):
        '''request the address book dont return until we get it'''
        
        # TODO: do we really need to retrieve the AB??
        return

        self.socket.debug( 'request Address book' )

        request = SoapManager.SoapRequest( 'http://www.msn.com/webservices/AddressBook/ABFindAll',\
                                'contacts.msn.com', 443,\
                                '/abservice/abservice.asmx' , XmlTemplates.addressBook, self.onGetAddressBook )

        SoapManager.put( request )

    def onGetAddressBook( self, response ):
        self.socket.debug( str( response.getStatus() ) )

        if response.getStatus()[ 0 ] == 200:
            self.addressBookXML = response.getBody()
            self.newCacheFile( self._user + '_ab.xml', self.addressBookXML )

    def getDynamicItemsSync( self ):
        '''request dynamic items (spaces and all that) dont return until we get that'''

        self.socket.debug( 'request Dynamic Items' )

        request = SoapManager.SoapRequest( 'http://www.msn.com/webservices/AddressBook/ABFindAll',\
                                'contacts.msn.com', 443,\
                                '/abservice/abservice.asmx' , XmlTemplates.dynamicItems )

        SoapManager.put( request )
        response = SoapManager.get()

        return self.onGetDynamicItems( response )

    def getDynamicItemsAsync( self ):
        '''request dynamic items (spaces and all that) return and put a callback'''

        self.socket.debug( 'request Dynamic Items' )

        request = SoapManager.SoapRequest( 'http://www.msn.com/webservices/AddressBook/ABFindAll',\
                                'contacts.msn.com', 443, '/abservice/abservice.asmx', \
                                     XmlTemplates.dynamicItems, self.onGetDynamicItems )

        SoapManager.put( request )

    def onGetDynamicItems( self, response ):
        self.socket.debug( str( response.getStatus() ) )

        if response.getStatus()[ 0 ] == 200:
            self.dynamicItemsXML = response.getBody()
            self.newCacheFile( self._user + '_di.xml', self.dynamicItemsXML )

            self.contactManager.setDynamicItemsXml( self.dynamicItemsXML )
            self.getNickFromDynamicItems()
            self.emit( 'user-list-change' )
            
            return True
        else:
            return False

    def getStorageAuthCache( self, email ):
        self.socket.debug( 'request storage auth cache ' + email )
        
        request = SoapManager.SoapRequest( 'http://www.msn.com/webservices/storage/w10/GetItemVersion',\
                                'storage.msn.com', 80, '/storageservice/schematizedstore.asmx', \
                                     XmlTemplates.schematizedStore%(email), self.onGetStorageAuthCache )

        SoapManager.put( request )

    def onGetStorageAuthCache( self, response ):
        print str( response.getStatus() ) + "\n" + response.getBody()

    def getUserProfile( self, email, storageAuthCache = None ):
        if storageAuthCache == None:
            self.getStorageAuthCache( email )
        else:
            self.socket.debug( 'request user profile ' + email )

            cid = str(self.contactManager.getContact( email ).cid)
            request = SoapManager.SoapRequest( 'http://www.msn.com/webservices/spaces/v1/GetXmlFeed',\
                                'cid-580B3F2402D4CDA4.cc.services.spaces.live.com', 80, '/contactcard/contactcardservice.asmx', \
                                     XmlTemplates.space%(self.t,"",cid,storageAuthCache,"en-US"), self.onGetUserProfile )

            SoapManager.put( request )
        
    def onGetUserProfile( self, response ):
        print str( response.getStatus() ) + "\n" + response.getBody()

    def getNickFromDynamicItems( self ):
        # get the nick from the XML
        try:
            temp = self.dynamicItemsXML.split( '<contactType>Me</contactType>' )\
                    [ 1 ].split( '</displayName>' )[ 0 ]
            nick = unescape( temp.split( '<displayName>' )[ 1 ] )
        except:
            nick = self._user
            
        if not nick == self.nick:
            oldNick = self.nick
            self.nick = nick
            self.emit( 'self-nick-changed', oldNick, self.nick )

    def process( self ):
        '''read a command from the server and process it'''

        SoapManager.process()
        
        # call the process method of every switchboard
        for i in self.switchboards.iterkeys():
            if self.switchboards[ i ]._status != 'closed':
                self.switchboards[ i ].process()
        
        #check connection
        if self.socket.lastpng == 0:
            self.ping()
        elif not self.checkConnection():
            self.emit( 'connection-problem' )
            print "connection problem"
        
        if (int( time.time() ) - self.socket.lastpng) > self.socket.SECS_BETWEEN_PINGS:
            self.ping()
            
        fds = self.socket.select()

        if len( fds[ 0 ] ) == 1:
            try:
                (command, tid, params) = self.socket.receiveCommand()
            except IOError:
                self.emit( 'connection-closed' )
        else:
            return
        
        if len( fds[ 1 ] ) == 1:
            if command == 'CHL':
                self.emit( 'challenge', command, tid, params )

        if command == 'ILN':
            self.emit( 'initial-status-change', command, tid, params )
        elif command == 'NLN':
            self.emit( 'status-online', command, tid, params )
        elif command == 'FLN':
            self.emit( 'status-offline', command, tid, params )
        elif command == 'RNG':
            self.emit( 'switchboard-invitation', command, tid, params )
        elif command == 'ADL':
            if params != 'OK':
                payload = self.socket.receivePayload( int( params ) )
                self.emit( 'add-notification', command, tid, params, payload )
        elif command == 'RML':
            if params != 'OK':
                payload = self.socket.receivePayload( int( params ) )
                self.emit( 'remove-notification', command, tid, params, payload )
        elif command == 'UBX':
            payload = self.socket.receivePayload( int( params ) )
            self.socket.debug( '<<< ' + payload )
            if int( params ) > 0:
                try:
                    pm = payload.split( '<PSM>' )[ 1 ].split( '</PSM>' )[ 0 ]
                    pm = unescape( pm )
                    self.contactManager.setContactPersonalMessage( tid, pm )
                    self.emit( 'user-list-change' )
                    
                    media = payload.split( '<CurrentMedia>' )[ 1 ].split( '</CurrentMedia>' )[ 0 ]

                    mhead = media.find('\\0Music\\01\\0')
                    if mhead != -1:
                        media = '♫ ' + media[mhead+12:]
                        margs = media.split('\\0')
                        media = margs[0]
                        for args in range(1, len(margs)):
                            media = media.replace( '{%s}' %str(args-1), margs[args])
                        media = unescape(media)
                        self.contactManager.setContactPersonalMessage( tid, media )
                        self.emit('personal-message-changed', tid, media)
                        self.emit( 'user-list-change' )
                    else:
                        self.contactManager.setContactPersonalMessage( tid, pm )
                        self.emit('personal-message-changed', tid, pm)
                        self.emit( 'user-list-change' )
                except Exception, e:
                    #raise ProtocolError, 'Unable to handle UBX: ' + str( params )
                    print 'Unable to handle UBX: ' + str( params )
        elif command == 'XFR':
            #XFR 32 SB 65.54.171.31:1863 CKI 1743299383.52212212.219110167 U messenger.msn.com\r\n
            self.switchboards[ int( tid ) ].setConnectionString( string.join( [ command, tid, params ] ) )
        elif command == 'MSG':
            try:
                payload = self.socket.receivePayload( int( params.split()[ 1 ] ) )
                self.emit( 'server-message', command, tid, params, payload )

            except Exception, e:
                debugmsg( 'Exception in msnp.process, continuing' )
                debugmsg( '(EE) ' + str( e ) )
        elif command == 'OUT':
            self.emit( 'user-disconnected', tid, params )
        elif command == 'QNG':
           self.socket.onQng(tid)
            
    def changeStatus( self , status ):
        '''change the current status'''
        
        if status == 'FLN' or status == 'offline':
            return
        
        msnobj = ''
        
        if self.msnobj:
            msnobj = ' ' + self.msnobj.quote()
        else:
            msnobj = ' 0'
        print msnobj
        clientid = ' ' + str(self.CLIENT_ID)

        if status_table.has_key( status ):
            self.socket.sendCommand( "CHG" , status_table[ status ] + clientid + msnobj )
            self.status = status_table[ status ]
            self.emit( 'self-status-changed', self.status )
        elif reverse_status.has_key( status ):
            self.socket.sendCommand( "CHG" , status + clientid + msnobj )
            self.status = status
            self.emit( 'self-status-changed', self.status )

    def sendDL( self,command, email, type ):
        '''send ADL or RML
        #values of l
        #1 = Forward List
        #2 = Allow List
        #4 = Block List
        #8 = Reverse List (not to be tampered with)
        #16 = Pending list

        FL Forward List Users who were added to your contact list
        RL Reverse List Users who added you to their contact list
        AL Allow List Users who are able to see your status
        BL Block List Users who are blocked from seeing your status'''

        try:
            (user, domain) = email.split( '@' )
        except:
            return

        payload = '<ml><d n="' + domain + '"><c n="' + user + '" t="1" l="' + str( type ) + '" /></d></ml>'

        self.socket.sendPayloadCommand( command , '' , payload  )

    def addUser( self, email, group ):
        '''add an user to the friend list'''

        if self.contactManager.getContact( email ) != None:
            # We already have that contact
            return

        self.sendDL( 'ADL', email, '1' )
        self.sendDL( 'ADL', email, '2' )

        # need to add it in prevention of ILN. Don't update user list
        self.contactManager.addNewContact( email )
        
        request = SoapManager.SoapRequest( 'http://www.msn.com/webservices/AddressBook/ABContactAdd',\
                            'omega.contacts.msn.com', 443,\
                            '/abservice/abservice.asmx', \
                            XmlTemplates.contactAdd%(email,), \
                            self.onUserAdded, ( email, group ) )

        SoapManager.put( request )

    def onUserAdded( self, response, email, group='' ):
        '''
        this method is called when the addUser soapRequest get a response
        '''

        self.socket.debug( 'add user: ' + email + ' ' + str( response.getStatus() ) )
        
        if response.getStatus()[ 0 ] == 200:
            # Now that we have the contact id, we show it
            self.contactManager.setContactIdXml( email, response.getBody() )
            if group == '':
                self.emit( 'user-list-change' )
            else:
                # once we now the id we can add it to a group
                self.addUserToGroup( email, group )
        else:
            self.contactManager.removeContact( email )
            self.emit( 'user-list-change' )
            # TODO: popup sth instead of exception
            raise SoapError, ( 'User could not be added:\n', response )

    def removeUserFromGroup( self, user, group ):
        '''remove user from a group'''

        contactID = self.contactManager.getContactId( user )
        sourceGid = self.contactManager.getGroupId( group )

        if sourceGid == '' or sourceGid == 'nogroup':
            return
        
        self.contactManager.removeUserFromGroup( user, sourceGid )
        self.emit( 'user-list-change')

        request = SoapManager.SoapRequest( 'http://www.msn.com/webservices/AddressBook/ABGroupContactDelete',\
                                'omega.contacts.msn.com', 443,\
                                '/abservice/abservice.asmx', XmlTemplates.deleteUserFromGroup%(contactID,sourceGid), self.onUserRemovedFromGroup, (sourceGid,) )

        SoapManager.put( request )


    def onUserRemovedFromGroup( self, response, groupId ):
        self.socket.debug( 'remove user from group: ' + str( response.getStatus() ) )

        if response.getStatus()[ 0 ] == 200:
            pass
        else:
            self.contactManager.addUserToGroup( user, groupId )
            self.emit( 'user-list-change')
            # TODO: manage this situation instead of raising an error
            raise SoapError, ( 'User could not be removed:\n', response )


    def addUserToGroup( self, user, group ):
        '''add a user to a group'''

        gid = self.contactManager.getGroupId( group )
        contactID = self.contactManager.getContactId( user )
        if gid == None:
            self.socket.debug( 'Group not found' )
            return
        if gid in self.contactManager.getContact( user ).groups:
            self.socket.debug( 'User already in group' )
            return
        if gid == 'nogroup':
            self.socket.debug( 'Cannot move to no group' )
            return            
        
        self.contactManager.addUserToGroup( user, gid )
        self.emit( 'user-list-change')

        request = SoapManager.SoapRequest( 'http://www.msn.com/webservices/AddressBook/ABGroupContactAdd',\
                                'omega.contacts.msn.com', 443,\
                                '/abservice/abservice.asmx', \
                                XmlTemplates.moveUserToGroup%(gid, contactID), \
                                self.onUserAddedToGroup,( user, gid) )

        SoapManager.put( request )

    def onUserAddedToGroup( self, response, user, groupId ):
        self.socket.debug( 'add user to group: ' + str( response.getStatus() ) )

        if response.getStatus()[ 0 ] == 200:
            pass
        else:
            self.contactManager.removeUserFromGroup( user, groupId )
            self.emit( 'user-list-change')
            # TODO: manage this situation instead of raising an error
            raise SoapError, ( 'User could not be added:\n', response )

    def moveUserToGroup( self, user, srcGroup, destGroup, stage=0 ):
        '''move a user from a group erasing it from de source group'''

        if stage == 0:
            contactID = self.contactManager.getContactId( user )
            sourceGid = self.contactManager.getGroupId( srcGroup )
            destGid = self.contactManager.getGroupId( destGroup )
            
            # check whether or not it's an allowed movement
            if srcGroup == destGroup:
                self.socket.debug( 'src and dest groups are the same' )
                return
            elif self.contactManager.getGroup( destGid ).getUser( user ) != None:
                self.socket.debug( 'dest group already contain the user' )
                return
            
            # moving to/from No group = adding to destGroup / removing from srcGroup
            if sourceGid == 'nogroup':
                self.addUserToGroup( user, destGroup)
                return
            if destGid == 'nogroup':
                self.removeUserFromGroup( user, srcGroup )
                return
            
            # make the visual changes
            self.contactManager.removeUserFromGroup( user, sourceGid )
            self.contactManager.addUserToGroup( user, destGid )
            self.emit( 'user-list-change')
    
            # remove the user from the srcGroup
            request = SoapManager.SoapRequest( 'http://www.msn.com/webservices/AddressBook/ABGroupContactDelete',\
                                    'omega.contacts.msn.com', 443,\
                                    '/abservice/abservice.asmx', \
                                    XmlTemplates.deleteUserFromGroup%(contactID,sourceGid), \
                                    self.onMoveUserToGroup, \
                                    ( user, srcGroup, destGroup, 0 ) )
            SoapManager.put( request )
        
        elif stage == 1:
            gid = self.contactManager.getGroupId( destGroup )
            contactID = self.contactManager.getContactId( user )
        
            # add the user to the destGroup
            request = SoapManager.SoapRequest( 'http://www.msn.com/webservices/AddressBook/ABGroupContactAdd',\
                                'omega.contacts.msn.com', 443,\
                                '/abservice/abservice.asmx', \
                                XmlTemplates.moveUserToGroup%(gid, contactID), \
                                self.onMoveUserToGroup,\
                                ( user, srcGroup, destGroup, 1 ) )
            SoapManager.put( request )
            
    def onMoveUserToGroup( self, response, user, srcGroup, destGroup, stage ):
        
        self.socket.debug( 'move user (stage ' + str( stage ) + '): ' \
                           + str( response.getStatus() ) )
        
        status = response.getStatus()
        
        if response.getStatus()[ 0 ] == 200:
            if stage == 0: # continue the moving procedure
                self.moveUserToGroup( user, srcGroup, destGroup, 1 )
        else:
            # restore the old visual
            sourceGid = self.contactManager.getGroupId( srcGroup )
            destGid = self.contactManager.getGroupId( destGroup )
            
            self.contactManager.removeUserFromGroup( user, destGid )
            self.contactManager.addUserToGroup( user, sourceGid )
            self.emit( 'user-list-change')
            # TODO: manage this situation instead of raising an error
            raise SoapError, ( 'User could not be moved:\n', response )

    def removeUser( self, email ):
        '''remove an user from the friendr list'''

        self.sendDL( 'RML', email, '1' )
        
        contact = self.contactManager.getContact( email )
        contactID = self.contactManager.getContactId( email )
        
        # make the contact not visible
        self.contactManager.removeContact( email )
        self.emit( 'user-list-change')

        request = SoapManager.SoapRequest( 'http://www.msn.com/webservices/AddressBook/ABContactDelete',\
                                'omega.contacts.msn.com', 443,\
                                '/abservice/abservice.asmx', \
                                XmlTemplates.contactRemove%(contactID,), \
                                self.onUserRemoved, ( email, contact) )
        SoapManager.put( request )

    def onUserRemoved( self, response, email, contact ):
        '''this method is called when the removeUser soapRequest get a response
        renes means if we have to renew the di,ml,ab data'''

        self.socket.debug( 'remove user: ' + email + ' ' + str( response.getStatus() ) )
        
        if response.getStatus()[ 0 ] == 200:
            pass
        else:
            # TODO: probably need to send an ADL or sth since
            # the removal has failed
            self.contactManager.addContact( contact )
            self.emit( 'user-list-change' )
                
            # TODO: manage this situation instead of raising an error
            raise SoapError, ( 'User could not be removed:\n', response )

    def blockUser( self, email, stage=0 ):
        '''
        block an user
        '''

        if stage == 0:
            self.sendDL( 'RML', email, '2' )
            self.sendDL( 'ADL', email, '4' )
    
            self.contactManager.blockContact( email )
            self.emit( 'user-list-change' )
            
            request = SoapManager.SoapRequest( 'http://www.msn.com/webservices/AddressBook/DeleteMember',\
                                    'omega.contacts.msn.com', 443,\
                                    '/abservice/SharingService.asmx', \
                                    XmlTemplates.deleteMember%('BlockUnblock', 'Allow', email), \
                                    self.onUserBlocked, \
                                    ( email, 0 ) )
            SoapManager.put( request )
            
        elif stage == 1:
            request = SoapManager.SoapRequest( 'http://www.msn.com/webservices/AddressBook/AddMember',\
                                    'omega.contacts.msn.com', 443,\
                                    '/abservice/SharingService.asmx', \
                                    XmlTemplates.addMember%('BlockUnblock', 'Block', email), \
                                    self.onUserBlocked, \
                                    ( email, 1 ) )
            SoapManager.put( request )

    def onUserBlocked( self, response, email, stage ):
        '''
        this method is called when the blockUser soapRequest get a response
        '''

        self.socket.debug( 'block user (stage ' + str( stage ) + '): ' \
                           + str( response.getStatus() ) )

        if response.getStatus()[ 0 ] == 200:
            if stage == 0: # continue the blocking process
                self.blockUser( email, 1 )
        else:
            self.unblockUser( email, 0 )
            self.contactManager.unblockContact( email )
            self.emit( 'user-list-change' )
            

    def unblockUser( self, email, stage=0 ):
        '''
        unblock an user
        '''

        if stage == 0:
            self.sendDL( 'RML', email, '4' )
            self.sendDL( 'ADL', email, '2' )
    
            self.contactManager.unblockContact( email )
            self.emit( 'user-list-change' )
    
            request = SoapManager.SoapRequest( 'http://www.msn.com/webservices/AddressBook/DeleteMember',\
                                    'omega.contacts.msn.com', 443,\
                                    '/abservice/SharingService.asmx', \
                                    XmlTemplates.deleteMember%('BlockUnblock', 'Block', email), \
                                    self.onUserUnblocked, \
                                    ( email, 0 ) )
    
            SoapManager.put( request )
        
        elif stage == 1:
            request = SoapManager.SoapRequest( 'http://www.msn.com/webservices/AddressBook/AddMember',\
                                    'omega.contacts.msn.com', 443,\
                                    '/abservice/SharingService.asmx', \
                                    XmlTemplates.addMember%('BlockUnblock', 'Allow', email), \
                                    self.onUserUnblocked, \
                                    ( email, 1 ) )
            SoapManager.put( request )

    def onUserUnblocked( self, response, email, stage ):
        '''
        this method is called when the unblockUser soapRequest get a response
        '''

        self.socket.debug( 'unblock user (stage ' + str( stage ) + '): ' \
                           + str( response.getStatus() ) )
        
        if response.getStatus()[ 0 ] == 200:
            if stage == 0:
                self.unblockUser( email, 1 )
        else:
            self.contactManager.blockContact( email )
            self.emit( 'user-list-change' )

    def addGroup( self, group ):
        '''
        add a group to the group list
        '''

        if self.contactManager.getGroupId( group ) != None:
            self.socket.debug( 'Unable to add: Group "' + group \
                               + '" already exists' )
            return

        name = group.replace( ' ' , '%20' )

        request = SoapManager.SoapRequest(
                    'http://www.msn.com/webservices/AddressBook/ABGroupAdd',\
                    'omega.contacts.msn.com', 443,\
                    '/abservice/abservice.asmx',
                    XmlTemplates.addGroup%(group,),
                    self.onGroupAdded, (group,) )
        SoapManager.put( request )

    def onGroupAdded( self, response, group ):
        '''
        this method is called when the addGroup soap request get a response
        '''

        self.socket.debug( 'add group ' + str( response.getStatus() ) )

        if response.getStatus()[ 0 ] == 200:
            try:
                gid = response.getBody().split( '<guid>' )[ 1 ].split( '</guid>' )[ 0 ]
                self.contactManager.addGroup( group, gid )
                self.emit( 'user-list-change' )
            except IndexError, e:
                self.socket.debug( 'cannot add group to userlist' )
                self.socket.debug( str( e ) )
        else:
            # TODO: manage this situation instead of raising an error
            raise SoapError, ( 'Group could not be added:\n', response )

    def removeGroup( self, group ):
        '''
        remove a group from the group list
        '''

        gid = self.contactManager.getGroupId( group )
        if gid:
            groupObj = self.contactManager.getGroup( gid )
            self.contactManager.removeGroup( gid )
            self.emit( 'user-list-change' )
            
            request = SoapManager.SoapRequest( 'http://www.msn.com/webservices/AddressBook/ABGroupDelete',\
                                'omega.contacts.msn.com', 443,\
                                '/abservice/abservice.asmx', \
                                XmlTemplates.deleteGroup%(gid,), \
                                self.onGroupRemoved, \
                                ( gid, groupObj ) )

            SoapManager.put( request )
        else:
            self.socket.debug( 'Unable to remove: Group "' + group \
                               + '" does not exist')

    def onGroupRemoved( self, response, gid, group ):
        '''
        this method is called when the removeGroup soap request get a response
        '''

        self.socket.debug( 'remove group ' + str( response.getStatus() ) )
        
        if response.getStatus()[ 0 ] == 200:
            pass
        else:
            # TODO: change it to setGroup when it manages contacts
            self.contactManager.setGroup( gid, group )
            self.emit( 'user-list-change' )
            # TODO: manage this situation instead of raising an error
            raise SoapError, ( 'Group could not be removed:\n', response )

    def renameGroup( self, oldGroup, newGroup ):
        '''
        rename a group from the group list
        '''

        if oldGroup == newGroup:
            self.socket.debug( 'oldgroup and new group are the same' )
            return
        if self.contactManager.getGroupId( newGroup ) != None:
            self.socket.debug( 'That group name is already in use' )
            return
            
        gid = self.contactManager.getGroupId( oldGroup )
        if gid == None:
            self.socket.debug( 'The specified group does not exist' )
            return
        else:
            self.contactManager.renameGroup( gid, newGroup )
            self.emit( 'user-list-change' )
            request = SoapManager.SoapRequest( 'http://www.msn.com/webservices/AddressBook/ABGroupUpdate',\
                                'omega.contacts.msn.com', 443,\
                                '/abservice/abservice.asmx', \
                                XmlTemplates.renameGroup%(gid,newGroup), \
                                self.onGroupRenamed, (oldGroup, newGroup) )
            SoapManager.put( request )

    def onGroupRenamed( self, response, oldGroup, newGroup ):
        '''
        this method is called when the renameGroup soap request get a response
        '''

        self.socket.debug( 'rename group ' + str( response.getStatus() ) )
        
        if response.getStatus()[ 0 ] == 200:
            pass
        else:
            gid = self.contactManager.getGroupId( newGroup )
            self.contactManager.renameGroup( gid, oldGroup )
            self.emit( 'user-list-change' )
            # TODO: manage this situation instead of raising an error
            raise SoapError, ( 'Group could not be renamed:\n', response )

    def logout( self ):
        '''disconnect from the server'''
        self.emit( 'logout' )
        try:
            self.socket.sendText( 'OUT\r\n' )
        except:
            pass

        self.contactManager = ContactData.ContactList( {} )
        # Send the soapmanager a stop signal
        SoapManager.put( 'quit' )
        SoapManager.process()
        #self.soapManager.join()
        
    def createADL( self ):
        '''
        Create a XML String with all the contacts we have for the ADL Command
        '''

        return self.contactManager.getADL()

    def changeNick( self, nick, user='Me' ):
        '''
        change the nick, type can be Me or Normal
        '''

        if user == 'Me':
            oldNick = self.nick
            
            if len(nick) > 129:
                # to avoid problems with utf-8
                return
            
            oldNick = self.nick
            self.nick = nick
            #self.contactManager.setContactNick( self._user, self.nick )
            self.emit( 'self-nick-changed', oldNick, self.nick )
            
            self.socket.sendCommand( "PRP" , "MFN " + urllib.quote( nick ) )

            nick = escape( nick )
            request = SoapManager.SoapRequest( 'http://www.msn.com/webservices/AddressBook/ABContactUpdate',\
                                'omega.contacts.msn.com', 443,\
                                '/abservice/abservice.asmx', \
                                XmlTemplates.changeNick%(user, nick), \
                                self.onNickChanged, ( self._user, oldNick ) )
        else:
            oldAlias = self.contactManager.getContactAlias( user )
            
            self.contactManager.setContactAlias( user, nick )
            self.emit( 'user-list-change' )

            contactID = self.contactManager.getContactId( user )
            nick = escape( nick )
            request = SoapManager.SoapRequest( 'http://www.msn.com/webservices/AddressBook/ABContactUpdate',\
                                'omega.contacts.msn.com', 443,\
                                '/abservice/abservice.asmx', \
                                XmlTemplates.renameContact%(contactID, nick), \
                                self.onNickChanged, ( user, oldAlias ) )

        SoapManager.put( request )

    def onNickChanged( self, response, user, oldNick ):
        '''
        this method is called when the changeNick soap request get a response
        '''

        self.socket.debug( 'change nick ' + str( response.getStatus() ) )
        
        if response.getStatus()[ 0 ] == 200:
            pass
        else:
            if user == 'Me':
                #self.contactManager.setContactNick( self._user, oldNick )
                self.emit( 'self-nick-changed', oldNick, self.nick )
            else:
                self.contactManager.setContactAlias( user, oldNick )
                self.emit( 'user-list-change' )
                
            # TODO: manage this situation instead of raising an error
            raise SoapError, ( 'Nick could not be changed:\n', response )

    def changePersonalMessage( self, pm ):
        '''change the personal message'''

        self.socket.sendPayloadCommand( 'UUX', '', '<Data><PSM>' + escape( pm )
                    + '</PSM><CurrentMedia>' + escape( self.currentMedia ) +
                    '</CurrentMedia><MachineGuid></MachineGuid></Data>' )
        
        self.personalMessage = pm
        self.emit( 'self-personal-message-changed', self._user, pm )
        
    def changeCurrentMedia( self, cm ):
        '''change the current media'''

        self.socket.sendPayloadCommand( 'UUX', '', '<Data><PSM>' + self.personalMessage + '</PSM><CurrentMedia>' + cm + '</CurrentMedia><MachineGuid></MachineGuid></Data>' )
        self.currentMedia = cm
        self.emit( 'self-current-media-changed', self._user, cm )
        
    def newSwitchboard( self ):
        '''
        create a new Switchboard and return it
        '''
        tid = self.socket.getTid()
        self.switchboards[ tid ] = Switchboard.Switchboard( tid, self, 'requested' )
        self.socket.sendCommand( 'XFR' , 'SB' )
        
        return self.switchboards[ tid ]

    def getSwitchboard( self, email ):
        '''try to retrieve an existing switchboard that has email as the first
        user and its not a group chat'''
        
        for sb in self.switchboards.values():
                if not sb.isGroupChat() and sb.getFirstUser() == email:
                    return sb

        return None

    def getGroups( self ):
        '''return the groups in the userlist'''

        return self.contactManager.getGroups()
    
    def getGroupNames(self):
        '''
        Returns a list with the group names in the user list
        '''
        
        return self.contactManager.getGroupNames()

    def checkPending( self ):
        '''Return all the pending users'''

        l = []
        lists = self.contactManager.lists
        for email in lists[ 'Pending' ]:
            if email not in lists[ 'Block' ] and email not in lists[ 'Allow' ]:
                l.append( email )

        return l

    def cacheFileExist( self, fileName ):
        '''return true if the file exist in the cache directory'''

        return os.path.isfile( self._cacheDir + os.sep + fileName )

    def newCacheFile( self, fileName, data ):
        '''create the file and put data on it'''

        f = file( self._cacheDir + os.sep + fileName, 'w' )
        f.write( data )
        f.close()

    def getCacheFileData( self, fileName ):
        '''get the data in filename, return None otherwise'''

        if self.cacheFileExist( fileName ):
            f = file( self._cacheDir + os.sep + fileName, 'r' )
            data = f.read()
            f.close()
            return data
        else:
            return None

    def getUserConfigDir( self ):
        return self._userConfigDir
     
    def setDisplayPicture(self, filename):
        '''try to open the picture and set the msnobj'''
        if filename == '':
            self.msnobj = None
        else:
            self.msnobj = self.msnObjectsManager.create( '', filename, type=Msnobj.Msnobj.DISPLAY_PICTURE )
            
        if self.status != 'FLN':
            self.changeStatus( self.status )
            
    def createCustomEmoticon( self, shortcut, filename ):
        self.msnObjectsManager.create( shortcut, filename, type=Msnobj.Msnobj.CUSTOM_EMOTICON )
        
    def getMsnObjectsManager( self ):
        return self.msnObjectsManager
    
    def getCacheDir( self ):
        return self._cacheDir
        
    def getUser( self ):
        return self._user
    
    def getNick( self ):
        return self.nick

dic = {
    '\"'    :    '&quot;',
    '\''    :    '&apos;'
}

dic_inv = {
    '&quot;'    :'\"',
    '&apos;'    :'\''
}

def escape( str ):
    return xml.sax.saxutils.escape( str, dic )

def unescape( str ):
    return xml.sax.saxutils.unescape( str, dic_inv )

def dumpXmlToFile( string, fileName ):
    import xml
    dom = xml.dom.minidom.parseString( string )
    f = file( fileName, 'w' )
    f.write( dom.toprettyxml() )
    f.close()

class LoginError( Exception ):
    def __init__( self, value ):
        self.value = value
        
    def __str__( self ):
        return "Couldn't perform login: " + repr( self.value )
    
class AuthError( Exception ):
    def __init__( self, value ):
        self.value = value
        
    def __str__( self ):
        return 'Authentication error: ' + repr( self.value )
    
class RedirectionError( Exception ):
    def __init__( self, value ):
        self.value = value
        
    def __str__( self ):
        return 'Redirection error: ' + repr( self.value )
    
class ProtocolError( Exception ):
    def __init__( self, value ):
        self.value = value
        
    def __str__( self ):
        return 'Protocol error: ' + repr( self.value )
    
class SoapError( Exception ):
    def __init__( self, value, response ):
        self.value = value
        
        from xml.dom import minidom
        self.responseStatus = response.getStatus()
        body = response.getBody()
        
        ml = minidom.parseString( body )
        fault = ml.getElementsByTagName( 'soap:Fault' )[ 0 ]
        try:
            self.faultcode = fault.getElementsByTagName( 'errorstring' )[ 0 ].childNodes[ 0 ].data
        except IndexError:
            self.faultcode = fault.getElementsByTagName( 'faultstring' )[ 0 ].childNodes[ 0 ].data
        
    def __str__( self ):
        return repr( self.value ) + ' ' + repr( self.responseStatus ) \
                    + ': ' + repr( self.faultcode )

DEBUG = 5

def debugmsg( arg, level=5 ):
    if level <= DEBUG:
        print ' ' * level + str(arg)
        
def errormsg( arg ):
    sys.stderr.write( str( arg )  + '\n')

gobject.type_register(Msnp)

