# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import re
from operator import truediv
from emesenecommon import debugmsg

colorCodes = {}
colorCodes[ '0' ] = 'ffffff'
colorCodes[ '1' ] = '000000'
colorCodes[ '2' ] = '00007D'
colorCodes[ '3' ] = '009100'
colorCodes[ '4' ] = 'FF0000'
colorCodes[ '5' ] = '7D0000'
colorCodes[ '6' ] = '9A009A'
colorCodes[ '7' ] = 'FC7D00'
colorCodes[ '8' ] = 'FFFF00'
colorCodes[ '9' ] = '00FC00'
colorCodes[ '10' ] = '009191'
colorCodes[ '11' ] = '00FFFF'
colorCodes[ '12' ] = '1E1EFC'
colorCodes[ '13' ] = 'FF00FF'
colorCodes[ '14' ] = '7D7D7D'
colorCodes[ '15' ] = 'D2D2D2'
colorCodes[ '16' ] = 'E7E6E4'
colorCodes[ '17' ] = 'cfcdd0'
colorCodes[ '18' ] = 'ffdea4'
colorCodes[ '19' ] = 'ffaeb9'
colorCodes[ '20' ] = 'ffa8ff'
colorCodes[ '21' ] = 'c1b4fc'
colorCodes[ '22' ] = 'bafbe5'
colorCodes[ '23' ] = 'ccffa3'
colorCodes[ '24' ] = 'fafda2'
colorCodes[ '25' ] = 'b6b4b7'
colorCodes[ '26' ] = 'a2a0a1'
colorCodes[ '27' ] = 'f9c152'
colorCodes[ '28' ] = 'ff6d66'
colorCodes[ '29' ] = 'ff62ff'
colorCodes[ '30' ] = '6c6cff'
colorCodes[ '31' ] = '68ffc3'
colorCodes[ '32' ] = '000000'
colorCodes[ '33' ] = 'f9ff57'
colorCodes[ '34' ] = '858482'
colorCodes[ '35' ] = '6e6d7b'
colorCodes[ '36' ] = 'ffa01e'
colorCodes[ '37' ] = 'F92411'
colorCodes[ '38' ] = 'FF1EFF'
colorCodes[ '39' ] = '1E29FF'
colorCodes[ '40' ] = '7dffa5'
colorCodes[ '41' ] = '60f913'
colorCodes[ '42' ] = 'fff813'
colorCodes[ '43' ] = '5e6464'
colorCodes[ '44' ] = '4b494c'
colorCodes[ '45' ] = 'd98812'
colorCodes[ '46' ] = 'eb0505'
colorCodes[ '47' ] = 'de00de'
colorCodes[ '48' ] = '0000d3'
colorCodes[ '49' ] = '03cc88'
colorCodes[ '50' ] = '59d80d'
colorCodes[ '51' ] = 'd4c804'
colorCodes[ '52' ] = '333335'
colorCodes[ '53' ] = '18171c'
colorCodes[ '54' ] = '944e00'
colorCodes[ '55' ] = '9b0008'
colorCodes[ '56' ] = '980299'
colorCodes[ '57' ] = '01038c'
colorCodes[ '58' ] = '01885f'
colorCodes[ '59' ] = '389600'
colorCodes[ '60' ] = '9a9e15'
colorCodes[ '61' ] = '473400'
colorCodes[ '62' ] = '4d0000'
colorCodes[ '63' ] = '5f0162'
colorCodes[ '64' ] = '000047'
colorCodes[ '65' ] = '06502f'
colorCodes[ '66' ] = '1c5300'
colorCodes[ '67' ] = '544d05'

colorCodesHex = re.compile('\[[cC]=#([0-9A-Fa-f]{6})\](.*?)\[/[cC]\]')
colorCodesCode = re.compile('\[[cC]=([0-9]{1,2})\](.*?)\[/[cC]\]')

backColorCodesHex = re.compile('\[[aA]=#([0-9A-Fa-f]{6})\](.*?)\[/[aA]\]')
backColorCodesCode = re.compile('\[[aA]=([0-9]{1,2})\](.*?)\[/[aA]\]')

#colorIrcCode = re.compile("·\$([0-9]{1,2}),([0-9]{1,2})|·\$([0-9]{1,2}).*?")
colorIrcCode = re.compile("·\$([0-9]{1,2})?,?([0-9]{1,2})?")
#colorIrcRGB = re.compile("·\$\(([0-9]{3}),([0-9]{3}),([0-9]{3})\)")

#don't try to pack them into one regexp. please.

formatBb = {}
formatIrc = {}

tmpBb = "bius"
tmpIrc = "#&@'"

for i in range(4):
    tag = tmpBb[i]
    doubleCase = (tag + tag.upper(), tag + tag.upper())
    doubleIrc = (tmpIrc[i],tmpIrc[i])
    formatBb[tag] = re.compile('(\[[%s]\](.*?)\[/[%s]\])' % doubleCase)
    formatIrc[tag] = re.compile("·(\%s)(.*?)(·\%s|$)" % doubleIrc)

span = re.compile('(<span.*?>)|(</span>)')

removeList = (colorCodesHex, colorCodesCode, backColorCodesHex, backColorCodesCode)

tagToHtml = {
    'b':'font-weight: bold',
    'u':'text-decoration:underline',
    'i':'font-style: italic',
    's':'text-decoration: line-through',
    'foreground':'color',
    'background':'background-color'
    }

tagToPangoSpan = {
    'b':'weight="bold"',
    'u':'underline="single"',
    'i':'style="italic"',
    's':'strikethrough="true"',
    'foreground':'foreground=',
}

#\xc2 is for utf-8..
customStatus = re.compile("^(.*)\xc2\xa0{([^}]*)}$")

def getCustomStatus(nick):
    '''parse custom status, return tuple (nick, status)'''
    result = customStatus.search(nick)
    if result:
        return result.groups()
    else:
        return (nick, '')

class msnPlusMarkup:
    def __init__( self ):
        self.isHtml = False

    def removeMarkup( self, text ):
        '''remove the [b][/b] etc markup for pango and html markup'''
        all = removeList + tuple(formatBb.values() + formatIrc.values())
        for i in all:
            text = re.sub( i, lambda x:x.groups()[1], text)
             
        text = re.sub( colorIrcCode, '', text )

        return text
        
    def replaceMarkup( self, text, isHtml = False ):
        '''replace the [b][/b] etc markup for pango and html markup'''
        
        text = text.replace('·&amp;','·&') #@ROGER: i hate this
        self.isHtml = isHtml
        self.more = 0
        self.backgroundColor = self.frontColor = ''
                
        self.openSpan = None
        def q(dictionary, tag, text): #quick handler
            handler = lambda x: self.toTag(x.groups()[1], tag)
            return re.sub(dictionary[tag], handler, text)
        for tag in 'bius':
            text = q(formatBb, tag, text)
            text = q(formatIrc, tag, text)

        text = re.sub( colorCodesHex, self.hexToTag, text )
        text = re.sub( colorCodesCode, self.codeToTag, text )

        text = re.sub( backColorCodesHex, self.bHexToTag, text )
        text = re.sub( backColorCodesCode, self.bCodeToTag, text )
        
        text = re.sub( colorIrcCode, self.ircCodeToTag, text ) 

        if self.openSpan != None:
            text += '</span>'
        
        return text.replace('·&','·&amp;')
        
    def toTag(self, data, tag):
        if self.isHtml:
            return '<span style="' + tagToHtml[tag] + '">' + data + '</span>'
        else:
            return '<span ' + tagToPangoSpan[tag] + '>' + data + '</span>'
        
    def ircCodeToTag( self, data ):
        text = None
        color1, color2 = data.groups()

        if self.frontColor != '' and color1 == None:
            color1 = self.frontColor

        if self.backgroundColor != '' and  color2 == None:
            color2 = self.backgroundColor
        
        self.frontColor = color1
        self.backgroundColor = color2

        if color1 != None and color2 != None and colorCodes.has_key(color1) and colorCodes.has_key(color2):
            fghex=colorCodes[color1].lower()
            bghex=colorCodes[color2].lower()
            if self.isHtml:
                text = '<span style="color:#' + fghex + '; background-color:#' + bghex + '">' 
            else:
                text = '<span foreground="#' + fghex + '" background="#' + bghex + '">'

        elif color1 != None and colorCodes.has_key(color1):
            fghex=colorCodes[color1].lower()
            if self.isHtml:
                text = '<span style="color:#' + fghex + '">' 
            else:
                text = '<span foreground="#' + fghex + '">'
                    
        elif color2 != None and colorCodes.has_key(color2):
            bghex=colorCodes[color2].lower()
            if self.isHtml:
                text = '<span style="background-color:#' + bghex + '">' 
            else:
                text = '<span background="#' + bghex + '">'
                    
        if self.openSpan != None:
            text = '</span>' + text

        if text != None:
            self.openSpan = True
            return text 
        
    def hexToTag( self, data ):
        color, text = data.groups()
        if self.isHtml:
            return '<span style="color: #' + color.lower() + '">' + text + '</span>'
        else:
            return '<span foreground="#' + color.lower() + '">' + text + '</span>'
  
    def codeToTag( self, data ):
        code, text = data.groups()
        if colorCodes.has_key(code):
            hex = colorCodes[code].lower()
            if self.isHtml:
                return '<span style="color: #' + hex + '">' + text + '</span>'
            else:
                return '<span foreground="#' + hex + '">' + text + '</span>'
        else:
            return text
            
    def bHexToTag( self, data ):
        color, text = data.groups()
        if self.isHtml:
            return '<span style="background-color: #' + color.lower() + '">' + text + '</span>'
        else:
            return '<span background="#' + color.lower() + '">' + text + '</span>'
        
    def bCodeToTag( self, data ):
        code, text = data.groups()
        if colorCodes.has_key(code):
            hex =  colorCodes[code].lower()
            if self.isHtml:
                return '<span style="background-color: #' + hex + '">' + text + '</span>'
            else:
                return '<span background="#' + hex + '">' + text + '</span>'
        else:
            return text     
            
    def RGBToHTMLColor( self, rgb_tuple ):
        """ convert an (R, G, B) tuple to #RRGGBB """
        return '#%02x%02x%02x' % rgb_tuple
       
    def HTMLColorToRGB( self, colorstring ):
        """ convert #RRGGBB to an (R, G, B) tuple """
        colorstring = colorstring.strip()
        if colorstring.startswith('#'): 
            colorstring = colorstring[1:]
            
        r, g, b = colorstring[:2], colorstring[2:4], colorstring[4:]
        r, g, b = [int(n, 16) for n in (r, g, b)]
        return (r, g, b)


if __name__=='__main__':
    parser=msnPlusMarkup()
    text = '[b][c=4]LOLOLOLOLOL!!!![/b][/c]'
    print "* Normal text with minor tag problem:",text
    print "Remove:", parser.removeMarkup(text)
    print "Replace:", parser.replaceMarkup(text)
    print
    text = '·# asd ·$4 whoaah·$ *meaningless ·&st·&ate·@·#men·@t*'
    print "* Similar problems, heavier irc tags:", text
    print "Remove:", parser.removeMarkup(text)
    print "Replace:", parser.replaceMarkup(text)
    print
    text = '[b][i][/b]·$4[/i]·#[c=4][u]·&[c=5][/u]·$2[/c]'
    print "* Terrible mess:", text
    print "Remove:", parser.removeMarkup(text)
    print "Replace:", parser.replaceMarkup(text)
    print

