# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
import os
import gobject

import Plugin
import currentSong

class MainClass( Plugin.Plugin ):
    '''Main plugin class'''
    
    def __init__( self, controller, msn ):
        '''Contructor'''
        
        Plugin.Plugin.__init__( self, controller, msn, 1000 )
        self.theme = controller.theme
        
        self.description = _( 'Show a personal message with the current song played in multiple players.' )
        self.authors = { 'Mariano Guerra' : 'luismarianoguerra at gmail dot com' }
        self.website = 'http://emesene.org'
        self.displayName = _( 'CurrentSong' )
        self.name = 'CurrentSong'
        
        self.config = controller.getConfig()
        self.config.readPluginConfig(self.name)
        self.usingDummy = False
        self.setPlayer( self.config.getPluginValue( self.name, 'player', 'CurrentSong' ) )
        self.player.setStyle( self.config.getPluginValue( self.name, 'style', '%title - %artist (%album)' ) )
        
        
    def start( self ):
        '''start the plugin'''
        
        gobject.timeout_add( self.period, self.action )
        self.enabled = True
        
    def stop( self ):    
        '''stop the plugin'''
        
        self.msn.changeCurrentMedia( "" )
        self.enabled = False
        
    def action( self ):
        if self.enabled and not self.usingDummy:
            if self.player.check() and self.controller.getMediaEnabled():
                self.msn.changeCurrentMedia( self.player.getCurrentSong() )
        else:
            self.setPlayer( self.config.getPluginValue( self.name, 'player', 'CurrentSong' ) )

        return True
        
    def is_on_path(self, fname):
        for p in os.environ['PATH'].split(os.pathsep):
            if os.path.isfile(os.path.join(p, fname)):
                return True
        
    def getPluginNames( self ):
        return [ x for x in dir( currentSong ) if not x.startswith( "__" ) ]
        
    def setPlayer( self, name ):
        '''try to set the player object to the name given by the name parameter
        if not set the dummy player to have something to play with'''
        
        players = self.getPluginNames()
        
        if name in players:
            try:
                # ugly but i dont have to make a dict with the classes :D
                self.player = eval( "currentSong." + name + "()" )
                self.usingDummy = False
            except Exception, ex:
                self.player = currentSong.CurrentSong()
                self.usingDummy = True
        
    def check( self ):
        '''check if everything is OK to start the plugin
        return a tuple whith a boolean and a message
        if OK -> ( True , 'some message' )
        else -> ( False , 'error message' )'''
        
        return self.player.getStatus()

    def configure( self ):
        '''display a configuration dialog'''
        # name, optionType, label, description, value, options
        l = [ Plugin.Option( 'style', str, 'Display style', 'Change display style of the song you are listening, possible variables are %title, %artist and %album', self.config.getPluginValue( self.name, 'style', '%title - %artist (%album)' ) ) ]
        l.append( Plugin.Option( 'player', list, 'Audio player', 'select the audio player that you are using', self.config.getPluginValue( self.name, 'player', 'Amarok' ), self.getPluginNames() ) )
        response = Plugin.ConfigWindow( _( 'Config Plugin' ), l ).run()
        if response != None:
            if response.has_key( 'player' ):
                self.setPlayer( response[ 'player' ].value )
                self.config.setPluginValue( self.name, 'player', response[ 'player' ].value )
                
            if response.has_key( 'style' ):
                self.player.setStyle( response[ 'style' ].value )
                self.config.setPluginValue( self.name, 'style', response[ 'style' ].value )

        # we have preferences so we return True
        return True
