# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

VERSION = '0.1'

import os
import gobject

try:
    import dbus
    DBUS = True
except:
    DBUS = False

import Plugin

from emesenecommon import LANG_PATH

import gettext

gettext.bindtextdomain('plugin_listen', LANG_PATH)
gettext.textdomain('plugin_listen')
_ = gettext.gettext


class ListenCom:
    ''' Communicate with GNOME's Listen player '''
    
    def __init__(self):
        self.artist =  self.title = self.album = ""
        style = "%title - %artist (%album)"
        
        self.setStyle(style)
        
        self.bus = dbus.SessionBus()
        self.mainbus = self.bus.get_object('org.freedesktop.DBus', '/org/freedesktop/DBus')
        
        
    def getCurrentSong(self):
        return self.style.replace('%artist', self.artist).replace('%title', self.title).replace('%album', self.album)    
        
    def setCurrentSongData(self):
        message = self.iface.current_playing()
        
        # process the received string. it's built this way: <m_track> - (<m_album> - <m_artist>)
        
        if message != "":
            buf = message.split(" - (")
            title = buf[0]
            secbuf = buf[1].split(" - ")
            album = secbuf[0]
            artist = secbuf[1][0:-1]
            
            if (title != self.title) or (album != self.album) or (artist != self.artist):
                self.title, self.album, self.artist = title, album, artist
                return True
            else:
                return False
            
    def isPlaying(self):
        try:
            self.iface = self.bus.get_object('org.gnome.Listen', '/org/gnome/listen')
            self.iface.current_playing()
        except:
            return False
        return True
        
    def isRunning(self):
        if self.mainbus.NameHasOwner('org.gnome.Listen'):
            return True
        else:
            return False
            
    def setStyle(self, string):
        ''' Set the message style '''
        self.style = "\\0Music\\01\\0" + string.replace('%title', '{0}').replace('%artist', '{1}').replace('%album', '{2}') + '\\0%title\\0%artist\\0%album\\0\\0'
        
class MainClass( Plugin.Plugin ):
    ''' Main plugin class - communicate with emesene this time :) '''
    
    def __init__( self, controller, msn ):
        '''Contructor'''
        
        Plugin.Plugin.__init__( self, controller, msn, 1000 )
        
        self.description = _( 'Show your contacts the song Listen is playing.' )
        self.authors = { 'Tomer Haimovich' : 'tomerha at gmail dot com' }
        self.website = 'http://emesene-im.com.ar'
        self.displayName = 'Listen'
        self.name = 'Listen'
        
        self.media = ''
        
        self.listen = ListenCom()
        
        self.config = controller.getConfig()
        self.config.readPluginConfig(self.name)
        self.listen.setStyle( self.config.getPluginValue(self.name, 'style', '%title - %artist (%album)'))
        
        
    def start(self):
        '''start the plugin'''
        gobject.timeout_add( self.period, self.action )
        self.enabled = True
        
    def stop(self):    
        '''stop the plugin'''
        
        self.msn.changeCurrentMedia("")
        self.enabled = False
        
    def action(self):
        if self.enabled:
            if self.controller.getMediaEnabled() and self.listen.isRunning():
                
                if self.listen.isPlaying():
                    if self.listen.setCurrentSongData():
                        self.media = self.listen.getCurrentSong()
                        self.msn.changeCurrentMedia( self.media )
            else:
                if self.media != '':
                    self.media = ''
                    self.msn.changeCurrentMedia('')
                
            return True
        else:
            return False
            
    def check( self ):
        '''
        check if everything is OK to start the plugin
        return a tuple whith a boolean and a message
        if OK -> ( True , 'some message' )
        else -> ( False , 'error message' )
        '''
        
        if os.name != 'posix':
            return ( False, _( 'This plugin only works in posix systems' ) )
        
        if not DBUS:
            return ( False, _( 'D-Bus cannot be initialized' ) )
        
        return ( True, 'Ok' )
        
    def configure( self ):
        '''display a configuration dialog'''
        # name, optionType, label, description, value, options
        l = []
        l.append( Plugin.Option( 'style', str, _( 'Display style' ), _( 'Change display style of the song you are listening, possible variables are %title, %artist and %' ), self.config.getPluginValue( self.name, 'style', '%title - %artist (%album)'  ) ) )
        
        response = Plugin.ConfigWindow( _( 'Config Listen Plugin' ), l ).run()
        if response != None:
            if response.has_key( 'style' ):
                self.listen.setStyle( response[ 'style' ].value )
                self.config.setPluginValue( self.name, 'style', response[ 'style' ].value )
