# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Foobar is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import os
import time
import string

import Plugin
class MainClass( Plugin.Plugin ):
    '''Main plugin class'''
    
    def __init__( self, controller, msn ):
        '''Contructor'''
        Plugin.Plugin.__init__( self, controller, msn )
        
        self.description = 'Log al events to a file'
        self.authors = { 'Mariano Guerra' : 'luismarianoguerra  at gmail dot com' }
        self.website = 'http://emesene.org'
        self.displayName =  'SickLogger'
        self.name = 'SickLogger'
        
        self.config = controller.getConfig()
        self.config.readPluginConfig(self.name)
        self.path = self.config.getPluginValue( self.name, 'path', self.msn.getCacheDir() )
        self.fileName = time.strftime( "%d-%m-%y" ) + ".log"
        self.fd = file( self.path + os.sep + self.fileName, "a" )
        self.ids = []
        
        self.signals = {}
        self.signals[ 'self-nick-changed' ] = self.cbSelfNickChanged
        self.signals[ 'connection-problem' ] = self.cbConnectionProblem
        self.signals[ 'disconnected' ] = self.cbDisconnected
        self.signals[ 'status-change' ] = self.cbStatusChange
        self.signals[ 'personal-message-changed' ] = self.cbPersonalMessageChanged
        self.signals[ 'self-status-changed' ] = self.cbSelfStatusChanged
        self.signals[ 'self-personal-message-changed' ] = self.cbSelfPersonalMessageChanged
        self.signals[ 'self-current-media-changed' ] = self.cbSelfCurrentMediaChanged
        self.signals[ 'new-mail-notification' ] = self.cbNewMailNotification
        self.signals[ 'nick-changed' ] = self.cbNickChanged
        self.signals[ 'contact-status-change' ] = self.cbContactStatusChange
        self.signals[ 'user-online' ] = self.cbUserOnline
        self.signals[ 'user-offline' ] = self.cbUserOffline
        
        allEvents = string.join( self.signals.keys(), "," )
        
        self.eventsEnabled =  self.config.getPluginValue( self.name, 'eventsEnabled', allEvents ).replace( " ", "" ).split( "," )
    
    def start( self ):
        '''start the plugin'''

        self.enabled = True
        
        for (key,value) in self.signals.iteritems():
            if key in self.eventsEnabled:
                self.ids.append( self.msn.connect( key, value ) )
        
    def stop( self ):    
        '''stop the plugin'''
        
        for id in self.ids:
            self.msn.disconnect( id )
        
        self.enabled = False
        
        self.fd.close()
        
    def action( self ):
        pass        
    
    def check( self ):
        '''check if everything is OK to start the plugin
        return a tuple whith a boolean and a message
        if OK -> ( True , 'some message' )
        else -> ( False , 'error message' )'''
        
        return ( True, 'Ok' )

    def quote( self, s ):
        return s.replace( " ", "%20" )
            
    def getTimeStamp( self ):
        return time.strftime( "%y-%m-%d %H:%M:%S" )
            
    def log( self, message ):
        self.fd.write( self.getTimeStamp() + " " + message + os.linesep )
        self.fd.flush()
            
# -------------------------- CALLBACKS -----------------------------------------( self, msn, ):

    def cbSelfNickChanged( self, msn, old, new ):
        self.log( "self-nick-changed " + self.quote( old ) + " " + self.quote( new ) )

    def cbConnectionProblem( self, msn ):
        self.log( "connection-problem" )

    def cbDisconnected( self, msn ):
        self.log( "disconnected" )

    def cbStatusChange( self, msn, command, tid, params ):
        self.log( "status-change " + command + " " + tid + " " + params )

    def cbPersonalMessageChanged( self, msn, email, pm ):
        if pm != "":
            self.log( "personal-message-changed " + email + " " + self.quote( pm ) )

    def cbSelfStatusChanged( self, msn, status ):
        self.log( "self-status-changed " + status )

    def cbSelfPersonalMessageChanged( self, msn, ourmail, pm ):
        self.log( "self-personal-message-changed " + self.quote( pm ) )

    def cbSelfCurrentMediaChanged( self, msn, ourmail, pm ):
        self.log( "self-current-media-chaged " + self.quote( pm ) )

    def cbNewMailNotification( self, msn, From, FromAddr, Subject, MessageURL , PostURL, id ):
        self.log( "new-mail-notification " + FromAddr + " " + self.quote( Subject ) )

    def cbNickChanged( self, msn, email, nick ):
        self.log( "nick-changed " + email + " " + self.quote( nick ) )

    def cbContactStatusChange( self, msn, email, status ):
        self.log( "contact-status-change " + email + " " + status )

    def cbUserOnline( self, msn, email, oldStatus):
        self.log( "user-online " + email + " " + oldStatus )

    def cbUserOffline( self, msn, email ):
        self.log( "user-offline " + email )

